/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.file;

/**
 * Class .
 * <br><b>Summary:</b><br>
 * Represent a Subtitle.
 */
public class Subtitle implements Comparable<Subtitle>, Cloneable{
    /**The index of the subtitle*/
    private int number;

    /**Its start date (in milliseconds)*/
    private int startDate;

    /**Its end date (in milliseconds)*/
    private int endDate;

    /**The subtitle to display*/
    private String subtitle;

    /** (<b>int</b>) anchor: The anchor defined for this subtitle, -1 means non anchored.*/
    private int anchor;
    
    /**
     * Constructor Subtitle.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     * @param _number           The index of the subtitle.  
     * @param _startDate        Its start date (in milliseconds)
     * @param _endDate          Its end date (in milliseconds)
     * @param _subtitle         The subtitle to display
     */
    public Subtitle(int _number, int _startDate, int _endDate, String _subtitle) {
        number = _number;
        startDate = _startDate;
        endDate = _endDate;
        subtitle = _subtitle;
        anchor = -1;
    }

    /**
     * Constructor Subtitle.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     * Copy another subtitle.
     * @param _subtitle  The <b>Subtitle</b> to copy.
     */
    public Subtitle(Subtitle _subtitle) {
        this( _subtitle.getNumber(), _subtitle.getStartDate(), _subtitle.getEndDate(), _subtitle.getSubtitle());
    }

    /**
     * Constructor Subtitle.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     */
    public Subtitle() {
    	anchor = -1;
    }

    /**
     * Method delay.
     * <br><b>Summary:</b><br>
     * Apply a delay to the subtitle.
     * @param delay The delay to apply.
     */
    public void delay(int delay) {
        startDate += delay;
        endDate += delay;
    }


    
    /**
	 * Method getEndDate.
	 * <br><b>Summary:</b><br>
	 * Return the endDate.
	 * @return the endDate
	 */
	public int getEndDate() {
		return endDate;
	}

	/**
	 * Method setEndDate.
	 * <br><b>Summary:</b><br>
	 * Set the endDate.
	 * @param endDate the endDate to set
	 */
	public void setEndDate(int endDate) {
		this.endDate = endDate;
	}

	/**
	 * Method getNumber.
	 * <br><b>Summary:</b><br>
	 * Return the number.
	 * @return the number
	 */
	public int getNumber() {
		return number;
	}

	/**
	 * Method setNumber.
	 * <br><b>Summary:</b><br>
	 * Set the number.
	 * @param number the number to set
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * Method getStartDate.
	 * <br><b>Summary:</b><br>
	 * Return the startDate.
	 * @return the startDate
	 */
	public int getStartDate() {
		return startDate;
	}

	/**
	 * Method setStartDate.
	 * <br><b>Summary:</b><br>
	 * Set the startDate.
	 * @param startDate the startDate to set
	 */
	public void setStartDate(int startDate) {
		this.startDate = startDate;
	}

	/**
	 * Method getSubtitle.
	 * <br><b>Summary:</b><br>
	 * Return the subtitle.
	 * @return the subtitle
	 */
	public String getSubtitle() {
		return subtitle;
	}

	/**
	 * Method setSubtitle.
	 * <br><b>Summary:</b><br>
	 * Set the subtitle.
	 * @param subtitle the subtitle to set
	 */
	public void setSubtitle(String subtitle) {
		this.subtitle = subtitle;
	}

	/* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object object){
        //the result of the method
        boolean result = false;
        //We can only compare Subtitle Objects.
        if(object instanceof Subtitle){
            Subtitle subtitle = (Subtitle) object;
            result = true;
            //check all parameters.
            //If one comparison fails, global result fails.
            //compare number.
            result = result && (getNumber() == subtitle.getNumber());
            //start date
            result = result && (getStartDate() == subtitle.getStartDate());
            //end date
            result = result && (getEndDate() == subtitle.getEndDate());
            //Subtitle
            result = result && 
                        (
                                (getSubtitle() == null && subtitle.getSubtitle() == null)
                                ||
                                (getSubtitle().equals(subtitle.getSubtitle()))
                        );
        }
        //return the result
        return result;
    }

    /**
     * Method accentRemove.
     * <br><b>Summary:</b><br>
     * This method remove the accents and other bad characters from subtitle.
     */
    public void accentRemove() {
        //Just have to remove accent chars, and replace them by their non accent equivalent
    	subtitle = subtitle.replaceAll("[ÀÁÂÃÄÅÆĀĂĄ]","A");
    	subtitle = subtitle.replaceAll("[àáâãäåæāăą]","a");   
        
    	subtitle = subtitle.replaceAll("[èéêëēĕėęě]","e");
    	subtitle = subtitle.replaceAll("[ÈÉÊËĒĔĖĘĚ]","E");
        
        subtitle = subtitle.replaceAll("[ìíîï]","i");
        subtitle = subtitle.replaceAll("[ÌÍÎÏ]","I");
        
        subtitle = subtitle.replaceAll("[ñ]","n");
        subtitle = subtitle.replaceAll("[Ñ]","N");
        
        subtitle = subtitle.replaceAll("[çćĉċč]","c");
        subtitle = subtitle.replaceAll("[ÇĆĈĊČ]","C");
        
        subtitle = subtitle.replaceAll("[ðòóôõöø]","o");
        subtitle = subtitle.replaceAll("[ÐÒÓÔÕÖ]","O");

        subtitle = subtitle.replaceAll("[ùúûü]","u");
        subtitle = subtitle.replaceAll("[ÙÚÛÜ]","U");
        
        subtitle = subtitle.replaceAll("[ýÿ]","y");
        subtitle = subtitle.replaceAll("[ÝŸ]","Y");
        
        subtitle = subtitle.replaceAll("[ďđ]","d");
        subtitle = subtitle.replaceAll("[ĎĐ]","D");
    }

    /**
     * Method htmlRemove.
     * <br><b>Summary:</b><br>
     * This method remove the Html tags from subtitle.
     */
    public void htmlRemove() {
        //remove the html tags.
        int first = subtitle.indexOf("<");
        int last = subtitle.indexOf(">", first);
        //while there is a html tag to remove, remove.
        while (first != -1 && last != -1){
            //get subtitle before tag.
            String temp =subtitle.substring(0,first);
            //If tag is not at the end of subtitle.
            if(last < subtitle.length()-1){
                //add subtitle part that is after the html tag.
                temp += subtitle.substring(last+1, subtitle.length());
            }
            //save subtitle.
            subtitle = temp;
            //get next tag index.
            first = subtitle.indexOf("<");
            last = subtitle.indexOf(">", first);
        }
    }

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Subtitle subtitle) {
		//		The result of the method.
		int result = 0;

		//Compare two subtitle, means compare the start dates.
		int start = subtitle.getStartDate();
		if (start > startDate) {
			result = -1;
		} else if (start < startDate) {
			result = 1;
		}

		//return the result.
		return result;
	}

	/**
	 * Method isAnchored.
	 * <br><b>Summary:</b><br>
	 * Return true, if the subtitle is anchored to a destination time.
	 * @return  (<b>boolean</b>)   true, if the subtitle is anchored to a destination time.
	 */
	public boolean isAnchored() {
		return anchor != -1;
	}
	
	/**
	 * Method anchor.
	 * <br><b>Summary:</b><br>
	 * Permits to set the anchor of the subtitle.
	 * @param anchor		The anchor to set (in seconds).
	 */
	public void anchor(int anchor){
		this.anchor = anchor;
	}

	/**
	 * Method unanchor.
	 * <br><b>Summary:</b><br>
	 * remove the anchor from a subtitle.
	 */
	public void unanchor() {
		anchor = -1;
	}

	/**
	 * Method getAnchor.
	 * <br><b>Summary:</b><br>
	 * Return the anchor in milliseconds.
	 * @return the anchor
	 */
	public int getAnchor() {
		return anchor*1000;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Subtitle cloneSubtitle(){
		return new Subtitle(number, startDate, endDate, new String(subtitle));
	}
	
}
