package sears.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.JButton;

import sears.gui.resources.SearsResources;

/**
 * An arrow button is a JButton with an arrow set to its icon.
 * <br>When user press down the button, the arrow flip to down and stay in this state.
 * <br>Press again and it returns to its normal state.
 * @see JButton
 */
public class ArrowButton extends JButton {

	private static final long serialVersionUID = -379552681655716687L;
	
	// Arrow icons:
	private static final Icon ARROW_UP = SearsResources.getIcon("ArrowUp");
	private static final Icon ARROW_DOWN = SearsResources.getIcon("ArrowDown");
	// to know the arrow state
	private boolean isTurnOn;
	
	/**
	 * Constructs a new <tt>ArrowButton</tt> object
	 * @param text	the text to display on the button (if null, no text will appear)
	 * @see JButton
	 */
	public ArrowButton(String text) {
		super(text);
		createArrowButton(); 
	}
	
	// set the immutable constraints
	private void createArrowButton() {
		isTurnOn = false;
		setIcon(ARROW_UP);
		
		setBorderPainted(false);	
		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if( !isTurnOn ) {
					turnOn();
				} else {
					turnOff();
				}
			}			
		});
	}
	
	/**
	 * To know the arrow state
	 * @return true if arrow is down, false if up, the normal state is sets to up
	 */
	public boolean isTurnOn() {
		return isTurnOn;
	}
	
	/**
	 * Turns on the arrow
	 */
	protected void turnOn() {
		if( !isTurnOn ) {
			isTurnOn = true;
			setIcon(ARROW_DOWN);			
		}
	}
	
	/**
	 * Turns off the arrow
	 */
	protected void turnOff() {
		if( isTurnOn ) {
			isTurnOn = false;
			setIcon(ARROW_UP);			
		}
	}
}