/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Class DelayDialog.
 * <br><b>Summary:</b><br>
 * This class is a Dialog which permits to delay some subtitles.
 */
public class DelayDialog extends SearsJDialog {

    private static final long serialVersionUID = -365612092452966123L;

    private JPanel mainPanel = null;

    private JPanel delayPanel = null;

    private JLabel labelDelay = null;

    private JTextField textDelay = null;

    private JSlider jSliderDelay;

    /**
     * This is the default constructor
     */
    public DelayDialog() {
        super(SearsResourceBundle.getResource("delay_title"));
        this.setContentPane(getMainPanel());
        configureSize();
    }
     
    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getMainPanel() {
        if (mainPanel == null) {
            mainPanel = new JPanel();
            mainPanel.setLayout(new BorderLayout());
            mainPanel.setBorder(super.createEmptyBorder());
            mainPanel.add(getDelayPanel(), java.awt.BorderLayout.CENTER);
            mainPanel.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
        }
        return mainPanel;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getDelayPanel() {
        if (delayPanel == null) {
            labelDelay = new JLabel();
            labelDelay.setText(SearsResourceBundle.getResource("delay_label"));
            delayPanel = new JPanel();
            delayPanel.setLayout(new GridBagLayout());
            GridBagConstraints gbc1 = new GridBagConstraints();
            gbc1.gridx = 0;
            gbc1.gridy = 0;
            gbc1.ipadx = 4;
            delayPanel.add(labelDelay, gbc1);
            GridBagConstraints gbc2 = new GridBagConstraints();
            gbc2.gridx = 1;
            gbc2.gridy = 0;
            gbc2.fill = GridBagConstraints.HORIZONTAL;
            gbc2.weightx = 1;
            delayPanel.add(getTextDelay(), gbc2);
            GridBagConstraints gbc3 = new GridBagConstraints();
            gbc3.gridx = 0;
            gbc3.gridy = 1;
            gbc3.gridwidth = 2;
            gbc3.fill = GridBagConstraints.HORIZONTAL;
            gbc3.weightx = 1;
            delayPanel.add(getJSliderDelay(), gbc3);
        }
        return delayPanel;
    }

    /**
     * Method getJSpinnerDelay.
     * <br><b>Summary:</b><br>
     * This method creates the JSpinner to control the delay
     * @return  JSpinner.
     */
    private JSlider getJSliderDelay() {
        if(jSliderDelay == null){
            jSliderDelay = new JSlider(-100, 100);
            jSliderDelay.setValue(10);
            jSliderDelay.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    valueChanged(e.getSource());
                }
            });
        }
        return jSliderDelay;
    }

    /**
     * Method valueChanged.
     * <br><b>Summary:</b><br>
     * This method is called when textField changed, or when JSpinner changed.
     * @param source    The one which change the first.
     */
    protected void valueChanged(Object source) {
        //If the slider triggered the change, update the textField (if necessary).
        if (source.equals(jSliderDelay)) {
            //by default we will not update the textField value.
            boolean updateTextField = false;
            //retrieve the slider delay value
            double sliderDelayValue = ((double) jSliderDelay.getValue()) / 10;
            updateTextField = true;
            try {
                //  retrieve the TextField value.
                double delay = getDoubleTextDelayValue();
                if (delay != sliderDelayValue) {
                    updateTextField = true;
                }
            } catch (NumberFormatException e) {
                //TextField value is not valid : update !
                updateTextField = true;
            }
            //update textField if necessary
            if (updateTextField) {
                getTextDelay().setText("" + sliderDelayValue);
            }
        } else if (source.equals(textDelay)) {
            //If the textDelay triggered the event, update the slider if necessary.
            //Source is the TextDelay
            try {
                //  retrieve the TextField value.
                double delay = getDoubleTextDelayValue();
                double sliderDelayValue = ((double) jSliderDelay.getValue()) / 10;
                if (delay != sliderDelayValue) {
                    // Udate slider.
                    //Beware if textField value is bigger than slider max
                    int newSliderValue =  (int) (delay * 10);
                    if(Math.abs(newSliderValue) > jSliderDelay.getMaximum()){
                        jSliderDelay.setMaximum(Math.abs(newSliderValue));
                        jSliderDelay.setMinimum(-Math.abs(newSliderValue));
                    }
                    jSliderDelay.setValue(newSliderValue);
                }
            } catch (NumberFormatException e) {
                //TextField value is not valid : do not update slider!
            }
        }
    }

    
    /**
     * Method getDoubleTextDelayValue.
     * <br><b>Summary:</b><br>
     * This method returns the double value of the delay in the textfield.
     * @return double   The double value of the delay in the textfield.
     * @throws NumberFormatException
     */
    public double getDoubleTextDelayValue() throws NumberFormatException{
        //the result of the method.
        double result = 0;
        String delayString = getTextDelay().getText();
        if (delayString != null && !delayString.equals("")) {
                result = Double.parseDouble(delayString);
        }else{
            result = 0;
        }
        //return the result;
        return result;
    }
    /**
     * Method okAction.
     * <br><b>Summary:</b><br>
     * This method is called when user validate the dialog.
     */
    protected void okAction() {
        //Just have to check parameters validity, if valids, dispose dialog,and set validation status to true.
        String error = checkParameters();
        if (error != null && !error.equals("")) {
            //Show error message.
            JOptionPane.showMessageDialog(this, error, SearsResourceBundle.getResource("error_delayConfigurationError"), JOptionPane.ERROR_MESSAGE);
            Trace.trace("Error in Delay parameters:" + error, Trace.WARNING_PRIORITY);
        } else {
            //else split configuration is valid, release dialog.
            validationStatus = true;
            dispose();
        }
    }

    /**
     * Method checkParameters.
     * <br><b>Summary:</b><br>
     * This method checks the parameters.
     * @return  <b>String</b>      "" if there is no error, or the error message.
     */
    private String checkParameters() {
        //Check file to open
        String errorMessage = "";
        //check delay.
        String delay = getTextDelay().getText();
        if (delay != null && !delay.equals("")) {
            try {
                Double.parseDouble(delay);
            } catch (NumberFormatException e) {
                //Delay value is not a number.
                errorMessage += SearsResourceBundle.getResource("error_delayNotValid")+"\n";
            }
        } else {
            //Delay value is null.
            errorMessage += SearsResourceBundle.getResource("error_delayNull")+"\n";
        }
        return errorMessage;
    }

    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getTextDelay() {
        if (textDelay == null) {
            textDelay = new JTextField(5);
            textDelay.setToolTipText(SearsResourceBundle.getResource("delay_tip"));
            textDelay.setText("1");
            //add an action listener to validate when user press 'enter' in textField.
            textDelay.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    okAction();
                }
            });
            //Add a focus listener, to update slider when textfield lost focus.
            textDelay.addFocusListener(new FocusListener() {
                public void focusLost(FocusEvent e) {
                    valueChanged(e.getSource());
                }
            
                public void focusGained(FocusEvent e) {
                    //Nothing to do on focus gain
                }
            });
        }
        return textDelay;
    }


    /**
     * Method getDelay.
     * <br><b>Summary:</b><br>
     * Return the entered delay.
     * @return      <b>double</b>      The entered delay.
     */
    public double getDelay() {
        return Double.parseDouble(getTextDelay().getText());
    }

    /* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
    protected String getDialogName() {
        return "delay";
    }
} 
