/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.text.BadLocationException;
import javax.swing.text.rtf.RTFEditorKit;

import sears.Version;
import sears.gui.resources.SearsResources;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Class JDialogAbout.
 * <br><b>Summary:</b><br>
 * Allows to display the about window.
 */
public class JDialogAbout extends SearsJDialog {
	private static final long serialVersionUID = 5648489779666362013L;

	/**The application icon*/
	private ImageIcon searsIcon;
	
	private JPanel jAboutPanel = null;
	
	private JLabel iconField = null;
	
	private JLabel titleLabel = null;	
	
	private JEditorPane aboutRTFViewer = null;

	/**
     * Constructor JDialogAbout.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
	 * @param title the title for this dialog
     */
	public JDialogAbout(String title) {
		super(SearsResourceBundle.getResource("about_title"));
		// we initialize the sears icon:
		searsIcon = SearsResources.getIcon("SearsGUIIcon");
		initialize();

	}

	/**
     * This method initializes this 
     */
	private void initialize(){
		setResizable(false);		
		// we add about panel to the content pane:
		setContentPane(getJAboutPanel());
		// we add components to the about panel:
		jAboutPanel.add(getIconLabel(), BorderLayout.NORTH);
		jAboutPanel.add(getTitleLabel(), BorderLayout.CENTER);
		jAboutPanel.add(getAboutRTFViewer(), BorderLayout.SOUTH);
	}
	
	/**
	 * this method initializes the about label.
	 * @return a JLabel
	 */
	private JPanel getJAboutPanel(){
		if(jAboutPanel == null){
			//we initialize the about panel:
			jAboutPanel = new JPanel(new BorderLayout(3,10));
			// we create border:
			jAboutPanel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		}
		return jAboutPanel;
	}
	
	/**
	 * this method initializes the icon label.
	 * @return a JLabel
	 */
	private JLabel getIconLabel(){
		if(iconField == null){
			iconField = new JLabel(searsIcon);
		}
		return iconField;
	}
	
	/**
	 * this method initializes the rft viewer.
	 * @return a JEditorPane, an RTF Viewer
	 */
	
	private JEditorPane getAboutRTFViewer(){
		// text area for display information about developpers...etc
		// Create an RTF viewer (non editable) window, aboutRTFViewer:
		if(aboutRTFViewer == null){
			// we create a RTF editor an transform it to a simple viewer:
			RTFEditorKit rtf = new RTFEditorKit();
			aboutRTFViewer = new JEditorPane();
			aboutRTFViewer.setEditorKit(rtf);
			aboutRTFViewer.setBackground(Color.white);
			aboutRTFViewer.setEditable(false);
			aboutRTFViewer.setDragEnabled(false);
			aboutRTFViewer.setFocusable(true);
			aboutRTFViewer.setMargin(new Insets(3,5,3,5));
			aboutRTFViewer.setMaximumSize(new Dimension(10,10));
			aboutRTFViewer.setMinimumSize(new Dimension(10,10));
			
			//we get back the rtf file path:
			URL aboutSearsRTFFile = MainWindow.instance.getClass().getResource("/sears/gui/resources/searsAbout.rtf");
			
			try {			
				InputStream fi = aboutSearsRTFFile.openStream();
				rtf.read( fi, aboutRTFViewer.getDocument(), 0 );	
			} catch( FileNotFoundException e ){
				Trace.warning("Can't find resource: " + aboutSearsRTFFile );
				//set text to none
				aboutRTFViewer.setText("");
			} catch (IOException e) {
				// read acces error:
				Trace.warning("Can't open or read: " + aboutSearsRTFFile.getFile());
				// set text to none
				aboutRTFViewer.setText("");
			} catch (BadLocationException e) {				
				// set text to none
				aboutRTFViewer.setText("");
			}		
		}
		
		return aboutRTFViewer;
	}
	
	
	/**
	 * this method return the title label.
	 * @return a JLabel
	 */
	private JLabel getTitleLabel(){
		//title label: Sears name and version:
		if(titleLabel == null){
			titleLabel = new JLabel("Sears " + Version.VERSION, JLabel.CENTER);            	           	
			titleLabel.setFont(new Font("Lucida", Font.PLAIN, 14));
		}
		
		return titleLabel;
	}
	
	
	public void windowClosed(WindowEvent e) {
       // don't save preference size
    }
	
	/* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
	protected String getDialogName() {
		return "about";
	}

}
