/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import sears.gui.resources.SearsResources;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Class JDialogAppend.
 * <br><b>Summary:</b><br>
 * Permit to perform an append action.
 */
public class JDialogAppend extends SearsJDialog {
    private static final long serialVersionUID = -6078008025583338318L;

    private JPanel jContentPane = null;

    private JPanel jPanelCenter = null;

    private JLabel jLabelFile = null;

    private JLabel jLabelDelay = null;

    private JTextField jTextFieldDelay = null;

    private JTextField jTextFieldFileToAppend = null;

    private JButton jButtonBrowse = null;



    /**
     * This is the default constructor
     */
    public JDialogAppend() {
        super(SearsResourceBundle.getResource("append_title"));
        setContentPane(getJContentPane());
        configureSize();
    }
    
    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.setBorder(super.createEmptyBorder());
            jContentPane.add(getJPanelCenter(), java.awt.BorderLayout.CENTER);
            jContentPane.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
        }
        return jContentPane;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelCenter() {
        if (jPanelCenter == null) {
            jPanelCenter = new JPanel();
            jPanelCenter.setLayout(new GridBagLayout());
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 2;
            gridBagConstraints2.gridy = 0;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints1.gridy = 0;
            gridBagConstraints1.weightx = 1.0;
            gridBagConstraints1.gridx = 1;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints.gridy = 0;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.gridy = 1;
            gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.gridx = 1;
            gridBagConstraints4.gridy = 1;
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
            jLabelFile = new JLabel();
            jLabelFile.setText(SearsResourceBundle.getResource("append_label"));
            jPanelCenter.add(jLabelFile, gridBagConstraints);
            jPanelCenter.add(getJTextFieldFileToAppend(), gridBagConstraints1);
            jPanelCenter.add(getJButtonBrowse(), gridBagConstraints2);
            jLabelDelay = new JLabel(SearsResourceBundle.getResource("delay_label"));
            jPanelCenter.add(jLabelDelay, gridBagConstraints3);
            jPanelCenter.add(getJTextFieldDelay(), gridBagConstraints4);
        }
        return jPanelCenter;
    }

    /**
     * Method getJTextFieldDelay.
     * <br><b>Summary:</b><br>
     * return the JTextField that contains the delay.
     */
    private JTextField getJTextFieldDelay() {
        if (jTextFieldDelay == null) {
            jTextFieldDelay = new JTextField(4);
            jTextFieldDelay.setToolTipText(SearsResourceBundle.getResource("delay_tip"));
            jTextFieldDelay.setText("1");
        }
        return jTextFieldDelay;
    }





    /**
     * Method okAction.
     * <br><b>Summary:</b><br>
     * This method is called when user validate the dialog.
     */
    protected void okAction() {
        //Just have to check parameters validity, if valids, dispose dialog,and set validation status to true.
        String error = checkParameters();
        if (error != null && !error.equals("")) {
            //Show error message.
            JOptionPane.showMessageDialog(this, error, SearsResourceBundle.getResource("error_appendConfigurationError"), JOptionPane.ERROR_MESSAGE);
            Trace.trace("Error in Append parameters:" + error, Trace.WARNING_PRIORITY);
        } else {
            //else split configuration is valid, release dialog.
            validationStatus = true;
            dispose();
        }
    }

    /**
     * Method checkParameters.
     * <br><b>Summary:</b><br>
     * This method checks the parameters.
     * @return  <b>String</b>      "" if there is no error, or the error message.
     */
    private String checkParameters() {
        //Check file to open
        String errorMessage = "";
        String fileName = getJTextFieldFileToAppend().getText();
        if (fileName != null && !fileName.equals("")) {
            File fileToAppend = new File(fileName);
            if (!fileToAppend.exists()) {
                errorMessage += SearsResourceBundle.getResource("error_fileDoesNotExist1")
                                +" " + fileToAppend.getAbsolutePath() 
                                +" " +SearsResourceBundle.getResource("error_fileDoesNotExist2")+"\n";
            }
        } else {
            errorMessage += SearsResourceBundle.getResource("error_fileIsNull")+"\n";
        }
        //check delay.
        String delay = getJTextFieldDelay().getText();
        if (delay != null && !delay.equals("")) {
            try {
                Double.parseDouble(delay);
            } catch (NumberFormatException e) {
                //Delay value is not a number.
                errorMessage += SearsResourceBundle.getResource("error_delayNotValid")+"\n";
            }
        } else {
            //Delay value is null.
            errorMessage += SearsResourceBundle.getResource("error_delayNull")+"\n";
        }
        return errorMessage;
    }


    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getJTextFieldFileToAppend() {
        if (jTextFieldFileToAppend == null) {
            jTextFieldFileToAppend = new JTextField(14);
        }
        return jTextFieldFileToAppend;
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonBrowse() {
        if (jButtonBrowse == null) {
            jButtonBrowse = new JButton(SearsResources.getIcon("BrowseIcon"));
            jButtonBrowse.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent e) {
                    browseFileToAppend();
                }
            });
        }
        return jButtonBrowse;
    }

    /**
     * Method getFileToAppend.
     * <br><b>Summary:</b><br>
     * Return the file that has been selected, or null if user canceled.
     * @return File The <b>File</b> to append.
     */
    public File getFileToAppend() {
        //the result of the method.
        File result = null;
        if (validationStatus) {
            result = new File(getJTextFieldFileToAppend().getText());
        }
        //return the result
        return result;
    }

    /**
     * Method browseFileToAppend.
     * <br><b>Summary:</b><br>
     * Use thid method to choose destination files.
     */
    protected void browseFileToAppend() {
        File choosenFile = MainWindow.instance.showSRTBrowser();
        if (choosenFile != null) {
            getJTextFieldFileToAppend().setText(choosenFile.getAbsolutePath());
        }
    }



    /**
     * Method getDelay.
     * <br><b>Summary:</b><br>
     * return the delay to apply before appending.
     * @return  <b>double</b>  The delay to apply before appending.
     */
    public double getDelay() {
        //The result
        double result = 1;
        //Get the entered delay.
        String delay = getJTextFieldDelay().getText();
        if (delay != null && !delay.equals("")) {
            try {
                result = Double.parseDouble(delay);
            } catch (NumberFormatException e) {
                Trace.trace("Delay is not a number.", Trace.ERROR_PRIORITY);
            }
        }
        return result;
    }
    /* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
    protected String getDialogName() {
        return "append";
    }
}
