/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import sears.tools.SearsResourceBundle;

/**
 * Class JDialogChainRepair.
 * <br><b>Summary:</b><br>
 * This class represent the dialog wich permit to apply various repair actions at once. 
 */
public class JDialogChainRepair extends SearsJDialog {
    private static final long serialVersionUID = -917472366618451891L;
    private static final int ACTION_NUMBER = 4;
    private JPanel jContentPane = null;
    private JPanel jPanelCenter = null;
    private JRadioButton jRadioAccentRepair = null;
    private JRadioButton jRadioHtmlRepair = null;
    private JRadioButton jRadioOrderRepair = null;
    private JRadioButton jRadioTimeRepair = null;

    private JButton jButtonSelectAll = null;
    private JButton jButtonApply = null;


    /**
     * This is the default constructor
     */
    public JDialogChainRepair() {
        super(SearsResourceBundle.getResource("chainRepair_title"));
        setContentPane(getJContentPane());
        configureSize();
    }
    
    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.setBorder(super.createEmptyBorder());
            jContentPane.add(getJPanelCenter(), java.awt.BorderLayout.CENTER);
            jContentPane.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
        }
        return jContentPane;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelCenter() {
        if (jPanelCenter == null) {
            jPanelCenter = new JPanel();
            jPanelCenter.setLayout(new GridLayout(2,2));
            jPanelCenter.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), SearsResourceBundle.getResource("chainRepair_borderTitle")));
            jPanelCenter.add(getJRadioAccentRepair(), null);
            jPanelCenter.add(getJRadioHtmlRepair(), null);
            jPanelCenter.add(getJRadioOrderRepair(), null);
            jPanelCenter.add(getJRadioTimeRepair(), null);
        }
        return jPanelCenter;
    }

    /**
     * This method initializes jRadioButton	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioAccentRepair() {
        if (jRadioAccentRepair == null) {
            jRadioAccentRepair = new JRadioButton();
            jRadioAccentRepair.setText(SearsResourceBundle.getResource("AccentRepairName"));
            jRadioAccentRepair.setToolTipText(SearsResourceBundle.getResource("AccentRepairTip"));
        }
        return jRadioAccentRepair;
    }

    /**
     * This method initializes jRadioButton	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioHtmlRepair() {
        if (jRadioHtmlRepair == null) {
            jRadioHtmlRepair = new JRadioButton();
            jRadioHtmlRepair.setText(SearsResourceBundle.getResource("HtmlRepairName"));
            jRadioHtmlRepair.setToolTipText(SearsResourceBundle.getResource("HtmlRepairTip"));
        }
        return jRadioHtmlRepair;
    }

    /**
     * This method initializes jRadioButton	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioOrderRepair() {
        if (jRadioOrderRepair == null) {
            jRadioOrderRepair = new JRadioButton();
            jRadioOrderRepair.setText(SearsResourceBundle.getResource("OrderRepairName"));
            jRadioOrderRepair.setToolTipText(SearsResourceBundle.getResource("OrderRepairTip"));
        }
        return jRadioOrderRepair;
    }

    /**
     * This method initializes jRadioButton	
     * 	
     * @return javax.swing.JRadioButton	
     */
    private JRadioButton getJRadioTimeRepair() {
        if (jRadioTimeRepair == null) {
            jRadioTimeRepair = new JRadioButton();
            jRadioTimeRepair.setText(SearsResourceBundle.getResource("TimeRepairName"));
            jRadioTimeRepair.setToolTipText(SearsResourceBundle.getResource("TimeRepairTip"));
        }
        return jRadioTimeRepair;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    protected JPanel getJPanelButtons() {
        if (jPanelButtons == null) {
            jPanelButtons = new JPanel();
            jPanelButtons.add(getJButtonSelectAll(), null);
            jPanelButtons.add(getJButtonApply(), null);
            jPanelButtons.add(getJButtonCancel(), null);
        }
        return jPanelButtons;
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonSelectAll() {
        if (jButtonSelectAll == null) {
            jButtonSelectAll = new JButton();
            jButtonSelectAll.setText(SearsResourceBundle.getResource("button_selectAll"));
            jButtonSelectAll.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    selectAllAction();
                }
            });
        }
        return jButtonSelectAll;
    }

    /**
     * Method selectAllAction.
     * <br><b>Summary:</b><br>
     * Select all actions.
     */
    protected void selectAllAction() {
        //Just select the radioButtons.
        boolean[] selection = new boolean[ACTION_NUMBER];
        for (int i = 0; i < selection.length; i++) {
            selection[i] = true;
        }
        selectActions(selection);
    }

    /**
     * Method selectActions.
     * <br><b>Summary:</b><br>
     * This method select the action, using the given boolean array.
     * @param selection     A boolean array which represent: [Accent, Html, Order, Time] 
     */
    private void selectActions(boolean[] selection) {
        getJRadioAccentRepair().setSelected(selection[0]);
        getJRadioHtmlRepair().setSelected(selection[1]);
        getJRadioOrderRepair().setSelected(selection[2]);
        getJRadioTimeRepair().setSelected(selection[3]);
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonApply() {
        if (jButtonApply == null) {
            jButtonApply = new JButton();
            jButtonApply.setText(SearsResourceBundle.getResource("button_apply"));
            jButtonApply.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    okAction();
                }
            });
        }
        return jButtonApply;
    }


    /**
     * Method getResult.
     * <br><b>Summary:</b><br>
     * This method return the repair actions array that has been choosen
     *  [Accent, Html, Order, Time] 
     * @return <b>boolean[]</b>     The repair actions array that has been choosen [Accent, Html, Order, Time] 
     */
    public boolean[] getResult() {
        //The result
        boolean[] result = new boolean[ACTION_NUMBER];
        //set false by default.
        for (int i = 0; i < result.length; i++) {
            result[i] = false;
        }
        //check if user validates result.
        if(validationStatus){
            result[0] = getJRadioAccentRepair().isSelected();
            result[1] = getJRadioHtmlRepair().isSelected();
            result[2] = getJRadioOrderRepair().isSelected();
            result[3] = getJRadioTimeRepair().isSelected();
        }
        return result;
    }
    
    /* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
    protected String getDialogName() {
        return "chainRepair";
    }
}
