/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com


package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.io.File;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import sears.file.FileSystemAccess;
import sears.file.Subtitle;
import sears.gui.resources.SearsResources;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Dialog to mix two subtitles.
 * <br>Mix a subtitle with another means that it takes one of the part of the other:
 * <br>subtitles or times instead of its part.
 *
 */
public class JDialogMixSubtitleFile extends SearsJDialog {

	private static final long serialVersionUID = 6258795853711630067L;

	// we get back all the text we need on the first time the class is used:
	private static String titleText = SearsResourceBundle.getResource("mix_title");
	private static String selectSubtitleText = SearsResourceBundle.getResource("mix_selectSubtitle");
	private static String optionsText = SearsResourceBundle.getResource("mix_options");
	private static String preserveText = SearsResourceBundle.getResource("mix_preserve");
	private static String subtitlesElementsText = SearsResourceBundle.getResource("mix_subtilesElements");
	private static String timesElementsText = SearsResourceBundle.getResource("mix_timesElement");

	// for checking the validity of the subtitle to mix with
	private int numberOfSubtitles;


	private JPanel jContentPane = null;

	private JTextField jTextFieldFileToMix = null;
	// to know what part of subtitle user want to keep:
	private JComboBox jComboBoxPartToMix = null;

	private JButton jButtonBrowse = null;

	private JPanel jPanelMain = null;
	private JPanel jPanelChoosingFile;
	private JPanel jPanelOptions;

	/**
	 * Constructs a new object
	 * @param numberOfSubtitles count of subtitles
	 */
	public JDialogMixSubtitleFile(int numberOfSubtitles) {
		super(titleText);
		this.numberOfSubtitles = numberOfSubtitles;
		setContentPane(getJContentPane());
		configureSize();
	}	

	private JPanel getJContentPane() {
		if (jContentPane  == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.setBorder(super.createEmptyBorder());
			jContentPane.add(getJPanelMain(), BorderLayout.CENTER);
			jContentPane.add(getJPanelButtons(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * construct if needed and return JPanel instance for choosing a file to mix
	 * @return JPanel instance
	 */
	private JPanel getJPanelChoosingFile() {
		if(jPanelChoosingFile == null) {
			jPanelChoosingFile = new JPanel();
			jPanelChoosingFile.setBorder(BorderFactory.createTitledBorder(selectSubtitleText));
			jPanelChoosingFile.setLayout(new GridBagLayout());

			// adding components:
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridx = 0;
			c.gridy = 0;			
			jPanelChoosingFile.add(getJTextFieldFileToMix());

			c.gridx = 1;
			c.gridy = 0;
			jPanelChoosingFile.add(getJButtonBrowse());
		}

		return jPanelChoosingFile;
	}

	/**
	 * construct if needed and return JPanel instance for mix options
	 * @return JPanel instance
	 */
	private JPanel getJPanelOptions() {
		if(jPanelOptions == null) {
			jPanelOptions = new JPanel();
			jPanelOptions.setBorder(BorderFactory.createTitledBorder(optionsText));
			// adding components:
			jPanelOptions.add(new JLabel(preserveText + ": "));
			jPanelOptions.add(getJComboBoxPartToMix());
		}

		return jPanelOptions;
	}



	private JPanel getJPanelMain() {
		if(jPanelMain  == null) {
			jPanelMain = new JPanel(new GridLayout(2,1));			
			jPanelMain.setBorder(new EmptyBorder(2,2,2,2));
			// adding components:
			jPanelMain.add(getJPanelChoosingFile());
			jPanelMain.add(getJPanelOptions());
		}

		return jPanelMain;
	}

	/**
	 * Use this method to choose destination files.
	 */
	protected void browseFileToMix() {
		File choosenFile = MainWindow.instance.showSRTBrowser();
		if (choosenFile != null) {
			getJTextFieldFileToMix().setText(choosenFile.getAbsolutePath());
		}
	}

	/**
	 * 
	 * @return the JTextField instance for display the other subfile path
	 */
	protected JTextField getJTextFieldFileToMix() {
		if(jTextFieldFileToMix == null) {
			jTextFieldFileToMix = new JTextField(20);
			jTextFieldFileToMix.setEditable(false);
			jTextFieldFileToMix.setDragEnabled(false);
		}
		return jTextFieldFileToMix;
	}

	/**
	 * construct if needed and return the JComboBox instance
	 * <br>that allows user to choose what part of subtitleFile 
	 * <br>he wants to keep
	 */
	private JComboBox getJComboBoxPartToMix() {
		if(jComboBoxPartToMix == null) {
			Object[] listOfChoice = {subtitlesElementsText,timesElementsText};		
			jComboBoxPartToMix = new JComboBox(listOfChoice);
		}	
		return jComboBoxPartToMix;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonBrowse() {
		if (jButtonBrowse  == null) {
			jButtonBrowse = new JButton(SearsResources.getIcon("BrowseIcon"));
			jButtonBrowse.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					browseFileToMix();
				}
			});
		}
		return jButtonBrowse;
	}

	/**
	 * This method is called when user validate the dialog.
	 */
	protected void okAction() {
		//Just have to check parameters validity, if valids, dispose dialog,and set validation status to true.
		String error = checkParameters();
		if (error != null && !error.equals("")) {
			//Show error message.
			JOptionPane.showMessageDialog(this, error, " incorrect number of subtitles ", JOptionPane.ERROR_MESSAGE);
			Trace.trace("Error in mix condition " + error, Trace.WARNING_PRIORITY);
		} else {
			//else mix configuration is valid, release dialog.
			validationStatus = true;
			dispose();
		}
	}

	/**
	 * this method check the needed conditions for a mix action.
	 * @return "" if there is no error, or the error message.
	 */
	private String checkParameters() {
		// Check file to open
		String errorMessage = "";
		String filepath = getJTextFieldFileToMix().getText();
		// if filepath is set:
		if ( filepath != null && !filepath.equals("") ) {
			File fileToMix = new File(filepath);
			// we create SubtitleFile instance:
			ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();        		
			FileSystemAccess.getInstance().openFile(fileToMix, subtitleList, null);      		
			// if fileToMix is not a subtitle file:
			if( subtitleList != null ) {
				// check the number of subtitles:
				if(subtitleList.size() != numberOfSubtitles) {
					errorMessage += "number of subtitle does not correspond !";
				}
			}  	
		} else {
			errorMessage += SearsResourceBundle.getResource("error_fileIsNull") + "\n";
		}

		return errorMessage;
	}

	/**
	 * this methods returns the user's choice
	 * <br> false if not
	 * @return true if user wants to keep subtitles, false if not
	 */
	protected boolean keepSubtitles() {
		// if subtitles item is selected return true, false if not
		return (jComboBoxPartToMix.getSelectedIndex() == 0);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.SearsJDialog#getDialogName()
	 */
	protected String getDialogName() {
		return "Mix";
	}
}