//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sears.tools.SearsResourceBundle;
import sears.tools.Trace;


/**
 * Class JDialogNormalizeAction.
 * This dialog permits to define the parameters of a normalizeDuration action.
 * i.e the min duration and the max duration of the ST.
 */
public class JDialogNormalizeAction extends SearsJDialog {

	/** (<b>long</b>) serialVersionUID: The serialVersionUID */
	private static final long serialVersionUID = -6105540923502362579L;
	private JPanel jContentPane;  
	private JPanel jPanelCenter;
	
	protected static final int MIN = 0;
	
	protected static final int MAX = 1;
	
	private static final int NB_TIMES = 2;
	
	private static final String[] MIN_MAX_SUFFIXES = new String[]{"min", "max"};
	private static final String[] DEFAULT_MIN_MAX =  new String[]{"0.5", "5"};
	
	private JPanel[] jPanelTime = null;
	private JLabel[] jLabelTime = null;
	private JTextField[] jTextFieldTime = null;
	private JSlider[] jSliderTime = null;
	
	/**
     * This is the default constructor
     */
    public JDialogNormalizeAction() {
        super(SearsResourceBundle.getResource("normalizeDuration_title"));
        jPanelTime = new JPanel[NB_TIMES];
        jTextFieldTime = new JTextField[NB_TIMES];
        jSliderTime = new JSlider[NB_TIMES];
        jLabelTime = new JLabel[NB_TIMES];
        setContentPane(getJContentPane());
        configureSize();
    }
	
    /**
     * Method getJContentPane.
     * <br><b>Summary:</b><br>
     * return the contentPane
     * @return  (<b>JPanel</b>)   A JPanel.
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setBorder(super.createEmptyBorder());
            jContentPane.setLayout(new BorderLayout());
            jContentPane.add(getJPanelCenter(), java.awt.BorderLayout.CENTER);
            jContentPane.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
        }
        return jContentPane;
    }
    
    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelCenter() {
        if (jPanelCenter == null) {
            jPanelCenter = new JPanel();
            jPanelCenter.setLayout(new GridLayout(NB_TIMES,1));
            jPanelCenter.add(getJPanelTime(MIN));
            jPanelCenter.add(getJPanelTime(MAX));
        }
        return jPanelCenter;
    }

    /**
     * Method getTimePanel.
     * <br><b>Summary:</b><br>
     * Return a penl to define a time, that use a slider, and a textField.
     * @param minMax	MIN or MAX.
     * @return  (<b>JPanel</b>)   A JPanel.
     */
    private JPanel getJPanelTime(int minMax) {
        if (jPanelTime[minMax] == null) {
            jLabelTime[minMax] = new JLabel();
            jLabelTime[minMax].setText(SearsResourceBundle.getResource("normalizeDuration_"+MIN_MAX_SUFFIXES[minMax]+"_label"));
            jPanelTime[minMax] = new JPanel();
            jPanelTime[minMax].setLayout(new GridBagLayout());
            GridBagConstraints gbc1 = new GridBagConstraints();
            gbc1.gridx = 0;
            gbc1.gridy = 0;
            jPanelTime[minMax].add(jLabelTime[minMax], gbc1);
            GridBagConstraints gbc2 = new GridBagConstraints();
            gbc2.gridx = 1;
            gbc2.gridy = 0;
            gbc2.fill = GridBagConstraints.HORIZONTAL;
            gbc2.weightx = 1;
            jPanelTime[minMax].add(getJTextFieldTime(minMax), gbc2);
            GridBagConstraints gbc3 = new GridBagConstraints();
            gbc3.gridx = 0;
            gbc3.gridy = 1;
            gbc3.gridwidth = 2;
            gbc3.fill = GridBagConstraints.HORIZONTAL;
            gbc3.weightx = 1;
            jPanelTime[minMax].add(getJSliderTime(minMax), gbc3);
        }
        return jPanelTime[minMax];
    }
    
	
	/**
	 * Method getJSliderTime.
	 * <br><b>Summary:</b><br>
	 * return the slider that correspond to the given min or max.
	 * @param minMax	MIN or MAX.
	 * @return  (<b>JSlider</b>)   A JSlider.
	 */
	private JSlider getJSliderTime(int minMax) {
		if(jSliderTime[minMax] == null){
			jSliderTime[minMax] = new JSlider(0, 100);
			jSliderTime[minMax].setValue((int) Double.parseDouble(DEFAULT_MIN_MAX[minMax])*10);
			final int minMaxFinal = minMax;
			jSliderTime[minMax].addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    valueChanged(e.getSource(), minMaxFinal);
                }
            });
        }
        return jSliderTime[minMax];
	}

	/**
	 * Method valueChanged.
	 * <br><b>Summary:</b><br>
	 * this method is called when a slider changed or value.
	 * @param source		The slider that generates the event.
	 * @param minMax		To know if it is min or max that changed.
	 */
	private void valueChanged(Object source, int minMax) {
        //If the slider triggered the change, update the textField (if necessary).
        if (source instanceof JSlider) {
            //by default we will not update the textField value.
            boolean updateTextField = false;
            //retrieve the slider delay value
            double sliderDelayValue = ((double) jSliderTime[minMax].getValue()) / 10;
            updateTextField = true;
            try {
                //  retrieve the TextField value.
                double delay = getDoubleTextDelayValue(minMax);
                if (delay != sliderDelayValue) {
                    updateTextField = true;
                }
            } catch (NumberFormatException e) {
                //TextField value is not valid : update !
                updateTextField = true;
            }
            //update textField if necessary
            if (updateTextField) {
                jTextFieldTime[minMax].setText("" + sliderDelayValue);
            }
        } else if (source instanceof JTextField) {
            //If the textDelay triggered the event, update the slider if necessary.
            //Source is the TextDelay
            try {
                //  retrieve the TextField value.
                double delay = getDoubleTextDelayValue(minMax);
                double sliderDelayValue = ((double) jSliderTime[minMax].getValue()) / 10;
                if (delay != sliderDelayValue) {
                    // Udate slider.
                    //Beware if textField value is bigger than slider max
                    int newSliderValue =  (int) (delay * 10);
                    if(Math.abs(newSliderValue) > jSliderTime[minMax].getMaximum()){
                    	jSliderTime[minMax].setMaximum(Math.abs(newSliderValue));
                    }
                    jSliderTime[minMax].setValue(newSliderValue);
                }
            } catch (NumberFormatException e) {
                //TextField value is not valid : do not update slider!
            }
        }
		
	}
	
    /**
     * Method getDoubleTextDelayValue.
     * <br><b>Summary:</b><br>
     * This method returns the double value of the delay in the textfield for given minMax
     * @param minMax	To select the MIN or the MAX
     * @return double   The double value of the delay in the textfield.
     * @throws NumberFormatException
     */
    public double getDoubleTextDelayValue(int minMax) throws NumberFormatException{
        //the result of the method.
        double result = 0;
        String delayString = jTextFieldTime[minMax].getText();
        if (delayString != null && !delayString.equals("")) {
                result = Double.parseDouble(delayString);
        }else{
            result = 0;
        }
        //return the result;
        return result;
    }
	
	
	/**
	 * Method getJTextFieldTime.
	 * <br><b>Summary:</b><br>
	 * return the textField that correspond to the given min or max.
	 * @param minMax	MIN or MAX
	 * @return  (<b>JTextField</b>)   A JTextField.
	 */
	private JTextField getJTextFieldTime(int minMax) {
		if (jTextFieldTime[minMax] == null) {
			jTextFieldTime[minMax] = new JTextField(5);
			jTextFieldTime[minMax].setToolTipText(SearsResourceBundle.getResource("delay_tip"));
			jTextFieldTime[minMax].setText(DEFAULT_MIN_MAX[minMax]);
            //add an action listener to validate when user press 'enter' in textField.
			jTextFieldTime[minMax].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    okAction();
                }
            });
            //Add a focus listener, to update slider when textfield lost focus.
			final int minMaxFinal = minMax;
			jTextFieldTime[minMax].addFocusListener(new FocusListener() {
                public void focusLost(FocusEvent e) {
                    valueChanged(e.getSource(), minMaxFinal);
                }
            
                public void focusGained(FocusEvent e) {
                    //Nothing to do on focus gain
                }
            });
        }
        return jTextFieldTime[minMax];
	}
	
	 /**
     * Method okAction.
     * <br><b>Summary:</b><br>
     * This method is called when user validate the dialog.
     */
    protected void okAction() {
        //Just have to check parameters validity, if valids, dispose dialog,and set validation status to true.
        String error = checkParameters();
        if (error != null && !error.equals("")) {
            //Show error message.
            JOptionPane.showMessageDialog(this, error, SearsResourceBundle.getResource("error_delayConfigurationError"), JOptionPane.ERROR_MESSAGE);
            Trace.trace("Error in Delay parameters:" + error, Trace.WARNING_PRIORITY);
        } else {
            //else split configuration is valid, release dialog.
            validationStatus = true;
            dispose();
        }
    }
    
    /**
     * Method checkParameters.
     * <br><b>Summary:</b><br>
     * This method checks the parameters.
     * @return  <b>String</b>      "" if there is no error, or the error message.
     */
    private String checkParameters() {
        //Check file to open
        String errorMessage = "";
        //check time for min and max.
        for(int i= 0; i < NB_TIMES;i++ ){
	        String delay = jTextFieldTime[i].getText();
	        if (delay != null && !delay.equals("")) {
	            try {
	                Double.parseDouble(delay);
	            } catch (NumberFormatException e) {
	                //Delay value is not a number.
	                errorMessage += "["+MIN_MAX_SUFFIXES[i]+"]"+SearsResourceBundle.getResource("error_delayNotValid")+"\n";
	            }
	        } else {
	            //Delay value is null.
	            errorMessage += "["+MIN_MAX_SUFFIXES[i]+"]"+SearsResourceBundle.getResource("error_delayNull")+"\n";
	        }
        }
        return errorMessage;
    }

    /**
     * Method getTimes.
     * <br><b>Summary:</b><br>
     * return the times configured by the user.
     * chack that user validates the dialog.
     * @return  (<b>double[]</b>)   The result of the normalize dialog, double[]=[MIN, MAX].
     */
    public double[] getTimes(){
    	//The result of the method.
    	double[] result = new double[NB_TIMES];
    	for(int i= 0; i<NB_TIMES;i++){
    		result[i] = Double.parseDouble(jTextFieldTime[i].getText());
    	}
    	//return the result.
    	return result;
    }
    
	/* (non-Javadoc)
	 * @see sears.gui.SearsJDialog#getDialogName()
	 */
	protected String getDialogName() {
		return "normalizeDuration";
	}

}
