/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com

package sears.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;

import sears.gui.resources.SearsResources;
import sears.tools.SearsProperties;
import sears.tools.SearsResourceBundle;
import sears.tools.Updater;
import sears.tools.Utils;
import sears.tools.player.VLCPlayer;

/**
 * Class JDialogOptions.
 * <br><b>Summary:</b><br>
 * This is the dialog options that permit to change the Sears options.
 */
public class JDialogOptions extends SearsJDialog {
	private static final long serialVersionUID = 1L;

	private JTabbedPane jTabbedPaneOptions = null;

	private JPanel jPanelMain = null;

	private JPanel jPanelPlayer = null;

	private JLabel jLabelLang = null;

	private JComboBox jComboBoxLang = null;

	private JLabel jLabelLookAndFeel = null;

	private JComboBox jComboBoxLookAndFeel = null;

	private JCheckBox jCheckBoxDOSLineSeparator = null;

	private JLabel jLabelPlayer = null;

	private JComboBox jComboBoxPlayer = null;

	private JPanel jTabbedPanePlayerOptions = null;

	private CardLayout jTabbedPanePlayerOptionsCardLayout = null;

	private JPanel jPanelVLCOptions = null;

	private JLabel jLabelVLCPath = null;

	private JTextField jTextFieldVLCPath = null;

	private JButton jButtonVLCBrowse = null;

	private JPanel jPanelOtherPlayerOptions = null;

	private JLabel jLabelOtherPlayerPath = null;

	private JTextField jTextFieldOtherPlayerPath = null;

	private JButton jButtonOtherPlayerBrowse = null;

	private JCheckBox jCheckBoxCheckForUpdates = null;

	private JCheckBox jCheckBoxVlcRestart = null;

	private JComboBox jComboBoxIConSet = null;

	private JLabel jLabelIconSet = null;

	private JButton jButtonReset = null;

	private JPanel jPanelWaitForReset = null;

	private boolean resetFlagIsSetOnInstance = false;

	/**
	 * Constructor JDialogOptions.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 */
	public JDialogOptions() {
		super(SearsResourceBundle.getResource("option_title"));
		initialize();
		configureSize();
	}

	/**
	 * This method initializes this
	 */
	private void initialize() { 
		// if reset operation was not performed before:
		if(!MainWindow.getResetFlag()) {
			getContentPane().add(getJPanelButtons(), BorderLayout.SOUTH);
			getContentPane().add(getJTabbedPaneOptions(), BorderLayout.CENTER);      	
		} else {
			getContentPane().add(getJPanelWaitForReset(), BorderLayout.CENTER);
		}

	}

	/**
	 * This method initializes jPanelWaitForReset
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelWaitForReset() {
		if(jPanelWaitForReset == null) {
			jPanelWaitForReset = new JPanel();

			jPanelWaitForReset.add(new JLabel(SearsResourceBundle.getResource("option_waitingResetOperation_1")));
			jPanelWaitForReset.add(new JLabel(SearsResourceBundle.getResource("option_waitingResetOperation_2")));
		}
		return jPanelWaitForReset;
	}

	/**
	 * This method initializes jTabbedPaneOptions	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getJTabbedPaneOptions() {
		if (jTabbedPaneOptions == null) {
			jTabbedPaneOptions = new JTabbedPane();
			jTabbedPaneOptions.addTab(SearsResourceBundle.getResource("option_mainTab"), null, getJPanelMain(),
					SearsResourceBundle.getResource("option_mainTabTip"));
			jTabbedPaneOptions.addTab(SearsResourceBundle.getResource("option_playerTab"), null, getJPanelPlayer(),
					SearsResourceBundle.getResource("option_playerTabTip"));
		}
		return jTabbedPaneOptions;
	}

	/**
	 * This method initializes jPanelMain	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelMain() {
		if (jPanelMain == null) {
			// constraints for horizontal line separator:
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.gridy = 5;

			// constraints for jButtonReset:
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.anchor = GridBagConstraints.WEST;
			gridBagConstraints8.gridwidth = 1;
			gridBagConstraints8.gridy = 6;


			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.anchor = GridBagConstraints.WEST;
			gridBagConstraints7.gridy = 2;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.fill = GridBagConstraints.BOTH;
			gridBagConstraints6.anchor = GridBagConstraints.WEST;
			gridBagConstraints6.gridwidth = 1;
			gridBagConstraints6.gridy =2;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.gridwidth = 2;
			gridBagConstraints5.gridy =4;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.anchor = GridBagConstraints.WEST;
			gridBagConstraints4.gridwidth = 2;
			gridBagConstraints4.gridy = 3;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.fill = GridBagConstraints.BOTH;
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.weightx = 1.0;
			gridBagConstraints3.gridx = 1;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.anchor = GridBagConstraints.WEST;
			gridBagConstraints2.gridy = 1;
			jLabelLookAndFeel = new JLabel();
			jLabelLookAndFeel.setText(SearsResourceBundle.getResource("option_lookAndFeelLabel"));
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.gridx = 1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.anchor = GridBagConstraints.WEST;
			gridBagConstraints.gridy = 0;
			jLabelLang = new JLabel();
			jLabelLang.setText(SearsResourceBundle.getResource("option_languageLabel"));
			jPanelMain = new JPanel();
			jPanelMain.setLayout(new GridBagLayout());
			jPanelMain.add(jLabelLang, gridBagConstraints);
			jPanelMain.add(getJComboBoxLang(), gridBagConstraints1);
			jPanelMain.add(jLabelLookAndFeel, gridBagConstraints2);
			jPanelMain.add(getJComboBoxLookAndFeel(), gridBagConstraints3);
			jPanelMain.add(getJLabelIconSet(), gridBagConstraints7);
			jPanelMain.add(getJComboBoxIconSet(), gridBagConstraints6);
			jPanelMain.add(getJCheckBoxDOSLineSeparator(), gridBagConstraints4);
			jPanelMain.add(getJCheckBoxCheckForUpdates(), gridBagConstraints5);
			// horizontal separator line:
			jPanelMain.add(new JSeparator(JSeparator.HORIZONTAL), gridBagConstraints9);
			jPanelMain.add(getJButtonReset(), gridBagConstraints8);
		}
		return jPanelMain;
	}

	/**
	 * Method getJComboBoxIconSet.
	 * <br><b>Summary:</b><br>
	 * @TODO complete summary
	 * @return  (<b>JComboBox</b>)   A JComboBox.
	 */
	private JComboBox getJComboBoxIconSet() {
		if(jComboBoxIConSet == null){
			//Construct the comboBox.
			jComboBoxIConSet = new JComboBox();
			//To construct the combobox of the available icon set, we need to know where is the icons folder.
			String iconFolder = SearsResources.getIconFolder();
			if(iconFolder != null){
				//Now that we have icon folder, parse the folder to retrieve all jar files in it.
				//which are the icon sets.
				File iconFolderFile = new File(iconFolder);
				if( iconFolderFile.exists() && iconFolderFile.isDirectory()){
					String[] iconSetsArray = iconFolderFile.list(new FilenameFilter() {
						public boolean accept(File dir, String name) {
							return name.endsWith(".jar");
						}
					});
					//parse them, and add them to ComboBox
					for (int i = 0; i < iconSetsArray.length; i++) {
						String currentIconSet = iconSetsArray[i];
						jComboBoxIConSet.addItem(currentIconSet);
					}
				}
				//Set the current one as selected item.
				jComboBoxIConSet.setSelectedItem(SearsProperties.getProperty(SearsProperties.ICON_SET, SearsProperties.DEFAULT_ICON_SET));
			}
		}
		return jComboBoxIConSet;
	}

	/**
	 * Method getJLabelIconSet.
	 * <br><b>Summary:</b><br>
	 * This method returns the JLabel used for the iconSets.
	 * @return  (<b>JLabel</b>)   the JLabel used for the iconSets.
	 */
	private JLabel getJLabelIconSet() {
		if(jLabelIconSet == null){
			jLabelIconSet = new JLabel(SearsResourceBundle.getResource("option_iconSetLabel"));
			jLabelIconSet.setToolTipText(SearsResourceBundle.getResource("option_iconSetLabelTip"));
		}
		return jLabelIconSet;
	}

	/**
	 * Method getJCheckBoxCheckForUpdates.
	 * <br><b>Summary:</b><br>
	 * Construct the JCheckBox to choose wether look for updates or not.
	 * @return  JCheckBox   The JCheckBox of check for update.
	 */
	private JCheckBox getJCheckBoxCheckForUpdates() {
		if (jCheckBoxCheckForUpdates  == null) {
			jCheckBoxCheckForUpdates = new JCheckBox();
			jCheckBoxCheckForUpdates.setText(SearsResourceBundle.getResource("option_checkForUpdates"));
			jCheckBoxCheckForUpdates.setToolTipText(SearsResourceBundle.getResource("option_checkForUpdatesTip"));
			//get the previous check box state.
			String previousCheckForUpdates = SearsProperties.getProperty(SearsProperties.UPDATE_ADDRESS, Updater.DEFAULT_UPDATE_ADDRESS);
			//check the box only if property is not ""
			jCheckBoxCheckForUpdates.setSelected(!previousCheckForUpdates.equals(""));
		}
		return jCheckBoxCheckForUpdates;
	}

	/**
	 * Method getJButtonReset.
	 * <br><b>Summary:</b><br>
	 * Conntruct the JButton to perform a reset properties operation.
	 * @return  JButton   The JButton of reset operation.
	 */
	private JButton getJButtonReset() {
		if(jButtonReset == null) {
			jButtonReset = new JButton("Reset");
			jButtonReset.setForeground(Color.RED);
			jButtonReset.setToolTipText(SearsResourceBundle.getResource("option_resetOperationTip"));
			jButtonReset.addActionListener(new ActionListener() {    			

				public void actionPerformed(ActionEvent arg0) {
					resetAction();					
				}
			});    		
		}

		if(MainWindow.getResetFlag()) {
			jButtonReset.setEnabled(false);
			jButtonReset.updateUI();
		}

		return jButtonReset;
	}

	/**
	 * This method initializes jPanelPlayer	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelPlayer() {
		if (jPanelPlayer == null) {
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			gridBagConstraints9.gridy = 4;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.weighty = 1.0;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.gridheight = 4;
			gridBagConstraints9.gridx = 0;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.gridy = 1;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.BOTH;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.gridx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.fill = GridBagConstraints.BOTH;
			gridBagConstraints6.gridy = 1;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.gridy = 0;
			jLabelPlayer = new JLabel();
			jLabelPlayer.setText(SearsResourceBundle.getResource("option_playerLabel"));
			jLabelPlayer.setToolTipText(SearsResourceBundle.getResource("option_playerLabelTip"));
			jPanelPlayer = new JPanel();
			jPanelPlayer.setLayout(new GridBagLayout());
			jPanelPlayer.add(jLabelPlayer, gridBagConstraints5);
			//Call this method before constructing comboboxPlayer , in order to avoid NPE.
			getJTabbedPanePlayerOptions();
			jPanelPlayer.add(getJComboBoxPlayer(), gridBagConstraints7);
			jPanelPlayer.add(getJTabbedPanePlayerOptions(), gridBagConstraints9);
		}
		return jPanelPlayer;
	}

	/**
	 * This method initializes jComboBoxLang	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxLang() {
		if (jComboBoxLang == null) {
			jComboBoxLang = new JComboBox(SearsResourceBundle.getAllAvailableLanguage());
			jComboBoxLang.setRenderer(new LangComboBoxRenderer());
			//get the previous selected locale.
			String previousLocale = SearsProperties.getProperty(SearsProperties.LANGUAGE, "en")+"_"+SearsProperties.getProperty(SearsProperties.COUNTRY, "US");
			jComboBoxLang.setSelectedItem(previousLocale);
		}
		return jComboBoxLang;
	}

	/**
	 * This method initializes jComboBoxLookAndFeel	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxLookAndFeel() {
		if (jComboBoxLookAndFeel == null) {
			String[] lookAndFeels = new String[] { "com.sun.java.swing.plaf.gtk.GTKLookAndFeel",
					UIManager.getCrossPlatformLookAndFeelClassName(),
					"com.sun.java.swing.plaf.motif.MotifLookAndFeel", UIManager.getSystemLookAndFeelClassName() };
			jComboBoxLookAndFeel = new JComboBox(lookAndFeels);
			//Get the previous selected look and feel.
			String previousLookAndFeel = SearsProperties.getProperty(SearsProperties.LOOK_AND_FEEL, UIManager
					.getSystemLookAndFeelClassName());
			//set the comboBox with this look and feel.
			jComboBoxLookAndFeel.setSelectedItem(previousLookAndFeel);
		}
		return jComboBoxLookAndFeel;
	}

	/**
	 * This method initializes jCheckBoxDOSLineSeparator	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxDOSLineSeparator() {
		if (jCheckBoxDOSLineSeparator == null) {
			jCheckBoxDOSLineSeparator = new JCheckBox();
			jCheckBoxDOSLineSeparator.setText(SearsResourceBundle.getResource("option_useDOSLineSeparator"));
			jCheckBoxDOSLineSeparator.setToolTipText(SearsResourceBundle.getResource("option_useDOSLineSeparatorTip"));
			//get the previous check box state.
			String previousDOSLineSeparator = SearsProperties.getProperty(SearsProperties.DOS_LINE_SEPARATOR, "0");
			//check the box only if property is not "0"
			jCheckBoxDOSLineSeparator.setSelected(!previousDOSLineSeparator.equals("0"));
		}
		return jCheckBoxDOSLineSeparator;
	}

	/**
	 * This method initializes jComboBoxPlayer	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxPlayer() {
		if (jComboBoxPlayer == null) {
			//construct teh comboBox that permit player's choice.
			String[] players = new String[2];
			players[SearsProperties.KEY_VLC] = SearsResourceBundle.getResource("option_vlcChoice");
			players[SearsProperties.KEY_OTHER] = SearsResourceBundle.getResource("option_otherPlayerChoice");
			jComboBoxPlayer = new JComboBox(players);
			//When user choose a player, switch panels.
			jComboBoxPlayer.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					playerChoosed();
				}
			});
			//get the previous selected property.
			String previousPlayer = SearsProperties.getProperty(SearsProperties.PLAYER_SELECTED, SearsResourceBundle
					.getResource("option_vlcChoice"));
			//select the previous player
			try{
				jComboBoxPlayer.setSelectedIndex(Integer.parseInt(previousPlayer));
			}catch(NumberFormatException e){
				//There is an error in the selected player property.
				//put VLC to correct the pb.
				jComboBoxPlayer.setSelectedIndex(SearsProperties.KEY_VLC);
				SearsProperties.setProperty(SearsProperties.PLAYER_SELECTED, ""+SearsProperties.KEY_VLC);
			}
		}
		return jComboBoxPlayer;
	}

	/**
	 * Method playerChoosed.
	 * <br><b>Summary:</b><br>
	 * This method is called when user choose a player.
	 * It changes the upper panel, using cardPanel show feature.
	 */
	protected void playerChoosed() {
		//retrieve the selected index for the player.
		jTabbedPanePlayerOptionsCardLayout.show(jTabbedPanePlayerOptions, "" + jComboBoxPlayer.getSelectedIndex());
	}

	/**
	 * This method initializes jTabbedPanePlayerOptions	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JPanel getJTabbedPanePlayerOptions() {
		if (jTabbedPanePlayerOptions == null) {
			jTabbedPanePlayerOptionsCardLayout = new CardLayout();
			jTabbedPanePlayerOptions = new JPanel(jTabbedPanePlayerOptionsCardLayout);
			jTabbedPanePlayerOptions.add(getJPanelVLCOptions(), "" + SearsProperties.KEY_VLC);
			jTabbedPanePlayerOptions.add(getJPanelOtherPlayerOptions(), "" + SearsProperties.KEY_OTHER);
		}
		return jTabbedPanePlayerOptions;
	}

	/**
	 * This method initializes jPanelVLCOptions	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelVLCOptions() {
		if (jPanelVLCOptions == null) {
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridx = 0;
			gridBagConstraints13.anchor = GridBagConstraints.WEST;
			gridBagConstraints13.gridy = 3;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridx = 1;
			gridBagConstraints12.anchor = GridBagConstraints.EAST;
			gridBagConstraints12.gridy = 0;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.fill = GridBagConstraints.BOTH;
			gridBagConstraints11.gridy = 1;
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.gridwidth = 2;
			gridBagConstraints11.gridx = 0;
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.anchor = GridBagConstraints.WEST;
			gridBagConstraints10.gridy = 0;
			jLabelVLCPath = new JLabel();
			jLabelVLCPath.setText(SearsResourceBundle.getResource("option_vlcPathLabel"));
			jPanelVLCOptions = new JPanel();
			jPanelVLCOptions.setLayout(new GridBagLayout());
			jPanelVLCOptions.add(jLabelVLCPath, gridBagConstraints10);
			jPanelVLCOptions.add(getJTextFieldVLCPath(), gridBagConstraints11);
			jPanelVLCOptions.add(getJButtonVLCBrowse(), gridBagConstraints12);
			jPanelVLCOptions.add(getJCheckBoxVlcRestart(), gridBagConstraints13);
		}
		return jPanelVLCOptions;
	}

	/**
	 * This method initializes jPanelVLCOptions 
	 *  
	 * @return javax.swing.JPanel   
	 */
	private JPanel getJPanelOtherPlayerOptions() {
		if (jPanelOtherPlayerOptions == null) {
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridx = 1;
			gridBagConstraints12.anchor = GridBagConstraints.EAST;
			gridBagConstraints12.gridy = 0;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.fill = GridBagConstraints.BOTH;
			gridBagConstraints11.gridy = 1;
			gridBagConstraints11.weightx = 1.0;
			gridBagConstraints11.gridwidth = 2;
			gridBagConstraints11.gridx = 0;
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.anchor = GridBagConstraints.WEST;
			gridBagConstraints10.gridy = 0;
			jLabelOtherPlayerPath = new JLabel();
			jLabelOtherPlayerPath.setText(SearsResourceBundle.getResource("option_otherPlayerPathLabel"));
			jLabelOtherPlayerPath.setToolTipText(SearsResourceBundle.getResource("option_otherPlayerPathTip"));
			jPanelOtherPlayerOptions = new JPanel();
			jPanelOtherPlayerOptions.setLayout(new GridBagLayout());
			jPanelOtherPlayerOptions.add(jLabelOtherPlayerPath, gridBagConstraints10);
			jPanelOtherPlayerOptions.add(getJTextFieldOtherPlayerPath(), gridBagConstraints11);
			jPanelOtherPlayerOptions.add(getJButtonOtherPlayerBrowse(), gridBagConstraints12);
		}
		return jPanelOtherPlayerOptions;
	}

	/**
	 * This method initializes jTextFieldOtherPlayerPath    
	 *  
	 * @return javax.swing.JTextField   
	 */
	private JTextField getJTextFieldOtherPlayerPath() {
		if (jTextFieldOtherPlayerPath == null) {
			jTextFieldOtherPlayerPath = new JTextField();
			//get the previous selected PATH
			String previousPath = SearsProperties.getProperty(SearsProperties.PLAYER_FULL_PATH, "/usr/bin/mplayer %f% -sub %s%");
			//set it in the text field.
			jTextFieldOtherPlayerPath.setText(previousPath);
			jTextFieldOtherPlayerPath.setToolTipText(SearsResourceBundle.getResource("option_otherPlayerPathTip"));
		}
		return jTextFieldOtherPlayerPath;
	}

	/**
	 * This method initializes jButtonOtherPlayerBrowse 
	 *  
	 * @return javax.swing.JButton  
	 */
	private JButton getJButtonOtherPlayerBrowse() {
		if (jButtonOtherPlayerBrowse == null) {
			jButtonOtherPlayerBrowse = new JButton(SearsResources.getIcon("BrowseIcon"));
			jButtonOtherPlayerBrowse.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showPlayerBrowser(SearsProperties.KEY_OTHER);
				}
			});
		}
		return jButtonOtherPlayerBrowse;
	}

	/**
	 * This method initializes jTextFieldVLCPath	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldVLCPath() {
		if (jTextFieldVLCPath == null) {
			jTextFieldVLCPath = new JTextField();
			//get the previous selected PATH
			String vlcPath = SearsProperties.getProperty(SearsProperties.PLAYER_FULL_PATH, "");
			//In order to make vlc path easy to enter, at first launch try to fill it in function of the system.
			//At first launch vlc_path is empty
			if(vlcPath == null || vlcPath.equals("")){
				if(Utils.isMacPlatform){
					//vlcPath = VLCPlayer.DEFAULT_VLC_PATH_MAC;
					vlcPath = "";
				}else if (Utils.isLinuxPlatform){
					vlcPath = VLCPlayer.DEFAULT_VLC_PATH_LINUX;
				}else if (Utils.isWindowsPlatform){
					vlcPath = VLCPlayer.DEFAULT_VLC_PATH_WINDOWS;
				}
			}
			//set it in the text field.
			jTextFieldVLCPath.setText(vlcPath);
		}
		return jTextFieldVLCPath;
	}

	/**
	 * This method initializes jButtonVLCBrowse	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonVLCBrowse() {
		if (jButtonVLCBrowse == null) {
			jButtonVLCBrowse = new JButton(SearsResources.getIcon("BrowseIcon"));
			jButtonVLCBrowse.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					showPlayerBrowser(SearsProperties.KEY_VLC);
				}
			});
		}
		return jButtonVLCBrowse;
	}

	/**
	 * Method getJCheckBoxVlcRestart.
	 * <br><b>Summary:</b><br>
	 * Construct the JCheckBox to choose wether the VLC must be restart at each play command.
	 * @return  JCheckBox   The JCheckBox of restart VLC.
	 */
	private JCheckBox getJCheckBoxVlcRestart() {
		if (jCheckBoxVlcRestart  == null) {
			jCheckBoxVlcRestart = new JCheckBox();
			jCheckBoxVlcRestart.setText(SearsResourceBundle.getResource("option_restartVLC"));
			jCheckBoxVlcRestart.setToolTipText(SearsResourceBundle.getResource("option_restartVLCTip"));
			//get the previous check box state.
			String previousVlcRestart = SearsProperties.getProperty(SearsProperties.VLC_RESTART, VLCPlayer.DEFAULT_VLC_RESTART);
			//check the box only if property is not "0"
			jCheckBoxVlcRestart.setSelected(!previousVlcRestart.equals("0"));
		}
		return jCheckBoxVlcRestart;
	}

	/**Method resetAction.
	 * <br><b>Summary:</b><br>
	 * This method set a flag in MainWindow class and update jButtonReset.
	 */
	protected void resetAction() {
		resetFlagIsSetOnInstance = true;    	
		MainWindow.setResetFlag(true);
		// update jButtonReset UI:
		jButtonReset = getJButtonReset();	
	}

	protected void cancelAction() {
		super.cancelAction();
		// if user perform a reset operation in this instance window
		// he has possibility to cancel this action:
		if(resetFlagIsSetOnInstance) {
			MainWindow.setResetFlag(false);
		}

	}

	/**
	 * Method showPlayerBrowser.
	 * <br><b>Summary:</b><br>
	 * This method show up a file chooser, and set the corresponding player path property.
	 * @param playerKey     The key of the player.
	 */
	protected void showPlayerBrowser(int playerKey) {
		//Display a fileChooser.
		File choosenPlayer = MainWindow.instance.showBrowser(null);
		if (choosenPlayer != null) {
			//construct the full path to player executable.
			String fullPath = choosenPlayer.getAbsolutePath();
			//now , set the property in the correct TextField.
			//if the os system is Mac OS:
			if(Utils.isMacPlatform){
				//we try to change fullPath:
				String newFullPath;
				newFullPath = sears.tools.InfoPlist.getExecutablePath(fullPath);
				// if we succeed:
				if(newFullPath != null){
					fullPath = newFullPath;
				}
				//else we don't touch to fullPath, nothing more happens...
			}
			switch (playerKey) {
			case SearsProperties.KEY_VLC:
				jTextFieldVLCPath.setText(fullPath);
				break;
			case SearsProperties.KEY_OTHER:
				jTextFieldOtherPlayerPath.setText(fullPath);
				break;
			}
		}
	}

	/* (non-Javadoc)
	 * @see sears.gui.SearsJDialog#getDialogName()
	 */
	protected String getDialogName() {
		return "options";
	}

	/**
	 * Method getSelectedLanguage.
	 * <br><b>Summary:</b><br>
	 * Return the selected locale.
	 * @return  <b>Locale</b>       The selected Locale.
	 */
	protected Locale getSelectedLocale() {
		//The result of the method.
		Locale result = null;
		//construct the locale.
		String localeChoosen = (String) jComboBoxLang.getSelectedItem();
		String lang = localeChoosen.substring(0, localeChoosen.indexOf("_"));
		String count = localeChoosen.substring(localeChoosen.indexOf("_")+1);
		//set the result with the given language and country.
		result = new Locale(lang, count);
		return result;
	}

	/**
	 * Method getSelectedLookAndFeel.
	 * <br><b>Summary:</b><br>
	 * Return the selected LookAndFeel.
	 * @return  <b>String</b>       The selected LookAndFeel.
	 */
	protected String getSelectedLookAndFeel() {
		return (String) jComboBoxLookAndFeel.getSelectedItem();
	}

	/**
	 * Method getConfiguredPlayerPath.
	 * <br><b>Summary:</b><br>
	 * Return the configured executable.
	 * @return <b>String</b>    The configured executable.
	 */
	protected String getConfiguredPlayerPath() {
		//The result of the method.
		String result = null;
		//switch on the selected player.
		switch (jComboBoxPlayer.getSelectedIndex()) {
		case SearsProperties.KEY_VLC:
			result = jTextFieldVLCPath.getText();
			break;
		case SearsProperties.KEY_OTHER:
			result = jTextFieldOtherPlayerPath.getText();
			break;
		}
		//return the result.
		return result;
	}

	/**
	 * Method getSelectedPlayer.
	 * <br><b>Summary:</b><br>
	 * Return the configured player.
	 * @return <b>int</b>    The configured player.
	 */
	protected int getSelectedPlayer() {
		//return the result
		return jComboBoxPlayer.getSelectedIndex();
	}

	/**
	 * Method getSelectedDOSLineSeparator.
	 * <br><b>Summary:</b><br>
	 * return "0" if DOSLineSeparator has been checked, "1" otherwise.
	 * @return  <b>String</b>   "0" if DOSLineSeparator has been checked, "1" otherwise.
	 */
	public String getSelectedDOSLineSeparator() {
		//The result of the method.
		String result = null;
		if(jCheckBoxDOSLineSeparator.isSelected()){
			result = "1";
		}else{
			result = "0";
		}
		//return the result
		return result;
	}

	/**
	 * Method getSelectedCheckForUpdates.
	 * <br><b>Summary:</b><br>
	 * return true if need to check for updates, false otherwise.
	 * @return  <b>boolean</b>   true if need to check for updates, false otherwise.
	 */
	public boolean getSelectedCheckForUpdates() {
		return getJCheckBoxCheckForUpdates().isSelected();
	}

	/**
	 * Method getSelectedVlcRestart.
	 * <br><b>Summary:</b><br>
	 * return "1" if VlcRestart has been checked, "0" otherwise.
	 * @return  <b>String</b>   "1" if VlcRestart has been checked, "0" otherwise.
	 */
	public String getSelectedVlcRestart() {
		//The result of the method.
		String result = null;
		if(jCheckBoxVlcRestart.isSelected()){
			result = "1";
		}else{
			result = "0";
		}
		//return the result
		return result;
	}

	/**
	 * Method getSelectedIconSet.
	 * <br><b>Summary:</b><br>
	 * return the selected icon set jar file.
	 * @return String the selected icon set jarFile.
	 */
	public String getSelectedIconSet() {
		return (String) jComboBoxIConSet.getSelectedItem();
	}
}

/**
 * This class is used to display a flag icon and a langage's name 
 * in a <tt>JComboBox</tt> object which sets this class instance as its default renderer
 */
class LangComboBoxRenderer extends JLabel implements ListCellRenderer {

	private static final long serialVersionUID = -3189303571995097527L;
	
	/**
	 * Creates a new renderer
	 */
	public LangComboBoxRenderer() {
		setOpaque(true);
		setVerticalAlignment(CENTER);
		setIconTextGap(3);
		setBorder(BorderFactory.createEmptyBorder(1,4,1,1));
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
	 */
	public Component getListCellRendererComponent(
			JList list,
			Object value,
			int index,
			boolean isSelected,
			boolean cellHasFocus) {

		String selectedItem = ((String) value);
		if (isSelected) {
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		} else {
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		// Sets icon and text
		setIcon(SearsResourceBundle.getIconForLocaleString(selectedItem));
		setText(SearsResourceBundle.getStringForLocaleString(selectedItem));
		return this;
	}
}