/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

package sears.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableColumn;

import sears.Version;
import sears.file.FileSystemAccess;
import sears.file.SrtFile;
import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.file.exception.io.FileConversionException;
import sears.gui.resources.SearsResources;
import sears.gui.search.FindBox;
import sears.gui.search.FindDialog;
import sears.gui.undo.SearsUndoAction;
import sears.gui.undo.SearsUndoManager;
import sears.search.core.FindModule;
import sears.tools.DialogUtils;
import sears.tools.SearsAction;
import sears.tools.SearsProperties;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;
import sears.tools.Updater;
import sears.tools.Utils;
import sears.tools.player.DefaultPlayer;
import sears.tools.player.PlayerException;
import sears.tools.player.PlayerInterface;
import sears.tools.player.VLCPlayer;

/**
 * Class MainWindow.
 * <br><b>Summary:</b><br>
 * This is the main Interface of the sears project.
 * It contains a JTable to displays the subtitles, and menus to interact and launch actions.
 */
public class MainWindow extends JFrame implements MouseListener, WindowListener {
	private static final long serialVersionUID = -6575979899788187615L;

	/** The instance of the mainWindow */
	public static MainWindow instance;

	/**The menuBar*/
	private JMenuBar jMenuBar;

	/**The subtitle file that is currently opened by Sears*/
	private SubtitleFile subtitleFile;

	/**The main panel.*/
	private JPanel mainPanel;

	/**The JTable that contains the subtitles.*/
	private JTable table;

	/**The JTableModel that is used by the JTable.*/
	private SubtitleTableModel tableModel;

	/**The list of subtitles actually in the subtitle file opened*/
	private ArrayList<Subtitle> subtitleList;

	/**The application icon*/
	private ImageIcon searsIcon;

	/** The application 'favicon', use in frame and dialog title bar (Windows and Linux only) */
	private ImageIcon searsFavicon;

	/**The Actions hashMap where key is (String) actionId and Value is corresponding SearsAction.*/
	private HashMap<String, SearsAction> actions;

	/**The JToolBar*/
	private JToolBar jToolBar;

	/**The JPopupMenu*/
	private JPopupMenu jPopupMenu;

	/**The Player*/
	protected PlayerInterface player = null;

	/**The JPanelVideoControler that controls the video.*/
	private JPanelVideoControler jPanelVideoControler;

	/**The full path to the player executable.*/
	private static String playerVideoFile = null;

	/** (<b>ArrayList<File></b>) recentFiles: The recentFiles */
	private ArrayList<File> recentFiles = null;

	/** (<b>JMenu</b>) recentFileMenu: The recentFileMenu */
	private JMenu recentFileMenu;
	
	/** (<b>int</b>) RECENT_FILE_NUMBER: The max number of recent files to be displayed.*/
	private static final int RECENT_FILE_NUMBER = 10;

	/** (<b>SearsUndoManager</b>) undoManager: The undoManager */
	private final SearsUndoManager undoManager;
	
	/**The Actions Keys*/
	/**Open action key, to open a subtitle file*/
	protected static String ACTION_KEY_OPEN = "open";

	/**Save action key, to save current file.*/
	protected static String ACTION_KEY_SAVE = "save";

	/**SaveAs action key, to save current file.*/
	protected static String ACTION_KEY_SAVE_AS = "saveAs";

	/**Quit action key, to save current file.*/
	protected static String ACTION_KEY_QUIT = "quit";

	/**Delay action key, to delay ST.*/
	protected static String ACTION_KEY_DELAY = "delay";

	/**Resynch action key, to resynch ST.*/
	protected static String ACTION_KEY_RESYNCH = "resynch";

	/**Split action key, to split ST.*/
	protected static String ACTION_KEY_SPLIT = "split";

	/**Append action key, to append ST.*/
	protected static String ACTION_KEY_APPEND = "append";

	/**Accent action key, to Accent repair.*/
	protected static String ACTION_KEY_ACCENT_REPAIR = "accentRepair";

	/**Chain action key, to chain repair.*/
	protected static String ACTION_KEY_CHAIN_REPAIR = "chainRepair";

	/**HTML action key, to html repair.*/
	protected static String ACTION_KEY_HTML_REPAIR = "htmlRepair";

	/**Order action key, to order repair.*/
	protected static String ACTION_KEY_ORDER_REPAIR = "orderRepair";

	/**Time action key, to time repair.*/
	protected static String ACTION_KEY_TIME_REPAIR = "timeRepair";

	/**NormalizeDuration action key, to time repair.*/
	protected static String ACTION_KEY_NORMALIZE_DURATION = "normalizeDuration";

	/**Select video action key.*/
	protected static String ACTION_KEY_SELECT_VIDEO = "selectVideo";

	/**Options action key.*/
	protected static String ACTION_KEY_OPTIONS = "options";

	/**Play action key.*/
	protected static String ACTION_KEY_PLAY = "play";

	/**Stop action key.*/
	protected static String ACTION_KEY_STOP = "stop";

	/**Pause action key.*/
	protected static String ACTION_KEY_PAUSE = "pause";

	/**Previous action key.*/
	protected static String ACTION_KEY_PREVIOUS = "previous";

	/**Next action key.*/
	protected static String ACTION_KEY_NEXT = "next";

	/**TrackTrigger action key.*/
	protected static String ACTION_KEY_TRACKER_TRIGGER = "trackTrigger";

	/**gotoSub action key.*/
	protected static String ACTION_KEY_GOTO_SUB = "gotoSub";

	/**magicResynchro action key.*/
	protected static String ACTION_KEY_MAGIC_RESYNCHRO = "magicResynchro";

	/**remoaveAllAnchors action key.*/
	protected static String ACTION_KEY_REMOVE_ALL_ANCHORS = "removeAllAnchors";

	/**about action key.*/
	protected static String ACTION_KEY_ABOUT = "about";

	/**find action key.*/
	protected static String ACTION_KEY_FIND = "find";

	/**find action key.*/
	protected static String ACTION_KEY_MIX = "mix";

	/** (<b>String[]</b>) FILE_OPEN_DEPENDANT_ACTIONS: The Actions that depend on a file open status*/
	private static final String[] FILE_OPEN_DEPENDANT_ACTIONS = new String[] {
		ACTION_KEY_ACCENT_REPAIR
		, ACTION_KEY_APPEND
		, ACTION_KEY_CHAIN_REPAIR
		, ACTION_KEY_DELAY
		, ACTION_KEY_HTML_REPAIR
		, ACTION_KEY_MAGIC_RESYNCHRO
		, ACTION_KEY_NORMALIZE_DURATION
		, ACTION_KEY_ORDER_REPAIR
		, ACTION_KEY_RESYNCH, ACTION_KEY_SAVE
		, ACTION_KEY_SAVE_AS
		, ACTION_KEY_SPLIT
		, ACTION_KEY_TIME_REPAIR 
		, ACTION_KEY_FIND
		, ACTION_KEY_MIX};

	/** (<b>String[]</b>) FILE_CHANGED_DEPENDANT_ACTIONS: The actions that depend on a file change status*/
	private static final String[] FILE_CHANGED_DEPENDANT_ACTIONS = new String[]{
		ACTION_KEY_SAVE};

	private static boolean resetFlag = false;

	private JScrollPane tableJScrollPane;
	// FindModule object
	private FindModule fm;
	private JSplitPane toolbarSplitPane;
	private FindBox findBox;
	
	private PlayerTimeHandler playerTimeHandler;

	/**
	 * 
	 * Constructor MainWindow.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * construct the GUI.
	 */
	public MainWindow() {
		super();
		instance = this;
		//create undo manager.
		undoManager = new SearsUndoManager();
		//restore recent files
		recentFiles = new ArrayList<File>();
		String recentFileProperty = SearsProperties.getProperty(SearsProperties.RECENT_FILES);
		StringTokenizer stk = new StringTokenizer(recentFileProperty, ";");
		while(stk.hasMoreTokens()){
			String recentFile = stk.nextToken();
			File aFile = new File(recentFile);
			// we add to the menu only the existing files
			// if a file was moved it does not appear at all
			if(aFile.exists()) {
				recentFiles.add(aFile);
			}
		}
		//try to get the locale from properties
		String lang = SearsProperties.getProperty(SearsProperties.LANGUAGE, "en");
		String count = SearsProperties.getProperty(SearsProperties.COUNTRY, "US");
		Locale locale = new Locale(lang, count);
		new SearsResourceBundle();
		setSelectedLocale(locale);
		//set the look and feel on the current OS.
		try {
			String selectedLookAndFeel = SearsProperties.getProperty(SearsProperties.LOOK_AND_FEEL, UIManager
					.getSystemLookAndFeelClassName());
			UIManager.setLookAndFeel(selectedLookAndFeel);
			//SwingUtilities.updateComponentTreeUI(instance);
			SearsProperties.setProperty(SearsProperties.LOOK_AND_FEEL, selectedLookAndFeel);
			//MainWindow.instance.repaint();
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (InstantiationException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IllegalAccessException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (UnsupportedLookAndFeelException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		//Set the size.
		int width = Integer.parseInt(SearsProperties.getProperty("MainWindow" + SearsProperties.SUFFIX_WIDTH, "600"));
		int heigth = Integer.parseInt(SearsProperties.getProperty("MainWindow" + SearsProperties.SUFFIX_HEIGTH, "400"));
		setSize(width, heigth);
		searsFavicon = SearsResources.getIcon("SearsGUIFavicon");
		searsIcon = SearsResources.getIcon("SearsGUIIcon");
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		//Set the frame's icon
		setIconImage(searsFavicon.getImage());
		updateWindowsName();
		centerFrame(instance);
		addWindowListener(this);
		//Construct the actions.
		constructActions();
		mainPanel = new JPanel();
		mainPanel.setLayout(new GridLayout(1, 1));
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		constructJMenuBar();
		setJMenuBar(jMenuBar);
		constructJToolBar();
		
		// -------------
		// search field:
		findBox = new FindBox(this);
		findBox.setMinimumSize(new Dimension(150,0));
		
		toolbarSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, jToolBar, findBox);
		toolbarSplitPane.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));		
		toolbarSplitPane.setContinuousLayout(true);
		// get back the divider pixel location:
		int dividerLocation;
		try {
			dividerLocation = Integer.parseInt(SearsProperties.getProperty(SearsProperties.TOOLBAR_DIVIDER_LOCATION,"T"));
			if( dividerLocation == 0 ) {
				// percent location:
				// width --> windows calculate width
				dividerLocation = 3*width/4;
			}
		} catch(NumberFormatException e) {
			dividerLocation = 3*width/4;
		}
		toolbarSplitPane.setDividerLocation(dividerLocation);
		// -------------
		
		getContentPane().add(toolbarSplitPane, BorderLayout.NORTH);
		getContentPane().add(getJPanelVideoControler(), BorderLayout.SOUTH);
		subtitleList = new ArrayList<Subtitle>();
		tableModel = new SubtitleTableModel(subtitleList);
		table = new JTable(tableModel);
		table.setShowHorizontalLines(false);
		table.setShowVerticalLines(false);
		// froze bug [ 1669501 ] by disable the reorder by user:
		table.getTableHeader().setReorderingAllowed(false);
		// -----------------------------------------------------
		table.setRowSelectionAllowed(true);
		table.addMouseListener(this);
		table.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				fireFindStop();

			}
		});
		//  Add the Transfer handler, which will handle the files Drag and drops.
		SubtitleFileTransfertHandler searsTransferHandler = new SubtitleFileTransfertHandler();
		getRootPane().setTransferHandler(searsTransferHandler);
		table.setTransferHandler(searsTransferHandler);
		//create table renderer.
		//setting the table column sizes and renderer.
		TableColumn column = null;
		for (int i = 0; i < SubtitleTableModel.COLUMN_NUMBER; i++) {
			column = table.getColumnModel().getColumn(i);
			column.setCellRenderer(new SubtitleTableCellRenderer(subtitleList));
			column.setHeaderRenderer(new SubtitleAnchorHeaderTableCellRenderer(
					table.getTableHeader().getDefaultRenderer()));
			switch (i) {
			case SubtitleTableModel.ANCHOR_COLUMN: {
				column.setPreferredWidth(18);
				column.setMaxWidth(18);
				break;
			}
			case SubtitleTableModel.ID_COLUMN: {
				column.setPreferredWidth(40);
				column.setMaxWidth(60);
				break;
			}
			case SubtitleTableModel.START_DATE_COLUMN: {
				column.setPreferredWidth(100);
				column.setMaxWidth(100);
				break;
			}
			case SubtitleTableModel.END_DATE_COLUMN: {
				column.setPreferredWidth(100);
				column.setMaxWidth(100);
				break;
			}
			}
		}
		tableJScrollPane = new JScrollPane(table);	
		mainPanel.add(tableJScrollPane);

		fm = new FindModule(tableJScrollPane);

		//
		// Create the thread of video time handler
		playerTimeHandler = new PlayerTimeHandler();
		Thread playerTimeHandlerThread = new Thread(playerTimeHandler, "playerTimeHandler");
		playerTimeHandlerThread.start();
		jPanelVideoControler.setJSliderSearchListener(playerTimeHandler);

		lookForUpdate();
	}

	/**
	 * Returns the UndoManager
	 * @return the undoManager
	 */
	public SearsUndoManager getUndoManager() {
		return undoManager;
	}

	/**
	 * Method getJPanelVideoControler.
	 * <br><b>Summary:</b><br>
	 * Return the JPanel that control the video.
	 * @return The JPanel video controler.
	 */
	private JPanel getJPanelVideoControler() {
		if (jPanelVideoControler == null) {
			jPanelVideoControler = new JPanelVideoControler();
		}
		return jPanelVideoControler;
	}

	/**
	 * Method lookForUpdate.
	 * <br><b>Summary:</b><br>
	 * This method look for an update, if the SearsProperties.UPDATE_ADDRESS is not equals to "".
	 * and will display a message to the user if a new Sears version is available.
	 */
	private void lookForUpdate() {
		//If an address update has been set, then update.
		String updateAddress = SearsProperties.getProperty(SearsProperties.UPDATE_ADDRESS, "firstLaunch");
		if (!updateAddress.equals("")) {
			//If it is the first launch
			if (updateAddress.equals("firstLaunch")) {
				setUpdateAddress(Updater.DEFAULT_UPDATE_ADDRESS);
				updateAddress = Updater.DEFAULT_UPDATE_ADDRESS;
			}
			//look for update.
			new Updater(this, updateAddress, Double.parseDouble(Version.VERSION));
		}
	}

	/**
	 * Method setUpdateAddress.
	 * <br><b>Summary:</b><br>
	 * This method set the new update address.
	 * @param address   The new update address to set.
	 */
	private void setUpdateAddress(String address) {
		SearsProperties.setProperty(SearsProperties.UPDATE_ADDRESS, address);
	}

	/**
	 * Method constructActions.
	 * <br><b>Summary:</b><br>
	 * This method construct the actions.
	 */
	private void constructActions() {
		actions = new HashMap<String, SearsAction>();
		//construct Open action
		actions.put(ACTION_KEY_OPEN, new OpenAction());
		//construct Save action
		actions.put(ACTION_KEY_SAVE, new SaveAction());
		//construct SaveAs action
		actions.put(ACTION_KEY_SAVE_AS, new SaveAsAction());
		//construct quit action
		actions.put(ACTION_KEY_QUIT, new QuitAction());
		//construct Delay action
		actions.put(ACTION_KEY_DELAY, new DelayAction());
		//construct Resynch action
		actions.put(ACTION_KEY_RESYNCH, new ResynchAction());
		//construct Split action
		actions.put(ACTION_KEY_SPLIT, new SplitAction());
		//construct Append action
		actions.put(ACTION_KEY_APPEND, new AppendAction());
		//construct AccentRepairAction
		actions.put(ACTION_KEY_ACCENT_REPAIR, new AccentRepairAction());
		//construct ChainRepairAction
		actions.put(ACTION_KEY_CHAIN_REPAIR, new ChainRepairAction());
		//construct HtmlRepairAction
		actions.put(ACTION_KEY_HTML_REPAIR, new HtmlRepairAction());
		//construct OrderRepairAction
		actions.put(ACTION_KEY_ORDER_REPAIR, new OrderRepairAction());
		//construct TimeRepairAction
		actions.put(ACTION_KEY_TIME_REPAIR, new TimeRepairAction());
		//construct NormalizeDurationAction
		actions.put(ACTION_KEY_NORMALIZE_DURATION, new NormalizeDurationAction());
		//construct SelectVideoAction
		actions.put(ACTION_KEY_SELECT_VIDEO, new SelectVideoAction());
		//construct optionsAction
		actions.put(ACTION_KEY_OPTIONS, new OptionsAction());
		//construct playAction
		actions.put(ACTION_KEY_PLAY, new PlayAction());
		//construct stopAction
		actions.put(ACTION_KEY_STOP, new StopAction());
		//construct pauseAction
		actions.put(ACTION_KEY_PAUSE, new PauseAction());
		//construct previousAction
		actions.put(ACTION_KEY_PREVIOUS, new PreviousAction());
		//construct nextAction
		actions.put(ACTION_KEY_NEXT, new NextAction());
		//construct trackerTriggerAction
		actions.put(ACTION_KEY_TRACKER_TRIGGER, new TrackerTriggerAction());
		//construct gotoSubAction
		actions.put(ACTION_KEY_GOTO_SUB, new GotoSubAction());
		//construct magicResynchroAction
		actions.put(ACTION_KEY_MAGIC_RESYNCHRO, new MagicResynchroAction());
		//Construct removeAllAnchorsAction
		actions.put(ACTION_KEY_REMOVE_ALL_ANCHORS, new RemoveAllAnchorsAction());
		//Construct aboutAction
		actions.put(ACTION_KEY_ABOUT, new AboutAction());
		//Construct findAction
		actions.put(ACTION_KEY_FIND, new FindAction());
		//construct mixAction
		actions.put(ACTION_KEY_MIX, new MixAction());		
		//Add undo Action
		actions.put(SearsUndoManager.ACTION_KEY_UNDO, undoManager.getUndoAction());
		//Add redo Action
		actions.put(SearsUndoManager.ACTION_KEY_REDO, undoManager.getRedoAction());
		//update the actions status
		updateActionStatus();
	}

	/**
	 * Method constructJMenuBar.
	 * <br><b>Summary:</b><br>
	 * This method construct the JMenuBar
	 */
	private void constructJMenuBar() {
		jMenuBar = new JMenuBar();
		/////////////
		//FILE MENU
		/////////////
		JMenu jMenuFile = new JMenu(SearsResourceBundle.getResource("menuBar_file"));
		jMenuBar.add(jMenuFile);
		//OPEN ITEM
		jMenuFile.add(new JMenuItem(getAction(ACTION_KEY_OPEN)));
		//OPEN RECENT FILE
		recentFileMenu = new JMenu(SearsResourceBundle.getResource("RecentFileName"));
		recentFileMenu.setToolTipText(SearsResourceBundle.getResource("RecentFileTip"));
		recentFileMenu.setIcon(SearsResources.getIcon("RecentFileIcon"));
		// menu is updated each time the user access to it,
		// if user delete a recent subtitle, it does not appear on the menu:
		recentFileMenu.addMouseListener(new MouseAdapter() {
			public void mouseEntered(MouseEvent arg0) {
				instance.updateRecentFileMenu();
			}
		});
		jMenuFile.add(recentFileMenu);
		//updateRecentFileMenu();
		//SAVE ITEM
		jMenuFile.add(new JMenuItem(getAction(ACTION_KEY_SAVE)));
		//SAVE AS ITEM
		jMenuFile.add(new JMenuItem(getAction(ACTION_KEY_SAVE_AS)));
		//QUIT ITEM
		//if platform isn't a Mac platform:
		if(!Utils.isMacPlatform){
			jMenuFile.add(new JMenuItem(getAction(ACTION_KEY_QUIT)));
		}		
		//////////////////
		//EDIT MENU
		/////////////////
		JMenu jMenuEdit = new JMenu(SearsResourceBundle.getResource("menuBar_edit"));
		jMenuBar.add(jMenuEdit);
		//DELAY ITEM
		jMenuEdit.add(new JMenuItem(getAction(ACTION_KEY_FIND)));	
		//UNDO REDO
		jMenuEdit.add(new JMenuItem(getAction(SearsUndoManager.ACTION_KEY_UNDO)));
		jMenuEdit.add(new JMenuItem(getAction(SearsUndoManager.ACTION_KEY_REDO)));

		//////////////////
		//ACTION MENU
		/////////////////
		JMenu jMenuAction = new JMenu(SearsResourceBundle.getResource("menuBar_action"));
		jMenuBar.add(jMenuAction);
		//DELAY ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_DELAY)));
		//RESYNCHRO ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_RESYNCH)));
		//MAGIC RESYNCHRO ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_MAGIC_RESYNCHRO)));
		//REMOVE ALL ANCHORS ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_REMOVE_ALL_ANCHORS)));
		//Add a separator
		jMenuAction.addSeparator();
		//SPLIT FILE ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_SPLIT)));
		//APPEND FILE ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_APPEND)));
		//MIX FILE ITEM
		jMenuAction.add(new JMenuItem(getAction(ACTION_KEY_MIX)));

		//////////////////
		//REPAIR MENU
		/////////////////
		JMenu jMenuRepair = new JMenu(SearsResourceBundle.getResource("menuBar_repair"));
		jMenuBar.add(jMenuRepair);
		//CHAIN REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_CHAIN_REPAIR)));
		//Add a separator
		jMenuRepair.addSeparator();
		//ACCENT REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_ACCENT_REPAIR)));
		//HTML REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_HTML_REPAIR)));
		//Add a separator
		jMenuRepair.addSeparator();
		//ORDER REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_ORDER_REPAIR)));
		//TIME REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_TIME_REPAIR)));
		//TIME REPAIR
		jMenuRepair.add(new JMenuItem(getAction(ACTION_KEY_NORMALIZE_DURATION)));
		//////////////////
		//PLAYER MENU
		/////////////////
		JMenu jMenuPlayer = new JMenu(SearsResourceBundle.getResource("menuBar_player"));
		jMenuBar.add(jMenuPlayer);
		//SELECT VIDEO
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_SELECT_VIDEO)));
		//LAUNCH PLAYER
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_PLAY)));
		//PAUSE PLAYER
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_PAUSE)));
		//STOP PLAYER
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_STOP)));
		//PREVIOUS PLAYER
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_PREVIOUS)));
		//NEXT PLAYER
		jMenuPlayer.add(new JMenuItem(getAction(ACTION_KEY_NEXT)));
		//////////////////
		//OPTIONS MENU
		/////////////////
		JMenu jMenuOptions = new JMenu(SearsResourceBundle.getResource("menuBar_options"));
		jMenuBar.add(jMenuOptions);
		//OPTIONS
		jMenuOptions.add(new JMenuItem(getAction(ACTION_KEY_OPTIONS)));
		//////////////////
		//HELP MENU
		/////////////////
		JMenu helpMenu = new JMenu(SearsResourceBundle.getResource("menuBar_help"));
		jMenuBar.add(helpMenu);
		JMenuItem helpItem = new JMenuItem(SearsResourceBundle.getResource("menuBar_help"));
		helpItem.setIcon(SearsResources.getIcon("HelpIcon"));
		helpItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				//default title and icon
				JOptionPane
				.showMessageDialog(
						instance,
						"You can find help on Sears\nby going to sears sourceforge page:\nhttp://sears.sourceforge.net/\n ",
						"Sears Help", JOptionPane.INFORMATION_MESSAGE, searsIcon);
			}
		});
		helpMenu.add(helpItem);
		helpMenu.add(new JMenuItem(getAction(ACTION_KEY_ABOUT)));
	}

	/**
	 * Method constructJMenuBar.
	 * <br><b>Summary:</b><br>
	 * This method construct the JMenuBar
	 */
	private void constructJToolBar() {
		jToolBar = new JToolBar();
		jToolBar.setFloatable(false);
		/////////////
		//FILE TOOLS
		/////////////
		//OPEN ITEM
		JButton button = new JButton(getAction(ACTION_KEY_OPEN));
		button.setText(null);
		button.addMouseListener(new MouseAdapter() {
			//
			// DEVELOPER: the idea is to show popup menu when
			// user pressed key more than 2 seconds.
			// The action on the open button calls a non thread-safe method
			// the showSRTBrowser do not released the button before begin showing the open dialog
			// and so it's a problem...
			public void mousePressed(MouseEvent e) {
				JButton button = (JButton) e.getSource();
				if( e.isMetaDown() ) {
					JPopupMenu popupMenu = new JPopupMenu();
					updateRecentFileMenu();
					Component[] components = recentFileMenu.getMenuComponents();
					for( Component cmp : components) {
						popupMenu.add(cmp);							
					} 
					popupMenu.show(button, button.getX(), button.getY());

				}
			}
		});
		jToolBar.add(button);
		//SAVE  ITEM
		button = new JButton(getAction(ACTION_KEY_SAVE));
		button.setText(null);
		jToolBar.add(button);
		//add separator
		jToolBar.addSeparator();
		//UNDO REDO
		button = new JButton(getAction(SearsUndoManager.ACTION_KEY_UNDO));
		button.setText(null);
		jToolBar.add(button);
		button = new JButton(getAction(SearsUndoManager.ACTION_KEY_REDO));
		button.setText(null);
		jToolBar.add(button);
		
		//////////////////
		//ACTION MENU
		/////////////////
		//DELAY ITEM
		button = new JButton(getAction(ACTION_KEY_DELAY));
		button.setText(null);
		jToolBar.add(button);
		//MAGIC RESYNCHRO ITEM
		button = new JButton(getAction(ACTION_KEY_MAGIC_RESYNCHRO));
		button.setText(null);
		jToolBar.add(button);
		//REMOVE ALL ANCHORS ITEM
		button = new JButton(getAction(ACTION_KEY_REMOVE_ALL_ANCHORS));
		button.setText(null);
		jToolBar.add(button);
		//SPLIT ITEM
		button = new JButton(getAction(ACTION_KEY_SPLIT));
		button.setText(null);
		jToolBar.add(button);
		//APPEND ITEM
		button = new JButton(getAction(ACTION_KEY_APPEND));
		button.setText(null);
		jToolBar.add(button);
		//add separator
		jToolBar.addSeparator();
		//////////////////
		//REPAIR MENU
		/////////////////
		//CHAIN REPAIR ITEM
		button = new JButton(getAction(ACTION_KEY_CHAIN_REPAIR));
		button.setText(null);
		jToolBar.add(button);

		//ACCENT REPAIR ITEM
		button = new JButton(getAction(ACTION_KEY_ACCENT_REPAIR));
		button.setText(null);
		jToolBar.add(button);
		//HTML REPAIR ITEM
		button = new JButton(getAction(ACTION_KEY_HTML_REPAIR));
		button.setText(null);
		jToolBar.add(button);
		jToolBar.addSeparator();
		//////////////////
		//TOOLS MENU
		//FIND
		/*button = new JButton(getAction(ACTION_KEY_FIND));
		button.setText(null);		
		jToolBar.add(button);*/

		/////////////////
		//OPTIONS BUTTON
		button = new JButton(getAction(ACTION_KEY_OPTIONS));
		button.setText(null);
		jToolBar.add(button);
		
		jToolBar.addSeparator(new Dimension(10,0));
	}

	/**
	 * Method getAction.
	 * <br><b>Summary:</b><br>
	 * Return the SearsActions that correspond to the given key.
	 * @param   key             The <b>String</b> Action key to search for.
	 * @return  SearsAction     The <b>SearsAction</b> that correspond to the key, or null if not found.
	 */
	protected SearsAction getAction(String key) {
		//Search and return the action in the actions map.
		return (SearsAction) actions.get(key);
	}

	/**
	 * Method delayAction.
	 * <br><b>Summary:</b><br>
	 * Use this method to set a delay on subtitles.
	 * It will show a dialog to enter a delay to the selected subtitles.
	 * @return  (<b>boolean</b>)   True if action succeeds.
	 */
	protected boolean delayAction() {
		//The result
		boolean result = false;
		//Must have opened a subtitle file to apply a delay.
		if (subtitleFile != null) {
			//get the selected rows to delay.
			int[] indexToDelay = table.getSelectedRows();
			//ask the user for delay to apply
			DelayDialog dd = new DelayDialog();
			dd.setVisible(true);
			//if user choose a delay.
			if (dd.hasBeenValidated()) {
				double delay = dd.getDelay();
				//delay the subtitle with the given delay.
				//multiply by 1000, because delay is indicated in second, with possible milliseconds.
				subtitleFile.delay(indexToDelay, (int) (delay * 1000));
				result = true;
				updateTableAndActions();
			}
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeDelay"), Trace.WARNING_PRIORITY);
		}
		return result;
	}
	
	private FindDialog _fd;
	public void addFB(FindDialog fd) {
		_fd = fd;
		
	}
	
	/**
	 * Method updateWholeTable.
	 * <br><b>Summary:</b><br>
	 * Updates the whole table.
	 */
	public void updateWholeTable(){
		//indicate table that data have changed
		tableModel.fireTableDataChanged();
	}

	/**
	 * Method resynchroAction.
	 * <br><b>Summary:</b><br>
	 * Use this method to resynchro subtitles.
	 * If user has selected the Two source point, the dialog shown will be pre-filled with good values.
	 * @return boolean true if action succeed.
	 */
	protected boolean resynchroAction() {
		boolean result = false;
		//Must have opened a subtitle file to apply a resynchro.
		if (subtitleFile != null) {
			//check wether user choose two source subtitles.
			int[] selectedIndex = table.getSelectedRows();
			ResynchroDialog rd;
			//If user choose sources, use them in shown dialog.
			if (selectedIndex.length > 1) {
				//Keep only 2 subtitles. interpolation of level n will come after :)
				int[] selectedIndex2 = new int[2];
				selectedIndex2[0] = selectedIndex[0];
				selectedIndex2[1] = selectedIndex[1];
				//TODO: create n-level interpolation in SubtitleFile!
				//construct the dialog.
				rd = new ResynchroDialog(subtitleList, selectedIndex2);
			} else {
				//If no subtitles has been selected, construct empty resynchro dialog.
				rd = new ResynchroDialog(subtitleList);
			}
			//show the constructed dialog.
			rd.setVisible(true);
			//check user's response.
			if (rd.hasBeenValidated()) {
				//if user has validated the dialog, get the result.
				int[] dialogResult = rd.getResult();
				//and resynchro the subtitles.
				subtitleFile.resynchro(dialogResult);
				result = true;
				//indicate the table that data have changed.
				updateWholeTable();
			}
			updateWindowsName();
			updateActionStatus();
		} else {
			//Must have opened a subtitle file to apply a resynchro.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeResynch"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Method magicResynchroAction.
	 * <br><b>Summary:</b><br>
	 * Apply a magic resynchro
	 */
	protected boolean magicResynchroAction(){
		boolean result = false;
		if (subtitleFile != null) {
			subtitleFile.magicResynchro();
			result = true;
			//After magic resynchro, remove all anchors.
			removeAllAnchorsAction();
			updateWindowsName();
			updateActionStatus();
		} else {
			//Must have opened a subtitle file to apply a resynchro.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeResynch"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Method saveAsAction.
	 * <br><b>Summary:</b><br>
	 * Method called when want to save the subtitle file.
	 * open a JFolderChooser to chooser where to save the file.
	 */
	protected void saveAsAction() {
		Trace.trace("SaveAsAction", Trace.ALGO_PRIORITY);
		DialogUtils du = new DialogUtils();
		// set options:
		du.setConditions(false, true);
		du.setApproveButtonText(SearsResourceBundle.getResource("button_saveAs"));
		du.setOverwriteText(SearsResourceBundle.getResource("various_overwrite"));
		du.setFileFilter(new FileFilter() {
			public String getDescription() {
				return SearsResourceBundle.getResource("various_subtitleFileDescription");
			}

			public boolean accept(File f) {
				boolean result = f.isDirectory() 
				|| Utils.getExtension(f).equals("srt")
				|| Utils.getExtension(f).equals("ssa")
				|| Utils.getExtension(f).equals("sub");
				return result;
			}
		});

		// we get back the file choosen
		File file = du.showSaveAsDialog(instance, SearsResourceBundle.getResource("SaveAsName"), subtitleFile.getFile());
		// if user cancel action, file is null:
		if (file != null){
			if( fsa.saveFile(file, subtitleFile, null) ) {
				openFile(file);
			}
			//updateWindowsName();
			//updateActionStatus();
		}
	}

	/**
	 * Method saveAction.
	 * <br><b>Summary:</b><br>
	 * Method called when want to save the subtitle file.
	 */
	protected void saveAction() {
		Trace.trace("SaveAction", Trace.ALGO_PRIORITY);
		if ( fsa.saveFile(subtitleFile.getFile(), subtitleFile, null) ) {
			updateWindowsName();
			updateActionStatus();
		}
	}

	/**
	 * Method quitAction.
	 * <br><b>Summary:</b><br>
	 * Use this method to quit Sears.
	 */
	public void quitAction() {
		dispose();
		System.exit(0);
	}

	public void dispose() {
		//check current file before quit.
		if (checkUnsavedFileBeforeAction()) {
			Trace.trace(SearsResourceBundle.getResource("various_quittingSears"), Trace.ALGO_PRIORITY);
			if (player != null) {
				player.quit();
			}

			// if user doesn't have press the reset button:
			if(!resetFlag) {
				// we save properties:
				saveMainWindowProperties();
				SearsProperties.saveProperties();
			} else {
				try {
					// reset operation:
					SearsProperties.resetProperty();
					Trace.trace("Deleting property file", Trace.MESSAGE_PRIORITY);
				} catch (IOException e) {
					// DEVELOPER: try to write some flag in property file, like delete=1 
					// or something like that. When Sears is reopened, check this mark and try
					// to do the job at this time. 
					Trace.trace("Cannot delete property file, I/O exception occurs", Trace.ERROR_PRIORITY);
				}
			}
		}
		super.dispose();
	}

	/**
	 * Method saveMainWindowProperties.
	 * <br><b>Summary:</b><br>
	 * This method saves the MainWindow properties of Sears.
	 */
	private void saveMainWindowProperties() {
		//save windows properties.
		SearsProperties.setProperty("MainWindow" + SearsProperties.SUFFIX_WIDTH, "" + this.getWidth());
		SearsProperties.setProperty("MainWindow" + SearsProperties.SUFFIX_HEIGTH, "" + this.getHeight());
		//save the folder property.
		if (subtitleFile != null) {
			SearsProperties.setProperty(SearsProperties.LAST_FOLDER, "" + subtitleFile.getContentDirectory());
		}
		//Save the locale.
		SearsProperties.setProperty(SearsProperties.LANGUAGE, getLocale().getLanguage().toString());
		SearsProperties.setProperty(SearsProperties.COUNTRY, getLocale().getCountry().toString());
		//Save the recent files list.
		String recentFilesProperty = "";
		for(File file: recentFiles){
			recentFilesProperty+=file.getAbsolutePath()+";";
		}
		SearsProperties.setProperty(SearsProperties.RECENT_FILES, recentFilesProperty);
		SearsProperties.setProperty(SearsProperties.TOOLBAR_DIVIDER_LOCATION, String.valueOf(toolbarSplitPane.getDividerLocation()));

	}

	/**
	 * Method openAction.
	 * <br><b>Summary:</b><br>
	 * Use this method to open a subtitle file.
	 * Displays a dialog to choose the file to open.
	 */
	protected void openAction() {
		Trace.trace("OpenAction", Trace.ALGO_PRIORITY);
		File file = showSRTBrowser();
		if (file != null) {
			//If user has choose a file to open, open it.
			Trace.trace(SearsResourceBundle.getResource("various_opening") + " " + file.getName() + ".");
			// Close the video file
			if (subtitleFile != null) {
				playerVideoFile = null;
			}
			openFile(file);
			// Check if the player is running
			if(player != null) {
				stopAction();
				playAction();
			}
		} else {
			//if user canceled opening action, do nothing.
			Trace.trace(SearsResourceBundle.getResource("various_openCommandCanceled"));
		}
	}

	/**
	 * Method showBrowser.
	 * <br><b>Summary:</b><br>
	 * This method shows a browser, to permits subtitle file selection.
	 * Open the browser at the current subtitle file location, if any.
	 * @param filter      The file filter
	 * @return  <b>File</b>  
	 */
	public File showBrowser(FileFilter filter) {
		//The result of the method.
		File result = null;
		if(subtitleFile != null){
			result = showBrowser(filter, subtitleFile.getFile());
		}else{
			result = showBrowser(filter, null);
		}
		//return the result.
		return result;
	}

	/**
	 * Method showBrowser.
	 * <br><b>Summary:</b><br>
	 * This method shows a browser, to permits subtitle file selection.
	 * Open the browser at the given file location.
	 * @param filter       The file filter.
	 * @param sourceFile   Open the browser in the file's parent path.
	 * @return  <b>File</b>  
	 */
	public File showBrowser(FileFilter filter, File sourceFile) {
		File result = null;
		DialogUtils du = new DialogUtils();
		du.setConditions(true, true);
		du.setFileFilter(filter);
		if (sourceFile != null) {
			result = du.showOpenDialog(instance, "open", sourceFile);
		} else {
			result = du.showOpenDialog(instance, "open", new File(SearsProperties.getProperty(SearsProperties.LAST_FOLDER, "")));
		}
		return result;
	}

	/**
	 * Method showSRTBrowser.
	 * <br><b>Summary:</b><br>
	 * This method shows a browser, to permits subtitle file selection.
	 * @return  <b>File</b>  
	 */
	public File showSRTBrowser() {
		return showBrowser(new FileFilter() {
			public String getDescription() {
				return SearsResourceBundle.getResource("various_subtitleFileDescription");
			}

			public boolean accept(File f) {
				boolean result = f.isDirectory() 
				|| Utils.getExtension(f).equals("srt")
				|| Utils.getExtension(f).equals("ssa")
				|| Utils.getExtension(f).equals("sub");
				return result;
			}
		});
	}

	/**
	 * Method showVideoBrowser.
	 * <br><b>Summary:</b><br>
	 * This method shows a browser, to permits video selection.
	 * @return  <b>File</b>  
	 */
	public File showVideoBrowser() {
		return showBrowser(new FileFilter() {
			public String getDescription() {
				return SearsResourceBundle.getResource("various_videoFileDescription");
			}

			public boolean accept(File f) {
				return f.isDirectory() || Utils.getExtension(f).equals("avi") || Utils.getExtension(f).equals("mkv")
				|| Utils.getExtension(f).equals("divx") || Utils.getExtension(f).equals("mov")
				|| Utils.getExtension(f).equals("ogm") || Utils.getExtension(f).equals("wmv")
				|| Utils.getExtension(f).equals("mpg") || Utils.getExtension(f).equals("mpeg");
			}
		});
	}

	private FileSystemAccess fsa;

	/**
	 * Opens a file with the specific charset
	 * @param file		the file to open
	 * @param charset	the charset name
	 */
	public void openFile(File file, String charset) {
		fsa = FileSystemAccess.getInstance();
		SubtitleFile openedSubtitleFile = fsa.openFile(file, subtitleList, charset);
		// if opening file succeeded,
		// check current file before open.
		if( openedSubtitleFile != null ) {
			if ( checkUnsavedFileBeforeAction() ) {
				setSubtitleFile(openedSubtitleFile);
			} // else do nothing user cancel the open operation
		}
		//discard all edits, since we open a new file.
		undoManager.discardAllEdits();
	}

	public String showOpenErrorDialog(String message, String detail, boolean reOpenChoice) {
		return ErrorDialog.showOpenErrorDialog(message, detail, reOpenChoice);
	}

	public void showSaveErrorDialog(String message, String detail) {
		ErrorDialog.showSaveErrorDialog(message, detail);
	}

	/**
	 * Sets a new subtitle file
	 * @param newSubtitleFile the new subtitle file, if it's null this method is no op
	 */
	public void setSubtitleFile(SubtitleFile newSubtitleFile) {
		if( newSubtitleFile != null ) {
			//Empty the UndoManager here, not to apply old edit to new file.
			undoManager.discardAllEdits();
			subtitleFile = newSubtitleFile;
			//indicate table that data have changed.					
			updateTableAndActions();
			addToRecentFileList(newSubtitleFile.getFile());
		}
		
		_fd.setFindEnabled(true);
	}

	/**
	 * Method openFile. <br>
	 * <b>Summary:</b><br>
	 * This method open a subtitle file, and put it in the table.
	 * 
	 * @param file
	 *            The subtitle file to open
	 */
	public void openFile(File file) {
		openFile(file, null);
	}

	/**
	 * Method addToRecentFileList.
	 * <br><b>Summary:</b><br>
	 * This method permits to add a new file to recent file list.
	 * @param file		The file that has been opened.
	 */
	private void addToRecentFileList(File file) {
		//remove if previously added.
		recentFiles.remove(file);
		//save the newly opened file under recent file list.
		recentFiles.add(file);
		//Remove oldest one, if list is too long.
		if(recentFiles.size() > RECENT_FILE_NUMBER){
			recentFiles.remove(0);
		}
		//Update the recentFile menu.
		updateRecentFileMenu();
	}

	/**
	 * Method updateRecentFileMenu.
	 * <br><b>Summary:</b><br>
	 * Permit to update the recent file menu.
	 */
	private void updateRecentFileMenu() {
		//Remove all previous menu items.
		recentFileMenu.removeAll();
		//And parse all the recent file adding a menuItem for each file.
		//parse in reverse order, to put older opened files at the end.
		for(int i= recentFiles.size()-1; i >= 0 ;i --){
			final File finalFile = recentFiles.get(i);
			JMenuItem jMenuItem = new JMenuItem(finalFile.getName());
			if(finalFile.exists()) {			
				jMenuItem.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {				
						openFile(finalFile);
					}
				});
				recentFileMenu.add(jMenuItem);
			}
		}
	}

	/**
	 * Method centerFrame.
	 * <br><b>Summary:</b><br>
	 * center frame on screen.
	 * @param window    The window to center.
	 */
	public static void centerFrame(Window window) {
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		int w = window.getSize().width;
		int h = window.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;
		window.setLocation(x, y);
	}

	/**
	 * Class DelayAction.
	 * <br><b>Summary:</b><br>
	 * The delay action.
	 */
	class DelayAction extends SearsUndoAction {
		private static final long serialVersionUID = -1004418974524773725L;

		public DelayAction() {
			super("Delay", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return delayAction();
		}
	}

	/**
	 * Class OpenAction.
	 * <br><b>Summary:</b><br>
	 * This is the open action, call when user wants to open a file.
	 */
	class OpenAction extends SearsAction {
		private static final long serialVersionUID = -6340643915376767725L;

		public OpenAction() {
			super("Open");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			openAction();
		}
	}

	/**
	 * Class SaveAction.
	 * <br><b>Summary:</b><br>
	 * The Save Action
	 */
	class SaveAction extends SearsAction {
		private static final long serialVersionUID = 4144500453750320579L;

		public SaveAction() {
			super("Save");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			saveAction();
		}
	}

	/**
	 * Class SaveAsAction.
	 * <br><b>Summary:</b><br>
	 * The SaveAs Action
	 */
	class SaveAsAction extends SearsAction {
		private static final long serialVersionUID = 8310536551059661660L;

		public SaveAsAction() {
			super("SaveAs");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			saveAsAction();
		}
	}

	/**
	 * Class ResynchAction.
	 * <br><b>Summary:</b><br>
	 * The Resynch Action
	 */
	class ResynchAction extends SearsUndoAction {
		private static final long serialVersionUID = -3381309933952740831L;

		public ResynchAction() {
			super("Resynch", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return resynchroAction();
		}
	}

	/**
	 * Class SplitAction.
	 * <br><b>Summary:</b><br>
	 * The Split Action
	 */
	class SplitAction extends SearsUndoAction {
		private static final long serialVersionUID = 4890494133812108855L;

		public SplitAction() {
			super("Split", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return splitAction();
		}
	}

	/**
	 * Class AppendAction.
	 * <br><b>Summary:</b><br>
	 * The Append Action
	 */
	class AppendAction extends SearsUndoAction {
		private static final long serialVersionUID = 307293671858946138L;

		public AppendAction() {
			super("Append", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return appendAction();
		}
	}

	/**
	 * Class QuitAction.
	 * <br><b>Summary:</b><br>
	 * The Quit Action
	 */
	class QuitAction extends SearsAction {
		private static final long serialVersionUID = -7251685640412615636L;

		public QuitAction() {
			super("Quit");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			quitAction();
		}
	}

	/**
	 * Class OrderRepairAction.
	 * <br><b>Summary:</b><br>
	 * The OrderRepair Action
	 */
	class OrderRepairAction extends SearsUndoAction {
		private static final long serialVersionUID = 8302695242689945983L;

		public OrderRepairAction() {
			super("OrderRepair", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return orderRepairAction();
		}
	}

	/**
	 * Class PlayAction.
	 * <br><b>Summary:</b><br>
	 * The Play Action
	 */
	class PlayAction extends SearsAction {
		private static final long serialVersionUID = -8943325412755437272L;

		public PlayAction() {
			super("Play");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			playAction();
		}
	}

	/**
	 * Class StopAction.
	 * <br><b>Summary:</b><br>
	 * The Stop Action
	 */
	class StopAction extends SearsAction {
		private static final long serialVersionUID = -5754273483777981782L;

		public StopAction() {
			super("Stop");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			stopAction();
		}
	}

	/**
	 * Class PauseAction.
	 * <br><b>Summary:</b><br>
	 * The Pause Action
	 */
	class PauseAction extends SearsAction {
		private static final long serialVersionUID = -4954880612379971754L;

		public PauseAction() {
			super("Pause");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			pauseAction();
		}
	}

	/**
	 * Class PreviousAction.
	 * <br><b>Summary:</b><br>
	 * The Previous Action
	 */
	class PreviousAction extends SearsAction {
		private static final long serialVersionUID = 7512470475148460128L;

		public PreviousAction() {
			super("Previous");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			previousAction();
		}
	}

	/**
	 * Class NextAction.
	 * <br><b>Summary:</b><br>
	 * The Next Action
	 */
	class NextAction extends SearsAction {
		private static final long serialVersionUID = -4498355509337819003L;

		public NextAction() {
			super("Next");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			nextAction();
		}
	}

	/**
	 * Class TrackerTriggerAction.
	 * <br><b>Summary:</b><br>
	 * The TrackerTrigger Action
	 */
	class TrackerTriggerAction extends SearsAction {
		private static final long serialVersionUID = 5091927415403379314L;

		public TrackerTriggerAction() {
			super("TrackerTrigger");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			trackerTriggerAction();
		}
	}

	/**
	 * Method orderRepairAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to repair the order of the subtitle file.
	 * It will check chronology, order ST's with their start time, and finally fix ST's numbers.
	 * @return boolean true if action succeeded
	 */
	public boolean orderRepairAction() {
		boolean result = false;
		//Must have opened a subtitle file to repair.
		if (subtitleFile != null) {
			//Apply order repair
			subtitleFile.orderRepair();
			result = true;
			updateTableAndActions();
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeRepair"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Method updateTableAndActions.
	 * <br><b>Summary:</b><br>
	 * update the whole table, the actions and the window name.
	 */
	public void updateTableAndActions() {
		//indicate table that data have changed
		updateWholeTable();
		tableChangedSinceTheLastSearch();
		updateWindowsName();
		updateActionStatus();
	}

	/**
	 * Method pauseAction.
	 * <br><b>Summary:</b><br>
	 * This method when user wants to pause the video
	 */
	public void pauseAction() {
		try {
			if(player != null) {
				//Pause the video player
				player.pause();
			}
		} catch (PlayerException e) {
			//display an error message.
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}

	/**
	 * Method trackerTriggerAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to change the tracker mode.
	 */
	public void trackerTriggerAction() {
		// TODO change the tracker mode.
		Trace.trace("TriggerAction");
	}

	/**
	 * Method nextAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to move forward in the video.
	 */
	public void nextAction() {
		try {
			if(player != null) {
				int currentOffset = player.getPosition();
				if(currentOffset != -1) {
					//Go forward using the time step.
					int timeStep = jPanelVideoControler.getTimeStep();
					player.setPosition(currentOffset + timeStep);
				}
			}
		} catch (PlayerException e) {
			//display an error message.
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}

	/**
	 * Method previousAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to go backward in the video.
	 */
	public void previousAction() {
		try {
			if(player != null) {
				int currentOffset = player.getPosition();
				if(currentOffset != -1) {        		
					//Go forward using the time step.
					int timeStep = jPanelVideoControler.getTimeStep();
					player.setPosition(currentOffset - timeStep);
				}
			}
		} catch (PlayerException e) {
			//display an error message.
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}

	/**
	 * Method stopAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to stop the video.
	 */
	public void stopAction() {
		if(player != null) {
			try {
				player.stop();
			} catch (PlayerException e) {
				//display an error message.
				Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
			}
		}
	}

	/**
	 * Method playAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to play the video.
	 */
	public void playAction() {
		String playerFile = SearsProperties.getProperty(SearsProperties.PLAYER_FULL_PATH);
		if (playerFile != "") {
			// Save the subtitle in tempory file
			SubtitleFile subFile = subtitleFile;
			if (subFile == null) {
				// Create a fake subtitle file if no file exists
				subFile = new SrtFile();
				subFile.addFakeSub();
			}
			subFile.writeToTemporaryFile();
			String subtitleFileName = subFile.getTemporaryFile().getAbsolutePath();

			// Launch VLC player
			try {
				if (player == null) {
					player = constructPlayer();
				}
				// Select the video file
				if (playerVideoFile == null) {
					selectVideoAction();
				} else {
					//Verify that user has chosen a file.
					if(playerVideoFile != null) {
						//Launch the video player
						player.play(playerVideoFile, subtitleFileName);
					}
				}
			} catch (PlayerException e) {
				//display an error message.
				Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
			}
		} else {
			JOptionPane.showOptionDialog(instance, 
					SearsResourceBundle.getResource("error_playerNotDefined"), 
					SearsResourceBundle.getResource("error_message"), 
					JOptionPane.CLOSED_OPTION,
					JOptionPane.ERROR_MESSAGE, null, null, null);
		}
	}

	/**
	 * Class AccentRepairAction.
	 * <br><b>Summary:</b><br>
	 * The AccentRepair Action
	 */
	class AccentRepairAction extends SearsUndoAction {
		private static final long serialVersionUID = 1881880058709461107L;

		public AccentRepairAction() {
			super("AccentRepair", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return accentRepairAction();
		}
		
		
	}

	/**
	 * Method accentRepairAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to remove 
	 * the accents and special characters from the selected subtitle file.
	 * @return true if action succeeded.
	 */
	public boolean accentRepairAction() {
		boolean result = false;
		//Must have opened a subtitle file to repair accents.
		if (subtitleFile != null) {
			//get the selected rows.
			int[] selectedIndex = table.getSelectedRows();
			//Remove accent from them.
			subtitleFile.accentRepair(selectedIndex);
			result = true;
			updateTableAndActions();
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeAccentRepair"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Class HtmlRepairAction.
	 * <br><b>Summary:</b><br>
	 * The HtmlRepair Action
	 */
	class HtmlRepairAction extends SearsUndoAction {
		private static final long serialVersionUID = 2595376205493266568L;

		public HtmlRepairAction() {
			super("HtmlRepair", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return htmlRepairAction();
		}
	}

	/**
	 * Method htmlRepairAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to remove 
	 * the htmls.
	 * @return true if action succeeded
	 */
	public boolean htmlRepairAction() {
		boolean result = false;
		//Must have opened a subtitle file to repair.
		if (subtitleFile != null) {
			//get the selected rows.
			int[] selectedIndex = table.getSelectedRows();
			//Remove html tags from them.
			subtitleFile.htmlRepair(selectedIndex);
			result = true;
			updateTableAndActions();
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeHTMLRepair"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Class SelectVideoAction.
	 * <br><b>Summary:</b><br>
	 * The SelectVideo Action
	 */
	class SelectVideoAction extends SearsAction {
		private static final long serialVersionUID = 3081640943800843335L;

		public SelectVideoAction() {
			super("SelectVideo");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			selectVideoAction();
		}
	}

	/**
	 * Method selectVideoAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to select the video file. 
	 */
	public void selectVideoAction() {
		File file = showVideoBrowser();
		if (file != null) {
			playerVideoFile = file.getAbsolutePath();
			SearsProperties.setProperty(SearsProperties.LAST_FOLDER, file.getParent());
			stopAction();
			playAction();
		}
	}

	/**
	 * Method constructPlayer.
	 * <br><b>Summary:</b><br>
	 * This method construct a new player, in function of the one selected in the properties.
	 * @return
	 */
	private PlayerInterface constructPlayer() throws PlayerException {
		//The result of the method.
		PlayerInterface result = null;
		//First is to get the selected player.
		try {
			int selectedPlayer = Integer.parseInt(SearsProperties.getProperty(SearsProperties.PLAYER_SELECTED));
			//Construct the player that correspond to the new selected one.
			switch (selectedPlayer) {
			case SearsProperties.KEY_VLC:
				result = new VLCPlayer();
				break;
			case SearsProperties.KEY_OTHER:
				result = new DefaultPlayer();
				break;
			}
		} catch (NumberFormatException e) {
			//There is an error in the player selected.
			throw new PlayerException("Selected player type is invalid:"
					+ SearsProperties.getProperty(SearsProperties.PLAYER_SELECTED));
		}
		//return the result.
		return result;
	}

	/**
	 * Class ChainRepairAction. <br>
	 * <b>Summary:</b><br>
	 * The ChainRepair Action
	 */
	class ChainRepairAction extends SearsUndoAction {
		private static final long serialVersionUID = 1973498050830247049L;

		public ChainRepairAction() {
			super("ChainRepair", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return chainRepairAction();
		}
	}

	/**
	 * Method chainRepairAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to chain repair.
	 * It will apply several repair action.
	 * @return boolean true if action succeeded
	 */
	public boolean chainRepairAction() {
		boolean result = false;
		//Must have opened a subtitle file to repair.
		if (subtitleFile != null) {
			//Show the chain repair dialog
			JDialogChainRepair cr = new JDialogChainRepair();
			cr.setVisible(true);
			//Get the result.
			boolean[] dialogResult = cr.getResult();
			//Apply repair action in function of the result.
			//The accent repair
			if (dialogResult[0]) {
				subtitleFile.accentRepair(null);
			}
			//The HTML repair
			if (dialogResult[1]) {
				subtitleFile.htmlRepair(null);
			}
			//The orderRepair
			if (dialogResult[2]) {
				subtitleFile.orderRepair();
			}
			//The Time Repair
			if (dialogResult[3]) {
				subtitleFile.timeRepair();
			}
			result = true;
			updateTableAndActions();
		} else {
			//Must have opened a subtitle file to apply a chain repair.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeChainRepair"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Class TimeRepairAction.
	 * <br><b>Summary:</b><br>
	 * The TimeRepair Action
	 */
	class TimeRepairAction extends SearsUndoAction {
		private static final long serialVersionUID = 1635491769577703287L;

		public TimeRepairAction() {
			super("TimeRepair", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return timeRepairAction();
		}
	}

	/**
	 * Class NormalizeDurationAction.
	 * <br><b>Summary:</b><br>
	 * The NormalizeDuration Action
	 */
	class NormalizeDurationAction extends SearsUndoAction {

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = -7636895212998797145L;

		public NormalizeDurationAction() {
			super("NormalizeDuration", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return normalizeDurationAction();
		}
		
	}


	/**
	 * Method getSubtitleFile.
	 * <br><b>Summary:</b><br>
	 * Return the subtitleFile.
	 * @return the subtitleFile
	 */
	public SubtitleFile getSubtitleFile() {
		return subtitleFile;
	}

	/**
	 * Method NormalizeDurationAction.
	 * <br><b>Summary:</b><br>
	 * This permit to normalize the ST duration.
	 * @return boolean	true if action succeeded
	 */
	private boolean normalizeDurationAction() {
		boolean result = false;
		//Must have opened a subtitle file to apply a delay.
		if (subtitleFile != null) {
			//get the selected rows to delay.
			int[] indexToNormalize = table.getSelectedRows();
			//ask the user for delay to apply
			JDialogNormalizeAction jd = new JDialogNormalizeAction();
			jd.setVisible(true);
			//if user choose a delay.
			if (jd.hasBeenValidated()) {
				double[] times = jd.getTimes();
				//delay the subtitle with the given delay.
				//multiply by 1000, because time is indicated in second, with possible milliseconds.
				subtitleFile.normalizeDuration(indexToNormalize, (int) (times[0] * 1000), (int) (times[1] * 1000));
				result = true;
				updateTableAndActions();
			}
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeNormalize"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Class OptionsAction.
	 * <br><b>Summary:</b><br>
	 * The Options Action
	 */
	class OptionsAction extends SearsAction {


		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = -4333319058198937067L;

		public OptionsAction() {
			super("Options");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			optionsAction();
		}
	}

	/**
	 * Method optionsAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to change sears options.
	 * It will display the options dialog, and set the sears properties.
	 */
	public void optionsAction() {
		//Show ans options dialog.
		JDialogOptions jDialogOptions = new JDialogOptions();
		jDialogOptions.setVisible(true);
		//If options has been validated, set the new options.
		if (jDialogOptions.hasBeenValidated()) {
			//A boolean to know if a message has to be displayed.
			boolean displayRebootMessage = false;
			//set the selected locale, if changed.
			Locale selectedLocale = jDialogOptions.getSelectedLocale();
			if (!SearsProperties.getProperty(SearsProperties.LANGUAGE, "").equals(
					selectedLocale.getLanguage().toString())
					|| !SearsProperties.getProperty(SearsProperties.COUNTRY, "").equals(
							selectedLocale.getCountry().toString())) {
				setSelectedLocale(selectedLocale);
				//To see changes, you have to reboot.
				displayRebootMessage = true;
			}
			//set the look and feel if changed.
			String selectedLookAndFeel = jDialogOptions.getSelectedLookAndFeel();
			if (!selectedLookAndFeel.equals(SearsProperties.getProperty(SearsProperties.LOOK_AND_FEEL, ""))) {
				setLookAndFeel(selectedLookAndFeel);
				//To see changes, you have to reboot.
				displayRebootMessage = true;
			}
			//set the player if changed.
			String selectedPlayerPath = jDialogOptions.getConfiguredPlayerPath();
			int selectedPlayer = jDialogOptions.getSelectedPlayer();
			if (!selectedPlayerPath.equals(SearsProperties.getProperty(SearsProperties.PLAYER_FULL_PATH, ""))
					|| !("" + selectedPlayer).equals(SearsProperties.getProperty(SearsProperties.PLAYER_SELECTED, ""))) {
				setPlayer(selectedPlayerPath, selectedPlayer);
			}
			//Set the DOS line separator options, if changed.
			String selectedDOSLineSeparator = jDialogOptions.getSelectedDOSLineSeparator();
			if (!selectedDOSLineSeparator.equals(SearsProperties.getProperty(SearsProperties.DOS_LINE_SEPARATOR, ""))) {
				SearsProperties.setProperty(SearsProperties.DOS_LINE_SEPARATOR, selectedDOSLineSeparator);
			}
			//Set the check for update option, if changed.
			boolean selectedCheckForUpdates = jDialogOptions.getSelectedCheckForUpdates();
			if (selectedCheckForUpdates
					&& SearsProperties.getProperty(SearsProperties.UPDATE_ADDRESS, "firstLaunch").equals("")) {
				//If user checks, and no address has been set in properties.
				//then set the default one.
				SearsProperties.setProperty(SearsProperties.UPDATE_ADDRESS, Updater.DEFAULT_UPDATE_ADDRESS);
			} else if (!selectedCheckForUpdates) {
				//Else if user does not want to check for updates, set to "".
				SearsProperties.setProperty(SearsProperties.UPDATE_ADDRESS, "");
			}
			//Set the VLC restart options, if changed.
			String selectedVlcRestart = jDialogOptions.getSelectedVlcRestart();
			if (!selectedVlcRestart.equals(SearsProperties.getProperty(SearsProperties.VLC_RESTART,
					VLCPlayer.DEFAULT_VLC_RESTART))) {
				SearsProperties.setProperty(SearsProperties.VLC_RESTART, selectedVlcRestart);
			}
			//Set the icon set option, if changed.
			String selectedIconSet = jDialogOptions.getSelectedIconSet();
			if (selectedIconSet != null && !selectedIconSet.equals(SearsProperties.getProperty(SearsProperties.ICON_SET))) {
				SearsProperties.setProperty(SearsProperties.ICON_SET, selectedIconSet);
				displayRebootMessage = true;
			}
			// if a reset properties operation has been called:
			if (getResetFlag()) {
				displayRebootMessage = true;
			}
			if (displayRebootMessage) {
				// if langage change is request, the text in the dialog is
				// in the new langage choised. 
				JOptionPane.showMessageDialog (
						instance,
						SearsResourceBundle.getResource("warning_rebootToSeeChanges"));
			}

		}
	}

	/**
	 * Class AboutAction.
	 * <br><b>Summary:</b><br>
	 * The About Action
	 */
	class AboutAction extends SearsAction {
		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = 2988713944030176952L;

		public AboutAction() {
			super("About");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			aboutAction();
		}
	}

	/**
	 * Method dialogAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to see the about dialog window.
	 * It will display the about dialog..
	 */
	public void aboutAction() {
		// we create a new JDialogAbout and set it visible:
		JDialogAbout jDialogAbout = new JDialogAbout("about");
		jDialogAbout.configureSize();
		jDialogAbout.setVisible(true);
	}

	/**
	 * Class GotoSubAction.
	 * <br><b>Summary:</b><br>
	 * The GotoSub Action
	 */
	class GotoSubAction extends SearsAction {
		private static final long serialVersionUID = 4897317673745710990L;

		public GotoSubAction() {
			super("GotoSub");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			gotoSubAction();
		}
	}

	/**
	 * Class MagicResynchroAction.
	 * <br><b>Summary:</b><br>
	 * The MagicResynchro
	 */
	class MagicResynchroAction extends SearsUndoAction {

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = 7188578370344829732L;

		public MagicResynchroAction() {
			super("MagicResynchro", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			return magicResynchroAction();
		}
	}

	/**
	 * Class RemoveAllAnchorsAction.
	 * <br><b>Summary:</b><br>
	 * The RemoveAllAnchorsAction
	 */
	class RemoveAllAnchorsAction extends SearsAction {

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = 2484034561626602996L;

		public RemoveAllAnchorsAction() {
			super("RemoveAllAnchors");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			removeAllAnchorsAction();
		}
	}

	/**
	 * Method removeAllAnchorsAction.
	 * <br><b>Summary:</b><br>
	 * Call this method to remove all the anchors from subtitles.
	 */
	private void removeAllAnchorsAction() {
		if(subtitleList != null){
			//Parse all the subtitles, and remove anchor from them.
			for(Subtitle subtitle : subtitleList){
				subtitle.unanchor();
			}
			updateActionStatus();
			//update table.
			updateWholeTable();
		}
	}

	/**
	 * Class FindAction.
	 * <br><b>Summary:</b><br>
	 * The search perform
	 */
	class FindAction extends SearsAction {		

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = 5092954733041300116L;

		public FindAction() {
			super("Find");
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			if( table.getCellEditor() != null ) {
				table.getCellEditor().cancelCellEditing();
			}
			new JDialogFind(instance).setVisible(true);
		}
	}

	/**
	 * indicate to the find module that the table changed
	 */
	private void tableChangedSinceTheLastSearch() {
		if( fm != null ) {
			fm.fireViewChange();
		}
	}

	/**
	 * find a word or a sub string in the subtitle file
	 * @param dialog	the dialog which calls the method
	 * @param direction	the direction of the search (<tt>false</tt>: forward, <tt>true</tt>: backward)
	 */
	public void find(FindDialog dialog, boolean direction) {
		//long start = System.nanoTime();
		fm.fire(dialog, subtitleList, direction);
		//System.out.println("Find, time of execution: " + (System.nanoTime() - start) + " ns");
	}

	public void fireFindStop() {
		if( fm != null ) {
			fm.cancel();
		}
	}


	/**
	 * MixAction
	 * this permits to Mix two subtitle files, inserting the subtitle at the right place.
	 */
	class MixAction extends SearsUndoAction {

		private static final long serialVersionUID = 7753043190773883390L;

		public MixAction() {
			super("Mix", undoManager);
		}

		@Override
		public boolean doUndoableAction(ActionEvent e) {
			// TODO Auto-generated method stub
			return mixAction();	
		}		
	}

	/**
	 * Method mixAction.
	 * <br><b>Summary:</b><br>
	 * This action mix 2 subtitle files.
	 * @return  (<b>boolean</b>)   true if action succeeded.
	 */
	private boolean mixAction() {
		boolean result = false;
		// we must have an opened subtitle file on Sears:
		if (subtitleFile != null) {
			// we display the mix dialog:
			JDialogMixSubtitleFile mixDialog = new JDialogMixSubtitleFile(
					subtitleList.size());
			mixDialog.setLocationRelativeTo(this);
			mixDialog.setVisible(true);

			// if action performed:
			if(mixDialog.validationStatus) {
				SubtitleFile subForTheMix = fsa.openFile(
						new File(mixDialog.getJTextFieldFileToMix().getText()),
						new ArrayList<Subtitle>(), null);

				subtitleFile.mixWithAnotherSubtitleFile(
						subForTheMix, mixDialog.keepSubtitles());
				result = true;
				updateTableAndActions();
			}
		}else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeMix"), Trace.WARNING_PRIORITY);
		}
		return result;
	}


	/**
	 * Method gotoSubAction.
	 * <br><b>Summary:</b><br>
	 * This method permits to set the video position to the selected subtitle.
	 */
	public void gotoSubAction() {
		//First is to get the selected subtitle.
		int selectedRow = table.getSelectedRow();
		//If selected and player is not null, go to this subtitle.
		if(player != null && selectedRow != -1){
			Subtitle selectedSubtitle = (Subtitle) subtitleList.get(selectedRow);
			if(selectedSubtitle != null){
				try {
					//Divide by 1000, because start date is in milliseconds.
					player.setPosition(selectedSubtitle.getStartDate()/1000);
				} catch (PlayerException e) {
					Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
				}
			}
		}
	}

	/**
	 * Method setPlayer.
	 * <br><b>Summary:</b><br>
	 * Set the player.
	 * @param selectedPlayerPath    The player full path to set.
	 * @param selectedPlayer        The player to set.
	 */
	private void setPlayer(String selectedPlayerPath, int selectedPlayer) {
		SearsProperties.setProperty(SearsProperties.PLAYER_FULL_PATH, selectedPlayerPath);
		SearsProperties.setProperty(SearsProperties.PLAYER_SELECTED, "" + selectedPlayer);
		if (player != null) {
			//quit previous player if exist.
			player.quit();
			//disable the player
			player = null;
		}
		//construct the new player
		try {
			player = constructPlayer();
		} catch (PlayerException e) {
			e.printStackTrace();
		}
	}


	/**
	 * Method setSelectedLocale.
	 * <br><b>Summary:</b><br>
	 * Use this to set the locale.
	 * @param selectedLocale        The locale to set.
	 */
	private void setSelectedLocale(Locale selectedLocale) {
		SearsResourceBundle.setLocale(selectedLocale);
		SearsProperties.setProperty(SearsProperties.LANGUAGE, selectedLocale.getLanguage());
		SearsProperties.setProperty(SearsProperties.COUNTRY, selectedLocale.getCountry());
		setLocale(selectedLocale);
	}

	/**
	 * Method setLookAndFeel.
	 * <br><b>Summary:</b><br>
	 * Use this method to sets the look and feel.
	 * @param selectedLookAndFeel       The look and feel to set.
	 */
	private void setLookAndFeel(String selectedLookAndFeel) {
		SearsProperties.setProperty(SearsProperties.LOOK_AND_FEEL, selectedLookAndFeel);
	}

	/**
	 * Method timeRepairAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to time repair.
	 * It will correct the time superposition problem.
	 * When subtitle ends after next ST start time.
	 * @return boolean true if action succeeded
	 */
	public boolean timeRepairAction() {
		boolean result = false;
		//Must have opened a subtitle file to repair.
		if (subtitleFile != null) {
			//Apply Time repair
			subtitleFile.timeRepair();
			result = true;
			updateTableAndActions();
		} else {
			//Must have opened a subtitle file to apply a delay.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeTimeRepair"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Method splitAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to split STs file in two files.
	 * Must have selected the ST to split from.
	 * @return boolean true if action succeed.
	 */
	public boolean splitAction() {
		boolean result = false;
		//Must have opened a subtitle.
		if (subtitleFile != null) {
			int selectedIndex = table.getSelectedRow();
			if (selectedIndex == -1) {
				selectedIndex = 0;
			}
			//show Split dialog.
			JDialogSplit splitDialog = new JDialogSplit(subtitleList, selectedIndex, subtitleFile);
			splitDialog.setLocationRelativeTo(this);
			splitDialog.setVisible(true);
			if (splitDialog.hasBeenValidated()) {
				//Get the destination files.
				File[] destinationFiles = splitDialog.getDestinationsFiles();
				//The the second part delay
				double secondPartDelay = splitDialog.getSecondPartDelay();
				//Look for open second part option.
				boolean openSecondPart = splitDialog.needToOpenSecondPart();
				//Get the choosen subtitle for split.
				int subtitleIndex = splitDialog.getChoosenSubtitle();
				//Apply spliting to subtitleFile.
				//multiply by 1000, because delay is indicated in second, with possible milliseconds.
				SubtitleFile[] parts = subtitleFile.split(destinationFiles, subtitleIndex,
						(int) (secondPartDelay * 1000));
				//At the end save the two files.
				Trace.trace("Saving splitted files", Trace.ALGO_PRIORITY);
				try {
					parts[0].writeToFile(parts[0].getFile());
					parts[1].writeToFile(parts[1].getFile());
				} catch (FileConversionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				//If need to open second part, open it !
				if (openSecondPart) {
					openFile(parts[1].getFile());
				}
			}
			result = true;
			updateWindowsName();
			updateActionStatus();
		} else {
			//Must have opened a subtitle file to apply a split.
			Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeSplit"), Trace.WARNING_PRIORITY);
		}
		return result;
	}

	/**
	 * Method appendAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to make an append Action.
	 * @return result	true if action succeeded.
	 */
	public boolean appendAction() {
		boolean result = false;
		//Must have opened a subtitle.
		if (subtitleFile != null) {
			//show Append dialog.
			JDialogAppend appendDialog = new JDialogAppend();
			appendDialog.setVisible(true);
			if (appendDialog.hasBeenValidated()) {
				File fileToAppend = appendDialog.getFileToAppend();
				double delay = appendDialog.getDelay();
				if (fileToAppend != null) {
					// instantiate subtileFile
					SubtitleFile subtitleFileToAppend = null;
					subtitleFileToAppend = fsa.openFile(fileToAppend, null);
					if( subtitleFileToAppend != null ) {
						// Append file to current one.
						// multiply by 1000, because delay is indicated in second, with possible milliseconds.
						subtitleFile.append(subtitleFileToAppend, (int) (delay * 1000));
						//indicate table that data have changed
						result = true;
						updateTableAndActions();
					} // else do nothing
				}
			} else {
				//Must have opened a subtitle file to apply an append.
				Trace.trace(SearsResourceBundle.getResource("warning_mustOpenBeforeAppend"), Trace.WARNING_PRIORITY);
			}
		}
		return result;
	}

	/**
	 * Method <b>watchForUnsavedFile</b>
	 * <br><b>Summary:</b><br>
	 * This method check that current file (if open) does not have unsaved modifications.
	 * If it has, ask for user to "save", "don't save" or "cancel".
	 * If user click on save, current file will be saved, method returns true.
	 * If user click on save, current file is not saved, method returns true.
	 * if user click on cancel, file is not saved, method returns false.
	 * Parameters:
	 * @return  <b>boolean</b>      True if user wants to continue (click on "save" or "don't save")
	 *                              , or if file has no modifications. false if user clicked on "cancel"
	 * 
	 */
	private boolean checkUnsavedFileBeforeAction() {
		//The result of the method
		boolean result = true;
		//If there already is an unsaved open file, warn the user before opening.
		if (subtitleFile != null && subtitleFile.isFileChanged()) {
			result = false;
			//File is not saved, ask for saving.
			String message = SearsResourceBundle.getResource("warning_fileNotSaved");
			//Custom button text
			Object[] options = { SearsResourceBundle.getResource("button_save"),
					SearsResourceBundle.getResource("button_dontSave"),
					SearsResourceBundle.getResource("button_cancel") };
			int userChoice = JOptionPane.showOptionDialog(instance, message, SearsResourceBundle
					.getResource("warning_fileNotSaved"), JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, options, options[2]);
			//If user choose to save file, save it, and return true.
			if (userChoice == 0) {
				saveAction();
				result = true;
			} else if (userChoice == 1) {
				//if user choosed don't save, just return true
				result = true;
			}//else return false.
		}
		//return the result
		return result;
	}

	/**
	 * Method updateWindowsName.
	 * <br><b>Summary:</b><br>
	 * Update the main windows name.
	 */
	private void updateWindowsName() {
		String title = SearsResources.getString("SearsGUITitle")+Version.VERSION;
		if (subtitleFile != null) {
			title += " - " + subtitleFile.getCharset() + " - " + subtitleFile.getFile().getName();
			if (subtitleFile.isFileChanged()) {
				title += "*";
			}
		}
		setTitle(title);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
		maybeShowPopup(e);
		mayBeAnchor(e);
	}

	/**
	 * Method mayBeAnchor.
	 * <br><b>Summary:</b><br>
	 * This method is called whn a click is detected.
	 * It permits to check an anchor event.
	 * @param e		The mouseEvent that called for this mayBeAnchor.
	 */
	private void mayBeAnchor(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1 && e.getComponent() == table) {
			//click must be on 4rth column.
			Point clickPoint = new Point(e.getX(), e.getY());
			if(table.columnAtPoint(clickPoint) == SubtitleTableModel.ANCHOR_COLUMN){
				//retrieve the clicked row.
				int row = table.rowAtPoint(clickPoint);
				anchorSubtitle(row);
			}
		}

	}

	/**
	 * Method anchorSubtitle.
	 * <br><b>Summary:</b><br>
	 * This method permits to anchor/un-anchor the given index subtitle.
	 * @param subtitleIndex		The subtitle index to anchor.
	 */
	private void anchorSubtitle(int row) {
		//Retrieve current playing time.
		if (player != null) {
			try {
				// Retrieve the given subtitle.
				Subtitle subtitle = subtitleList.get(row);
				if (subtitle.isAnchored()) {
					subtitle.unanchor();
				} else {
					int currentPosition = player.getPosition();
					subtitle.anchor(currentPosition);
				}
				//Indicate to table, that data changed.
				updateWholeTable();
				//Update actions
				updateActionStatus();
			} catch (PlayerException e) {
				e.printStackTrace();
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
		maybeShowPopup(e);
	}

	/**
	 * Method maybeShowPopup.
	 * <br><b>Summary:</b><br>
	 * Show the popup menu if necessary
	 * @param e The mouse event.
	 */
	private void maybeShowPopup(MouseEvent e) {
		// emulate the right click on Mac (crtl down and click):
		boolean macCondition = false;
		// if we are running on Mac OS X system:
		if(Utils.isMacPlatform) {
			macCondition = (e.getButton() == MouseEvent.BUTTON1);
			macCondition = macCondition && (e.isControlDown());
		}	

		// Check wether it is a right button, or a system pop-up trigger, 
		// or an emulation of a right click (Mac OS X only).
		if (e.getButton() == MouseEvent.BUTTON3 || e.isPopupTrigger() || macCondition) {			
			getJPopUpMenu().show(e.getComponent(), e.getX(), e.getY());
			table.changeSelection(table.rowAtPoint(new Point(e.getX(), e.getY())),
					0, false, false);
		}
	}

	/**
	 * Method getJPopUpMenu.
	 * <br><b>Summary:</b><br>
	 * Construct/return the JPopUpMenu.
	 * @return  <b>JPopupMenu</b>   The Sears JPopupMenu.
	 */
	private JPopupMenu getJPopUpMenu() {
		if (jPopupMenu == null) {
			jPopupMenu = new JPopupMenu("Sears");
			//set border for a better display:
			jPopupMenu.setBorder(new EmptyBorder(4,0,4,0));
			//////////////////
			//ACTION MENU
			/////////////////
			//DELAY ITEM
			jPopupMenu.add(getAction(ACTION_KEY_DELAY));
			//RESYNCHRO ITEM
			jPopupMenu.add(getAction(ACTION_KEY_RESYNCH));
			//Add a separator.
			jPopupMenu.addSeparator();
			//SPLIT ITEM
			jPopupMenu.add(getAction(ACTION_KEY_SPLIT));
			//APPEND ITEM
			jPopupMenu.add(getAction(ACTION_KEY_APPEND));
			//Add a separator.
			jPopupMenu.addSeparator();
			//GOTO SUB
			jPopupMenu.add(getAction(ACTION_KEY_GOTO_SUB));
		}
		return jPopupMenu;
	}

	/**
	 * Method <b>fileChanged</b>
	 * <br><b>Summary:</b><br>
	 * Set the fileChanged status flag to true.
	 */
	public void fileChanged() {
		if (subtitleFile != null) {
			subtitleFile.fileChanged();
			tableChangedSinceTheLastSearch();
		}
		updateWindowsName();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
		updateActionStatus();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
		dispose();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		quitAction();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
	}


	/**
	 * Method updateActionStatus.
	 * <br><b>Summary:</b><br>
	 * This method permits to update the action enabling status
	 */
	private void updateActionStatus(){
		//File dependant Actions.
		boolean enableFileDependant = false;
		if(subtitleFile != null){
			enableFileDependant = true;		
		}
		for (int i = 0; i < FILE_OPEN_DEPENDANT_ACTIONS.length; i++) {
			actions.get(FILE_OPEN_DEPENDANT_ACTIONS[i]).setEnabled(enableFileDependant);
		}
		//File changed dependant.
		boolean enableFileChangedDependant = false;
		if(subtitleFile != null && subtitleFile.isFileChanged()){
			enableFileChangedDependant = true;
		}
		for (int i = 0; i < FILE_CHANGED_DEPENDANT_ACTIONS.length; i++) {
			actions.get(FILE_CHANGED_DEPENDANT_ACTIONS[i]).setEnabled(enableFileChangedDependant);
		}
		//Then update actions, that depends on anchors number.
		int anchorsNumber = getAnchorsNumber();
		if(anchorsNumber == 0){
			actions.get(ACTION_KEY_MAGIC_RESYNCHRO).setEnabled(false);
			actions.get(ACTION_KEY_REMOVE_ALL_ANCHORS).setEnabled(false);
		}else if (anchorsNumber < 2){
			//need at least 2 anchors to apply a magic resynchro
			actions.get(ACTION_KEY_MAGIC_RESYNCHRO).setEnabled(false);
			actions.get(ACTION_KEY_REMOVE_ALL_ANCHORS).setEnabled(true);
		}else {
			actions.get(ACTION_KEY_MAGIC_RESYNCHRO).setEnabled(true);
			actions.get(ACTION_KEY_REMOVE_ALL_ANCHORS).setEnabled(true);
		}
	}

	/**
	 * Method getAnchorsNumber.
	 * <br><b>Summary:</b><br>
	 * This method returns the number of defined anchors.
	 * @return  (<b>int</b>)   The number of defined anchors.
	 */
	private int getAnchorsNumber() {
		//the result of the method.
		int result = 0;
		if(subtitleList != null){
			for(Subtitle subtitle : subtitleList){
				if(subtitle.isAnchored()){
					result ++;
				}
			}
		}
		//return the result.
		return result;
	}

	protected static void setResetFlag(boolean resetOnExit) {
		resetFlag = resetOnExit;
	}

	/**
	 * Class SubtitleFileTransfertHandler.
	 * Summary:
	 * This TransferHanlder check for subtitle file drag and drop.
	 * It permits to open Subtitle file that are dragged from OS's location, such as 
	 * Navigator.
	 */
	protected class SubtitleFileTransfertHandler extends TransferHandler {
		private static final long serialVersionUID = 5833961095020955565L;

		public SubtitleFileTransfertHandler() {
			super();
		}

		/* (non-Javadoc)
		 * @see javax.swing.TransferHandler#canImport(javax.swing.JComponent, java.awt.datatransfer.DataFlavor[])
		 */
		public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
			//Check import available.
			//The result of the method.
			boolean result = false;
			//Parse the array.
			for (int i = 0; i < transferFlavors.length; i++) {
				//retrieve current element.
				DataFlavor dataFlavor = transferFlavors[i];
				//We could only import FileList flavor.
				//Which are files to be opened by Sears.
				if (dataFlavor.isFlavorJavaFileListType()) {
					result = true;
				}
			}
			//return the result.
			return result;
		}

		/* (non-Javadoc)
		 * @see javax.swing.TransferHandler#importData(javax.swing.JComponent, java.awt.datatransfer.Transferable)
		 */
		@SuppressWarnings("unchecked")
		public boolean importData(JComponent comp, Transferable t) {
			//the result of the method.
			boolean result = false;
			try {
				Object list = t.getTransferData(DataFlavor.javaFileListFlavor);
				if (list instanceof List && ((List<File>) list).size() > 0) {
					//Get the first file of the list, and open it.
					File fileToOpen = ((List<File>) list).get(0);
					instance.openFile(fileToOpen);
					result = true;
				}
			} catch (UnsupportedFlavorException exception) {
				// TODO Auto-generated catch block
				exception.printStackTrace();
			} catch (IOException exception) {
				// TODO Auto-generated catch block
				exception.printStackTrace();
			}
			//return the result
			return result;
		}
	}

	/**
	 * Class PlayerTimeHandler.
	 * Summary:
	 * This PlayerTimeHandler check for video length and position.
	 */
	private class PlayerTimeHandler implements Runnable, ChangeListener {
		private boolean jSliderIsMoving  = false;
		private boolean jSliderIsUpdating  = false;
		private long updateDate = 0;
		private int currentPosition = 0;
		private int oldPosition = 0;
		private int subtitleFocusIndex = -1;
		private boolean run = true;

		/* (non-Javadoc)
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			int previousOffset = -1;
			int previousLength = -1;
			while (run) {
				try {
					Thread.sleep(200);
					if(player != null) {
						// get the position and length of video
						int currentOffset = player.getPosition();
						if(currentOffset == -1) 
							currentOffset = 0;
						int videoLength = player.getLength();
						if(videoLength == -1) 
							videoLength = 0;
						// update only if it is not the same position and length
						if(videoLength != previousLength) {
							previousLength = videoLength;
							//
							// Update the JSlider bar
							jSliderIsUpdating = true;
							jPanelVideoControler.setJSliderSearchLength(videoLength);
							jSliderIsUpdating = false;
						}
						if(currentOffset != previousOffset) {
							previousOffset = currentOffset;
							//
							// Update the video position label and slider bar.
							if(jSliderIsMoving == false) {
								jSliderIsUpdating = true;
								jPanelVideoControler.setJSliderSearchPosition(currentOffset);
								jSliderIsUpdating = false;
							}
							//
							// Update the current subtitle focus
							if((subtitleFile != null) && (jPanelVideoControler.jCheckBoxTrackModeIsSelected())) {
								Subtitle activeSubtitle = subtitleFile.getSubtitleAtDate((currentOffset+1)*1000);
								final int row = activeSubtitle.getNumber() - 1;
								if(subtitleFocusIndex !=  row) {
									subtitleFocusIndex = row;
									SwingUtilities.invokeLater(new Runnable(){
										public void run() {
											table.changeSelection(row, 0, false, false);
											table.changeSelection(row+4, 0, true, true);
										}
									});
								}
							}
						}
						// Update video according to the JSlider
						if(jSliderIsMoving == true) {
							Date date = new Date();
							if(((date.getTime() - updateDate) > 1000) && (oldPosition != currentPosition)) {
								// set the new reference date
								updateDate = date.getTime();
								oldPosition = currentPosition;
								// Set the new position
								try {
									player.setPosition(oldPosition);
								} catch (Exception e) {
									Trace.trace("PlayerTimeHandler failed cannot set video position:"+e.getMessage());
								}
							}
						}
					}
				} catch (Exception e) {
					Trace.trace("PlayerTimeHandler failed:"+e.getMessage());
				}
			}
		}

		/* (non-Javadoc)
		 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
		 */
		public void stateChanged(ChangeEvent changeEvent) {
			JSlider source = (JSlider)changeEvent.getSource();
			// get the current slider position
			currentPosition = (int)source.getValue();
			jSliderIsMoving = true;
			// slider stops moving
			if (!source.getValueIsAdjusting()) {
				jSliderIsMoving = false;
				if((jSliderIsUpdating == false) && (oldPosition != currentPosition)) {
					// Set the new position
					try {
						player.setPosition(currentPosition);
					} catch (Exception e) {
						Trace.trace("PlayerTimeHandler failed cannot set video position:"+e.getMessage());
					}
				}
			}
			// update the video time
			jPanelVideoControler.setJLabelTime(Utils.formatTime(currentPosition));
		}
	}

	public static boolean getResetFlag() {
		return resetFlag;
	}

	public void finalize() {
		try {
			super.finalize();
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}