/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is availbale under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.tools.SearsResourceBundle;

/**
 * Class ResynchroDialog.
 * <br><b>Summary:</b><br>
 * The GUI to do a resynchro
 */
public class ResynchroDialog extends SearsJDialog {
    private static final long serialVersionUID = 7702754672054592460L;

    private JPanel jContentPane = null;

    private JPanel mainPanel = null;

    private JLabel[] jLabelSource = null;

    private JTextField[] jTextSource = null;

    private JLabel[] jLabelDestination = null;

    private JTextField[] jTextDestination = null;
    
    private JSlider[] jSliders = null;
    
    private JLabel[] jLabelSubtitles = null;

    /**The sources, if precised at Dialog construction*/
    private int[] sources;
    
    /**The subtitleList, that contains all the subtitles.*/
    private ArrayList<Subtitle> subtitleList;

    /**
     * Constructor ResynchroDialog.
     * <br><b>Summary:</b><br>
     * The constructor of the class ResynchroDialog
     * Construct a new Resynchro dialog, using the given sources.
     * @param subtitleList		The subtitles.
     * @param sources			The index of the selected subtitles for the resynchro.
     */
    public ResynchroDialog(ArrayList<Subtitle> subtitleList, int[] sources) {
        super(SearsResourceBundle.getResource("resynch_title"));
        this.sources = sources;
        this.subtitleList = subtitleList;
        //construct the source/Destination arrays using sources size.
        jLabelSource = new JLabel[sources.length];
        jLabelDestination = new JLabel[sources.length];
        jTextSource = new JTextField[sources.length];
        jTextDestination = new JTextField[sources.length];
        jSliders = new JSlider[sources.length];
        jLabelSubtitles = new JLabel[sources.length];
        initialize();
        configureSize();
    }

    /**
     * Constructor ResynchroDialog.
     * <br><b>Summary:</b><br>
     * The constructor of the class ResynchroDialog
     * @param subtitleList	The subtitles.
     */
    public ResynchroDialog(ArrayList<Subtitle> subtitleList) {
        this(subtitleList, new int[]{0, 0});
    }

    /**
     * This method initializes this
     */
    protected void initialize() {
        this.setContentPane(getJContentPane());
        setLocationRelativeTo(getParent());   
        validationStatus=false;
    }

    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.add(getMainPanel(), java.awt.BorderLayout.CENTER);
            jContentPane.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
        }
        return jContentPane;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getMainPanel() {
        if (mainPanel == null) {
            mainPanel = new JPanel();
            mainPanel.setLayout(new GridLayout(sources.length, 1));
            mainPanel.setBorder(super.createEmptyBorder());
            for(int i = 0; i < sources.length; i++){
            	mainPanel.add(getJPanelSubtitle(i));
            }
        }
        return mainPanel;
    }

    /**
     * Method getJPanelSubtitle.
     * <br><b>Summary:</b><br>
     * This method returns the JPanel that is used to define the subtitle of the given index.
     * @param index					The index of the resynchro subtitle.
     * @return  (<b>JPanel</b>)   the JPanel that is used to define the subtitle of the given index.
     */
    private JPanel getJPanelSubtitle(int index) {
    	//The result of the method.
    	JPanel result = new JPanel();
    	result.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "["+index+"]"));
    	result.setLayout(new GridBagLayout());
    	final int myIndex = index;
    	//Now construct the panel content.
    	jTextSource[index] = new JTextField("00:00:00,000");
    	jLabelSource[index] = new JLabel(SearsResourceBundle.getResource("resynch_source")+" "+index+":");
    	jTextDestination[index] = new JTextField("00:00:00,000");
    	//change the destination value if subttitle is anchored.
    	Subtitle subtitle = subtitleList.get(sources[index]);
    	if(subtitle.isAnchored()){
    		jTextDestination[index].setText(SubtitleFile.timeToString(subtitle.getAnchor()));
    	}
    	jLabelDestination[index] = new JLabel(SearsResourceBundle.getResource("resynch_destination")+" "+index+":");
    	//Construct the JSlider, current value is the selected source index.
    	jLabelSubtitles[index] = new JLabel(((Subtitle) subtitleList.get(sources[index])).getSubtitle());
    	jSliders[index] = new JSlider(0, subtitleList.size()-1);
    	jSliders[index].addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				sliderValueChanged(myIndex);
			}
		});
    	jSliders[index].setValue(sources[index]);
    	//Construct a button that will permit to copy source to destination.
    	JButton jButtonCopy = new JButton("=>"); 
    	jButtonCopy.setToolTipText(SearsResourceBundle.getResource("resynch_copy"));
    	jButtonCopy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				copyAction(myIndex);
			}
		});
    	
    	//Add the component in the panel using GridBagLayout.
    	GridBagConstraints gbc1 =  new GridBagConstraints();
    	gbc1.gridx = 0;
    	gbc1.gridy = 0;
    	gbc1.ipadx = 4;
    	result.add(jLabelSource[index], gbc1);
    	GridBagConstraints gbc2 =  new GridBagConstraints();
    	gbc2.gridx = 1;
    	gbc2.gridy = 0;
    	gbc2.fill = GridBagConstraints.HORIZONTAL;
    	result.add(jTextSource[index], gbc2);
    	GridBagConstraints gbc3 =  new GridBagConstraints();
    	gbc3.gridx = 2;
    	gbc3.gridy = 0;
    	gbc3.weightx = 1.0;
    	gbc3.anchor = GridBagConstraints.CENTER;
    	result.add(jButtonCopy, gbc3);
    	GridBagConstraints gbc4 =  new GridBagConstraints();
    	gbc4.gridx = 3;
    	gbc4.gridy = 0;
    	gbc4.ipadx = 4;
    	result.add(jLabelDestination[index], gbc4);
    	GridBagConstraints gbc5 =  new GridBagConstraints();
    	gbc5.gridx = 4;
    	gbc5.gridy = 0;
    	gbc5.fill = GridBagConstraints.HORIZONTAL;
    	result.add(jTextDestination[index], gbc5);
    	//Line 2
    	GridBagConstraints gbc6 =  new GridBagConstraints();
    	gbc6.gridx = 0;
    	gbc6.gridy = 1;
    	gbc6.weightx = 1.0;
    	gbc6.fill = GridBagConstraints.HORIZONTAL;
    	gbc6.gridwidth = 5;
    	result.add(jLabelSubtitles[index], gbc6);
    	//line3
    	GridBagConstraints gbc7 =  new GridBagConstraints();
    	gbc7.gridx = 0;
    	gbc7.gridy = 2;
    	gbc7.weightx = 1.0;
    	gbc7.fill = GridBagConstraints.HORIZONTAL;
    	gbc7.gridwidth = 5;
    	result.add(jSliders[index], gbc7);
    	//return the result.
    	return result;
	}


    /**
     * Method copyAction.
     * <br><b>Summary:</b><br>
     * This method permits to copy the value of a source to the corresponding destination.
     * @param index			The index of the source/destination.
     */
    protected void copyAction(int index) {
    	//retrieve the source value.
    	String sourceValue = jTextSource[index].getText();
    	//set it in the corresponding destination
    	jTextDestination[index].setText(sourceValue);
	}

	/**
     * Method sliderValueChanged.
     * <br><b>Summary:</b><br>
     * This method is called by the slider to indicate that their position changed.
     * @param index		The index of the slider that change
     */
    protected void sliderValueChanged(int index) {
		//A slider has moved.
    	//get the new slider's value.
    	int subtitleIndex = jSliders[index].getValue();
    	//Changed the text of the JLabelSubtitle.
    	//Get the subtitle at the given index.
        Subtitle subtitle = (Subtitle) subtitleList.get(subtitleIndex);
        //construct message to show in label.
        //Put ST number, and append the message.
        String message = "[" + subtitle.getNumber() + "]" + " " + subtitle.getSubtitle();
        //Cut long ST, not to perturb GUI.
        int maxLength = 35;
        if (message.length() > maxLength) {
            message = message.substring(0, maxLength - 1) + "...";
        }
        //set the label to the content of the subtitle.
        jLabelSubtitles[index].setText(message);
        //And set the source textField.
        jTextSource[index].setText(SubtitleFile.timeToString(subtitle.getStartDate()));
	}

	/* (non-Javadoc)
     * @see sears.gui.SearsJDialog#hasBeenValidated()
     */
    public boolean hasBeenValidated() {
        return validationStatus;
    }

    /**
     * Method getResult.
     * <br><b>Summary:</b><br>
     * This method returns the resynchro dialog result.
     * @return      <b>int[]</b>    An array of int:[source1, destination1, source2, destination2 .... etc]
     */
    public int[] getResult() {

        int[] result = new int[sources.length * 2];
        for (int i = 0; i < sources.length; i++) {
			result[2*i] = SubtitleFile.stringToTime(jTextSource[i].getText());
			result[2*i+1] = SubtitleFile.stringToTime(jTextDestination[i].getText());
		}
        return result;
    }

    /* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
    protected String getDialogName() {
        return "resynchro";
    }
}
