/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com

package sears.gui;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.border.Border;

import sears.tools.SearsProperties;
import sears.tools.SearsResourceBundle;
import sears.tools.Utils;

/**
 * Class SearsJDialog.
 * <br><b>Summary:</b><br>
 * This class defines the general comportement of the Sears JDialog.
 */
public abstract class SearsJDialog extends JDialog implements WindowListener {
	
	private static final long serialVersionUID = 1L;

	protected static int border = 4;
	
	protected JPanel jPanelButtons = null;

	protected JButton jButtonOk = null;

	protected JButton jButtonCancel = null;

	/**A boolean to know if user has validated*/
	protected boolean validationStatus;

	/**
	 * Constructor SearsJDialog.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param title     The title of the JDialog.
	 */
	public SearsJDialog(String title) {
		super(MainWindow.instance, title, true);
		//manage closing event.
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(this);
		configureSize();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JDialog#createRootPane()
	 */
	protected JRootPane createRootPane() {
		// **************************
		// ADDING KEYBOARD SHORTCUT *
		// **************************
		
		// we create a new action listener:
		ActionListener theActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent actionEvent) {
				// we close the window:
				cancelAction();
			}
		};
		
		KeyStroke stroke = null;
		// on a Mac platform:
		if(Utils.isMacPlatform) {
			stroke = KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.META_MASK);
		} else {
			stroke = KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK);
		}
		
		JRootPane rootPane = new JRootPane();
		rootPane.registerKeyboardAction(theActionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
		return rootPane;
	}

	/**
	 * Method configureSize.
	 * <br><b>Summary:</b><br>
	 * This method permits to restore dialog size, or pack it for the first launch.
	 */
	protected void configureSize() {
		//try to restaure the dimension and position
		Dimension restauredDimension = null;
		Point location = null;
		try {
			int width = Integer.parseInt(SearsProperties.getProperty(getDialogName() + SearsProperties.SUFFIX_WIDTH));
			int heigth = Integer.parseInt(SearsProperties.getProperty(getDialogName() + SearsProperties.SUFFIX_HEIGTH));
			restauredDimension = new Dimension(width, heigth);
			int posX = Integer.parseInt(SearsProperties.getProperty(getDialogName() + SearsProperties.SUFFIX_POSX));
			int posY = Integer.parseInt(SearsProperties.getProperty(getDialogName() + SearsProperties.SUFFIX_POSY));
			location = new Point(posX, posY);
			//set the size
			setSize(restauredDimension);
		} catch (NumberFormatException e) {
			//use packed size.
			pack();
		}
		//set the location
		if (location == null) {
			setLocationRelativeTo(MainWindow.instance);
		} else {
			setLocation(location);
		}
	}

	/**
	 * Method getDialogName.
	 * <br><b>Summary:</b><br>
	 * This method return the dialog name.
	 * It will be used to save the dialog dimensions in the config file.
	 * @return  String      The dialog name.
	 */
	protected abstract String getDialogName();

	/**
	 * This method initializes jButton  
	 *  
	 * @return javax.swing.JButton  
	 */
	protected JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton(SearsResourceBundle.getResource("button_cancel"));
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelAction();
				}
			});
		}
		return jButtonCancel;
	}

	/**
	 * Method cancelAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user wants to cancel split dialog.
	 */
	protected void cancelAction() {
		validationStatus = false;
		//saveDialogProperties();
		dispose();
	}

	/**
	 * This method initializes jPanel   
	 *  
	 * @return javax.swing.JPanel   
	 */
	protected JPanel getJPanelButtons() {
		if (jPanelButtons == null) {
			jPanelButtons = new JPanel();
			jPanelButtons.add(getJButtonOk(), null);
			jPanelButtons.add(getJButtonCancel(), null);
		}
		return jPanelButtons;
	}

	/**
	 * This method initializes jButton  
	 *  
	 * @return javax.swing.JButton  
	 */
	private JButton getJButtonOk() {
		if (jButtonOk == null) {
			jButtonOk = new JButton(SearsResourceBundle.getResource("button_ok"));
			jButtonOk.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					okAction();
				}
			});
		}
		return jButtonOk;
	}

	/**
	 * Method okAction.
	 * <br><b>Summary:</b><br>
	 * This method is called when user validate the dialog.
	 */
	protected void okAction() {
		//saveDialogProperties();
		validationStatus = true;
		dispose();
	}

	/**
	 * Method hasBeenValidated.
	 * <br><b>Summary:</b><br>
	 * return true if user has validated.
	 * @return  <b>boolean</b>  True if user has validated. False otherwise.
	 */
	public boolean hasBeenValidated() {
		return validationStatus;
	}

	/**
	 * Method saveDialogProperties.
	 * <br><b>Summary:</b><br>
	 * This method save the current Dialog properties in the properties.
	 */
	protected void saveDialogProperties() {
		SearsProperties.setProperty(getDialogName() + SearsProperties.SUFFIX_WIDTH, "" + getWidth());
		SearsProperties.setProperty(getDialogName() + SearsProperties.SUFFIX_HEIGTH, "" + getHeight());
		SearsProperties.setProperty(getDialogName() + SearsProperties.SUFFIX_POSX, "" + getX());
		SearsProperties.setProperty(getDialogName() + SearsProperties.SUFFIX_POSY, "" + getY());
	}
	
	/**
	 * Create empty border related to <tt>border</tt> class constant.
	 * @return an empty border
	 */
	protected static Border createEmptyBorder() {
		return BorderFactory.createEmptyBorder(border,border,0,border);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e) {
		saveDialogProperties();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e) {
		cancelAction();
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
	}

	/* (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
	}
}
