/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
//
// $Id: SubtitleTableModel.java,v 1.9 2007/08/16 09:07:21 floriaen Exp $
////////////////////////////////////////////////
package sears.gui;

import java.util.ArrayList;

import javax.swing.event.UndoableEditEvent;
import javax.swing.table.DefaultTableModel;

import sears.file.SrtFile;
import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.gui.undo.SearsSimpleUndoEdit;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Defines table data and ensure synchronisation with array of subtitles 
 */
public class SubtitleTableModel extends DefaultTableModel {
    private static final long serialVersionUID = 3262302656286772741L;

    /**The subtitle list.*/
    private ArrayList<Subtitle> subtitleList;

    /** (<b>int</b>) ANCHOR_COLUMN: The ANCHOR_COLUMN index. */
    public static final int ANCHOR_COLUMN = 0;
    /** (<b>int</b>) ID_COLUMN: The ID_COLUMN index. */
    public static final int ID_COLUMN = 1;
    /** (<b>int</b>) START_DATE_COLUMN: The START_DATE_COLUMN index. */
    public static final int START_DATE_COLUMN = 2;
    /** (<b>int</b>) END_DATE_COLUMN: The END_DATE_COLUMN index. */
    public static final int END_DATE_COLUMN = 3;
    /** (<b>int</b>) SUBTITLE_COLUMN: The SUBTITLE_COLUMN index. */
    public static final int SUBTITLE_COLUMN = 4;
    
    
    /** (<b>int</b>) COLUMN_NUMBER: The COLUMN_NUMBER */
    public static final int COLUMN_NUMBER = 5;
    
    /**
     * Constructor SubtitleTableModel.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     * @param _subtitleList     The subtitleList to display
     */
    public SubtitleTableModel(ArrayList<Subtitle> _subtitleList) {
        subtitleList = _subtitleList;
    }
    
    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount() {
        return COLUMN_NUMBER;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnName(int)
     */
    public String getColumnName(int column) {
    	//The result of the method.
    	String result = null;
    	switch (column) {
		case ANCHOR_COLUMN:
			//Will use an anchor icon instead of text.
			result = "";
			break;
		case ID_COLUMN:
			result = SearsResourceBundle.getResource("table_id");
			break;
		case START_DATE_COLUMN:
			result = SearsResourceBundle.getResource("table_startDate");
			break;
		case END_DATE_COLUMN:
			result = SearsResourceBundle.getResource("table_endDate");
			break;
		case SUBTITLE_COLUMN:
			result = SearsResourceBundle.getResource("table_subtitle");
			break;
		default:
			break;
		}
    	//return the result
        return result;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount() {
        int result = 0;
        if (subtitleList != null) {
            result = subtitleList.size();
        }
        return result;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt(int row, int column) {
        //The result of the method
        Object result = null;
        //first, get the subtitle a this row.
        Subtitle subtitle = (Subtitle) subtitleList.get(row);
        //switch on column number to know what to return.
        switch (column) {
            case ID_COLUMN: {
                //We are searching for the ID
                result = new Integer(subtitle.getNumber());
                break;
            }
            case START_DATE_COLUMN: {
                //getting start Date
                result = SubtitleFile.timeToString(subtitle.getStartDate());
                break;
            }
            case END_DATE_COLUMN: {
                //getting end Date
                result = SubtitleFile.timeToString(subtitle.getEndDate());
                break;
            }
            case SUBTITLE_COLUMN: {
                //getting subtitle
                result = subtitle.getSubtitle();
                break;
            }
        }
        return result;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#isCellEditable(int, int)
     */
    public boolean isCellEditable(int row, int column) {
        return column != ID_COLUMN && column != ANCHOR_COLUMN;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
     */
    public void setValueAt(Object aValue, int row, int column) {
        //first, get the subtitle a this row.
        Subtitle subtitle = (Subtitle) subtitleList.get(row);
        //keep an copy, to know if it has changed.
        Subtitle oldSubtitle = new Subtitle(subtitle);
        
        try {
            //switch on column number to know what to return.
            switch (column) {
            case ID_COLUMN: {
                //We are setting the ID
                subtitle.setNumber(((Integer) aValue).intValue());
                break;
            }
            case START_DATE_COLUMN: {
                //setting start Date
                subtitle.setStartDate(SrtFile.stringToTime((String) aValue));
                break;
            }
            case END_DATE_COLUMN: {
                //setting end Date
                subtitle.setEndDate(SrtFile.stringToTime((String) aValue));
                break;
            }
            case SUBTITLE_COLUMN: {
                //setting subtitle
                subtitle.setSubtitle((String) aValue);
                break;
            }
            }
            //If subtitle has changed.
            if(!subtitle.equals(oldSubtitle)){
                //indicate main window that file has changed.
                MainWindow.instance.fileChanged();
                //Add this edit to undo manager
                Trace.trace("Undoable edit happened");
                //Add it to manager
                MainWindow.instance.getUndoManager().undoableEditHappened(new UndoableEditEvent(this, new SearsSimpleUndoEdit(subtitleList, row, oldSubtitle, subtitle)));
            }
        } catch (NumberFormatException e) {
            //entered value was bad do nothing.
        }
    }
}
