/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.gui.glassPane;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JViewport;

/**
 * This class, although it's a non Abstract class and could be use like it is,
 * is a <i>prototype</i> class and should be overwritted.
 */
public class DefaultViewportGlassPane extends ViewportGlassPane {

	private static final long serialVersionUID = -7782241093516309531L;

	/** the default color of the virtual glass pane */
	protected static final Color DEFAULT_COLOR = Color.BLACK;
	/** the default factor of the virtual glass pane alpha transparency*/
	protected static final float DEFAULT_ALPHA = .22f;

	private Color virtualBoundsColor;
	private float alpha;

	/**
	 * Creates a new class instance with default color and alpha factor transparency
	 * @param viewport	the viewport
	 * @throws 			NullPointerException if viewport is null
	 * @see ViewportGlassPane#ViewportGlassPane(JViewport)
	 */
	public DefaultViewportGlassPane(JViewport viewport) {
		super(viewport);
		init(DEFAULT_COLOR, DEFAULT_ALPHA);
	}

	/**
	 * Creates a new class instance with non default color and alpha factor transparency
	 * 
	 * @param viewport				the viewport
	 * @param virtualBoundsColor	the color of the drawing virtual glass pane
	 * @param alpha					a float that define the opacity of the virtual glass pane
	 * @throws 			NullPointerException if viewport is null
	 * @see ViewportGlassPane#ViewportGlassPane(JViewport)
	 */
	public DefaultViewportGlassPane(JViewport viewport, Color virtualBoundsColor, float alpha) {
		super(viewport);
		init(virtualBoundsColor, alpha);
	}

	/**
	 * Initializes color and alpha factor with values given on parameters
	 * <br>If null values is given, default values is set.
	 * @param c	the color 
	 * @param a	the alpha factor
	 */
	private void init(Color c, float a) {
		if( c == null ) {
			c = DEFAULT_COLOR;
		}
		if( a < 0 ) {
			a = DEFAULT_ALPHA;
		}

		virtualBoundsColor = c;
		alpha = a;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.glassPane.ViewportGlassPane#paintWithGraphics(java.awt.Graphics2D)
	 */
	protected void paintComponentWithGraphics(Graphics2D gr) {
		Color color = gr.getColor();
		Composite comp = gr.getComposite();	
		gr.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha ));	
		gr.setColor(virtualBoundsColor);	
		gr.fill(this.getVirtualBounds());
		gr.setColor(color);
		gr.setComposite(comp);
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#paintChildren(java.awt.Graphics)
	 */
	public void paintChildren(Graphics g) {
		super.paintChildren(g);
		Graphics2D gr = (Graphics2D) g;
		paintChildrenWithGraphics(gr);
	}

	// This method is an HOOK method.
	// It's mean that it would have to be reimplemented
	//
	/**
	 * This method is called in <code>paintChildren(g)</code> method.
	 * Allow a way to use the graphics context of this component
	 * 
	 * @param gr the graphics context of this component
	 * @see ViewportGlassPane#paintComponent(Graphics)
	 */
	protected void paintChildrenWithGraphics(Graphics2D gr) {
		// NOTHING IS DONE HERE
	}
}