package sears.gui.glassPane;

import java.awt.Component;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;

import javax.swing.JScrollPane;
import javax.swing.JViewport;


/**
 * Defines a generic view port glass pane module
 */
public abstract class ViewportGlassPaneModule {
	
	protected Component view;
	protected JViewport viewport;
	protected boolean isViewportChanged;
	
	/**
	 * Constructs a new <code>ViewportGlassPaneModule</code> object.
	 * @param scrollPane			the scroll pane
	 * @throws NullPointerException if <tt>scrollPane</tt> is null
	 * @throws NullPointerException	if its view port is null
	 * @throws NullPointerException	if the view of the view port is null
	 * @see JScrollPane
	 * @see JViewport
	 */
	public ViewportGlassPaneModule(JScrollPane scrollPane) {
		if( scrollPane == null ) {
			throw new NullPointerException("JScrollPane object cannot be null");
		}

		viewport = scrollPane.getViewport();
		if( viewport == null ) {
			throw new NullPointerException("JScrollPane object have not a valid viewport");
		}

		view = viewport.getView();
		if( view == null ) {
			throw new NullPointerException("JScrollPane object viewport have not a valid view");
		}	
		isViewportChanged = false;
		scrollPane.getVerticalScrollBar().addAdjustmentListener(new AdjustmentListener() {

			public void adjustmentValueChanged(AdjustmentEvent e) {
				fireAdjustmentValueChanged(e);	
			}			
		});
	}

	/**
	 * Use when view change
	 */
	public void fireViewChange() {
		isViewportChanged = true;
	}
	
	public abstract void fireAdjustmentValueChanged(AdjustmentEvent e);
	
}
