/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.gui.resources;

import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.net.URL;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import sears.gui.JDialogOptions;
import sears.gui.MainWindow;
import sears.tools.JarResources;
import sears.tools.SearsProperties;
import sears.tools.Trace;
import sears.tools.Utils;

/**
 * Class SearsResourceBundle.
 * <br><b>Summary:</b><br>
 * This class contains resources for Sears GUI.
 */
public class SearsResources {
    private static Object[][] resources = { 
        //General stuff
    	// 'favicon' :) displayed by Windows and Linux OS
    	// [ 20x20 size is good for a nice icon view ]
    	{ "SearsGUIFavicon", "searsFavicon.png" },
        { "SearsGUIIcon", "sears.png" },
        { "SearsGUITitle", "Sears Beta V" },
        { "BrowseIcon", "browse.png" },
        // used by ArrowButton class
        { "ArrowDown", "arrowDown.png" },
        { "ArrowUp", "arrowUp.png" },
        
        //Open Action
        { "OpenIcon", "open.png"},
        { "OpenKey" , KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_DOWN_MASK)},
        { "OpenKeyMac", KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.META_MASK)},
        
        //Save Action
        { "SaveIcon", "save.png"},
        { "SaveKey" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK)},
        { "SaveKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.META_MASK)},
        
        //SaveAs Action
        { "SaveAsIcon", "saveAs.png"},
        { "SaveAsKey" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_MASK)},
        { "SaveAsKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.META_MASK | InputEvent.SHIFT_MASK)},
        
        //Quit Action
        { "QuitIcon", "quit.png"},
        //{ "QuitKey" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_DOWN_MASK)},
        //{ "QuitKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.META_MASK)},

        //Delay Action
        { "DelayIcon", "delay.png"},
        { "DelayKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F1, InputEvent.CTRL_DOWN_MASK)},
        { "DelayKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.META_MASK | InputEvent.ALT_DOWN_MASK)},
        
        //Resynch Action
        { "ResynchIcon", "resynch.png"},
        { "ResynchKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_DOWN_MASK)},
        { "ResynchKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.META_MASK | InputEvent.ALT_DOWN_MASK)},
        
        //Split Action
        { "SplitIcon", "split.png"},
        { "SplitKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F3, InputEvent.CTRL_DOWN_MASK)},
        { "SplitKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.META_MASK | InputEvent.ALT_DOWN_MASK)},
        
        //Append Action
        { "AppendIcon", "append.png"},
        { "AppendKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.CTRL_DOWN_MASK)},
        { "AppendKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.META_MASK | InputEvent.ALT_DOWN_MASK)},
        
        //ChainRepair Action
        { "ChainRepairIcon", "chainRepair.png"},
        { "ChainRepairKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F5, InputEvent.CTRL_DOWN_MASK)},
        { "ChainRepairKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.META_MASK | InputEvent.ALT_DOWN_MASK)},
        
        //AccentRepair Action
        { "AccentRepairIcon", "accentRepair.png"},
        { "AccentRepairKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F6, InputEvent.CTRL_DOWN_MASK)},
        { "AccentRepairKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},
        
        //HtmlRepair Action
        { "HtmlRepairIcon", "htmlRepair.png"},
        { "HtmlRepairKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F7, InputEvent.CTRL_DOWN_MASK)},
        { "HtmlRepairKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},
        
        //OrderRepair Action
        { "OrderRepairIcon", "orderRepair.png"},
        { "OrderRepairKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F8, InputEvent.CTRL_DOWN_MASK)},
        { "OrderRepairKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},
        
        //TimeRepair Action
        { "TimeRepairIcon", "timeRepair.png"},
        { "TimeRepairKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F9, InputEvent.CTRL_DOWN_MASK)},
        { "TimeRepairKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},

        //Undo Action
        { "UndoIcon", "undo.png"},
        { "UndoKey" , KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_DOWN_MASK)},
        { "UndoKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},

        //Redo Action
        { "RedoIcon", "redo.png"},
        { "RedoKey" , KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_DOWN_MASK)},
        { "RedoKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.META_MASK | InputEvent.SHIFT_DOWN_MASK)},

        
        //NormalizeDuration Action
        { "NormalizeDurationIcon", "normalizeDuration.png"},
        { "NormalizeDurationKey" , null},
        
        //Split GUI stuff
        { "SplitGUIBanner", "splitBanner.png"},
        
        //SelectVideo Action
        { "SelectVideoIcon", "openVideo.png"},
        { "SelectVideoKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F10, InputEvent.CTRL_DOWN_MASK)},      
        { "SelectVideoKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.META_DOWN_MASK | InputEvent.SHIFT_MASK)}, 
        
        //Options Action
        { "OptionsIcon", "options.png"},
        { "OptionsKey" , KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_DOWN_MASK)},
        { "OptionsKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_COMMA, InputEvent.META_MASK)},
        
        //Play Action
        { "PlayIcon", "play.png"},
        { "PlayKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F11, InputEvent.CTRL_DOWN_MASK)},
        { "PlayKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.META_DOWN_MASK | InputEvent.SHIFT_MASK)},
        
        //Pause Action
        { "PauseIcon", "pause.png"},
        
        //Stop Action
        { "StopIcon", "stop.png"},
        
        //Previous Action
        { "PreviousIcon", "reward.png"},

        //Next Action
        { "NextIcon", "forward.png"},
 
        //GotoSub Action
        { "GotoSubIcon", "gotoSub.png"},

        //MagicResynchro Action
        { "MagicResynchroIcon", "magicResynchro.png"},
 
        //RemoveAllAnchors Action
        { "RemoveAllAnchorsIcon", "removeAllAnchors.png"},
 
        //RecentFile Action
        { "RecentFileIcon", "openRecent.png"},
 
        //Anchor icon
        { "AnchorIcon", "anchor.png"},
        
        //Dot icon
        { "DotIcon", "dot.png"},
        
        //About Action
        { "AboutIcon", "about.png"},
        
        //Help
        { "HelpIcon", "help.png"},
        // no question_mark keyEvent...
        
        //Find Action
        { "FindIcon", "find.png"},
        { "FindKey" , KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK)},
        { "FindKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.META_DOWN_MASK)},

        { "TinyFindIcon", "tinyFind.png" },
        
        //Mix Action
        { "MixIcon", "mix.png"},
        { "MixKey" , KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK)},
        { "MixKeyMac" , KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.META_DOWN_MASK | InputEvent.ALT_DOWN_MASK)},
   
        // Error icon, use in JDialogErrorMessage
        { "ErrorIcon", "error.png" },
        
        // Flag:
        { "fr", "flag/fr.png" },
        { "en", "flag/us.png" },
        { "it", "flag/it.png" },
        { "es", "flag/es.png" },
        { "nl", "flag/nl.png" },
        
        { "delete", "s_delete.png"},
        { "deletePressed", "s_delete_pressed.png"},
        
        { "blank","blank.png"}
    
    };

    private static HashMap<String, Object> resourcesMap;

	private static JarResources jarResources;
	
	private static String iconFolder;
    

    /**
     * Method getString.
     * <br><b>Summary:</b><br>
     * This method return the value that correspond to the given key.
     * @param key       The key to search.
     * @return          The <b>String</b> searched value, or "" if does not found.
     */
    public static String getString(String key) {
        //the result of the method.
        String result = "";
        if (resourcesMap == null) {
            constructMap();
        }
        String value = (String) resourcesMap.get(key);
        //return value only if non null
        if (value != null) {
            result = value;
        }
        //return the result
        return result;
    }

    /**
     * Method getIcon.
     * <br><b>Summary:</b><br>
     * This method return the icon that correspond to the given key.
     * @param key       The key to search.
     * @return          The <b>ImageIcon</b> searched icon, or blankIcon if does not found.
     */
    public static ImageIcon getIcon(String key) {
        //the result of the method.
        ImageIcon result = null;
        String iconFile = getString(key);
        if (!iconFile.equals("")) {
            //retrieve icon URL.
            //check if a skin is precised.
            String iconSetFile = SearsProperties.getProperty(SearsProperties.ICON_SET);
            if(iconSetFile != null && !iconSetFile.equals("")){
            	//We have to reach the icon in the jar file using the jarResources.
            	//create the jarResources if not already set.
            	if(jarResources == null){
            		jarResources = new JarResources(getIconFolder()+File.separator+iconSetFile);
            	}
            	//Reach the icon in it.
            	byte[] byteImage = jarResources.getResource(iconFile);
            	if(byteImage != null){
            		result = new ImageIcon(Toolkit.getDefaultToolkit().createImage(byteImage));
            	}else{
            		Trace.warning("Can't find resource "+iconFile+" in jar resources "+iconSetFile+". Using default icon.");
            	}
            }
            //If can not find resource, use default icon.
            if(result == null){
            	URL url = MainWindow.instance.getClass().getResource("/sears/gui/resources/" + iconFile);
                if(url != null){
                    result = new ImageIcon(url);
                }
            }
        }
        //if could not find icon, return blank icon
        if(result == null){
            result = new ImageIcon(MainWindow.instance.getClass().getResource("/sears/gui/resources/blank.png"));
        }
        //return the result
        return result;
    }
    
    /**
     * Method getKey.
     * <br><b>Summary:</b><br>
     * This method return the Key accelerator that correspond to the given key.
     * @param key       The key to search.
     * @return          The <b>KeyStroke</b> searched key, or null if does not found.
     */
    public static KeyStroke getKey(String key) {
    	// if we are running on Mac OS:
    	if(Utils.isMacPlatform){
    		key = key + "Mac";
    	} 
        //the result of the method.
        KeyStroke result = (KeyStroke) resourcesMap.get(key);
        //return the result
        return result;
    }

    /**
     * Method constructMap.
     * <br><b>Summary:</b><br>
     * This contruct the HashMap of resources.
     */
    private static void constructMap() {
        resourcesMap = new HashMap<String, Object>();
        for (int i = 0; i < resources.length; i++) {
            resourcesMap.put((String) resources[i][0], resources[i][1]);
        }
    }
    
	/**
	 * Method getIconFolder.
	 * <br><b>Summary:</b><br>
	 * This method return the Icon folder.
	 * @return  (<b>String</b>)   the Icon folder.
	 */
	public static String getIconFolder() {
		if (iconFolder == null) {
			// To do that retrieve the location of the JDialogOption class
			String classResource = JDialogOptions.class.getResource(
					"JDialogOptions.class").getFile();
			// DEVELOPER:
			// spaces are not allowed in URL and replaced by %20
			// there's a maybe a better solution to correct this
			// than the one under:
			classResource = classResource.replace("%20", " ");
			// Then retrieve the index of the "file:" string, which indicate we
			// are executing from a jar file.
			int indexOfFile = classResource.indexOf("file:");
			// the beginning of the jar file contain, if it exists:
			int indexOfJar = classResource.indexOf("!");
			// if we are in a jar file:
			if (indexOfFile != -1 && indexOfJar != -1) {
				// we get the parent file of the jar file:
				File parent = new File(classResource.substring(0, indexOfJar)).getParentFile();
				// and with the parent of the parent, we get back the icon directory path:
				iconFolder = new File(parent.getParent()) + System.getProperty("file.separator") + "icons";
				//remove begining "file:" pattern
				iconFolder = iconFolder.substring(5);
			} else {
				// we are not in a jar file, let's assume we are in eclipse.
				// And eclipse project does export the icons folder.
				// search for the last sears pattern, which is the root for the
				// sears class.
				int indexOfSearsPackage = classResource.lastIndexOf("sears");
				if (indexOfSearsPackage != -1) {
					// we have retrieved the sears folder, go up, and find the
					// icon folder.
					iconFolder = classResource
							.substring(0, indexOfSearsPackage);
					iconFolder += "icons";
				}
			}
		}
		return iconFolder;
	}
	
	/**
	 * Sets to null all utilized variables
	 * <br>Useful when an icon change request is launched
	 */
	public static void dispose() {
		//resourcesMap = null;
		jarResources = null;
		//iconFolder = null;
	}
}
