/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
// $Id
////////////////////////////////////////////////

package sears.gui.undo;

import java.util.ArrayList;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;

import sears.file.Subtitle;

/**
 * Class SearsSimpleUndoEdit
 * This represents a simple subtitle edit
 */
public class SearsSimpleUndoEdit implements UndoableEdit{
	
	/** The subtitle before the edit.*/
	private Subtitle _oldSubtitle = null;
	/** The subtitle after the edit.*/
	private Subtitle _newSubtitle = null;
	/** The subtitle list*/
	private ArrayList<Subtitle> _subtitleList = null;
	/** The row of the edit.*/
	private int _row;

	/**
	 * Constructor of the SimpleUndoEdit
	 * @param subtitleList		The list that contains the subtitles
	 * @param row				The row of the edit
	 * @param oldSubtitle		The subtitle before the edit
	 * @param newSubtitle		The subtitle after the edit
	 */
	public SearsSimpleUndoEdit(ArrayList<Subtitle> subtitleList, int row, Subtitle oldSubtitle, Subtitle newSubtitle){
		_subtitleList = subtitleList;
		_oldSubtitle = oldSubtitle;
		_newSubtitle = newSubtitle;
		_row = row;
	}
	
	@Override
	public boolean addEdit(UndoableEdit anEdit) {
		return false;
	}

	@Override
	public boolean canRedo() {
		return true;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public void die() {
		_oldSubtitle = null;
		_newSubtitle = null;
	}

	@Override
	public String getPresentationName() {
		return _oldSubtitle+"=>"+_newSubtitle;
	}

	@Override
	public String getRedoPresentationName() {
		return "=>"+_newSubtitle;
	}

	@Override
	public String getUndoPresentationName() {
		return ""+_oldSubtitle+"<=";
	}

	@Override
	public boolean isSignificant() {
		return true;
	}

	@Override
	public void redo() throws CannotRedoException {
		_subtitleList.set(_row, _newSubtitle);
	}

	@Override
	public boolean replaceEdit(UndoableEdit anEdit) {
		return false;
	}

	@Override
	public void undo() throws CannotUndoException {
		_subtitleList.set(_row, _oldSubtitle);
	}
}
