//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui.undo;

import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.event.UndoableEditEvent;

import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.gui.MainWindow;
import sears.tools.SearsAction;
import sears.tools.SearsResourceBundle;

/**
 * SearsUndoAction This is a generic undo/redoable action. It hardly save whole
 * subtitles just before the actions and after. 
 * And creates a basic SearsUndoEdit that could restore old list.
 */
public abstract class SearsUndoAction extends SearsAction {

	/** Default serial UID */
	private static final long serialVersionUID = 1L;

	/** (<b>SearsUndoListener</b>) listener: The listener for this undo action. */
	private SearsUndoManager undoManager;

	/**
	 * (<b>String</b>) UNDOTEXT_LABEL: The UNDOTEXT_LABEL a label to store
	 * undo text in action.
	 */
	private final static String UNDOTEXT_LABEL = "UndoTextLabel";

	/**
	 * Constructor SearsUndoAction. <br>
	 * <b>Summary:</b><br>
	 * The constructor of the class SearsUndoAction
	 * 
	 * @param actionTag
	 *            The action tag off the action.
	 * @param undoManager
	 *            The undoManager to be notified when this action happens.
	 */
	public SearsUndoAction(String actionTag, SearsUndoManager undoManager) {
		super(actionTag);
		this.undoManager = undoManager;
		putValue(UNDOTEXT_LABEL, SearsResourceBundle.getResource(actionTag
				+ "UndoText"));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public final void actionPerformed(ActionEvent e) {
		SubtitleFile subtitleFile = MainWindow.instance.getSubtitleFile();
		if (subtitleFile != null) {
			// Create a clone of the current subtitle list.
			ArrayList<Subtitle> beforeList = (ArrayList<Subtitle>) subtitleFile
					.getSubtitleListClone();
			// Do the action.
			boolean success = doUndoableAction(e);
			// Create a clone of the current subtitle list.
			ArrayList<Subtitle> afterList = (ArrayList<Subtitle>) subtitleFile
					.getSubtitleListClone();
			if (success) {
				// Create the corresponding undoable edit.
				SearsUndoEdit searsUndoEdit = new SearsUndoEdit(subtitleFile,
						beforeList, afterList,
						(String) getValue(UNDOTEXT_LABEL));
				// Add it to manager
				undoManager.undoableEditHappened(new UndoableEditEvent(e
						.getSource(), searsUndoEdit));
			}
		} else {
			// IF there is no subtitle files, this action will not be undoable,
			// just do it.
			doUndoableAction(e);
		}
	}

	/**
	 * Method doAction. <br>
	 * <b>Summary:</b><br>
	 * Do the undoable action. Return true if action has been done, false
	 * otherwise, not to keep in undo/redo queue actions that failed.
	 * 
	 * @param e
	 *            The initial actionEvent.
	 * @return (<b>boolean</b>) Return true if action has been done, false
	 *         otherwise.
	 */
	public abstract boolean doUndoableAction(ActionEvent e);
}
