//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui.undo;

import java.util.ArrayList;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;

import sears.file.Subtitle;
import sears.file.SubtitleFile;

/**
 * @author booba
 * 
 */
public class SearsUndoEdit implements UndoableEdit {

	/** (<b>SubtitleFile</b>) subtitleFile: The subtitleFile */
	private SubtitleFile subtitleFile;
	/** (<b>ArrayList<Subtitle></b>) beforeActionList: The beforeActionList */
	private ArrayList<Subtitle> beforeActionList;
	/** (<b>ArrayList<Subtitle></b>) afterActionList: The afterActionList */
	protected ArrayList<Subtitle> afterActionList;
	/** (<b>String</b>) editTitle: The editTitle */
	private String editTitle;

	public SearsUndoEdit(SubtitleFile subtitleFile,
			ArrayList<Subtitle> beforeActionList,
			ArrayList<Subtitle> afterActionList, String editTitle) {
		this.subtitleFile = subtitleFile;
		this.beforeActionList = beforeActionList;
		this.afterActionList = afterActionList;
		this.editTitle=editTitle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#addEdit(javax.swing.undo.UndoableEdit)
	 */
	public boolean addEdit(UndoableEdit anEdit) {
		//The result of the method.
		boolean result = false;
		//Only merge non atomic edit!
		if(anEdit instanceof SearsUndoEdit && !((SearsUndoEdit) anEdit).isAtomic()){
			//change final point here.
			afterActionList = ((SearsUndoEdit) anEdit).afterActionList;
			result = true;
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#canRedo()
	 */
	public boolean canRedo() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#canUndo()
	 */
	public boolean canUndo() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#die()
	 */
	public void die() {
		//nothing to do here.
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#getPresentationName()
	 */
	public String getPresentationName() {
		return editTitle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#getRedoPresentationName()
	 */
	public String getRedoPresentationName() {
		return editTitle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#getUndoPresentationName()
	 */
	public String getUndoPresentationName() {
		return editTitle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#isSignificant()
	 */
	public boolean isSignificant() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#redo()
	 */
	public void redo() throws CannotRedoException {
		//To redo the action is to put as subtitle list the final one.
		subtitleFile.setSubtitleList(afterActionList);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#replaceEdit(javax.swing.undo.UndoableEdit)
	 */
	public boolean replaceEdit(UndoableEdit anEdit) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.undo.UndoableEdit#undo()
	 */
	public void undo() throws CannotUndoException {
		//To undo the action is to put as subtitle list the original one.
		subtitleFile.setSubtitleList(beforeActionList);
	}
	
	/**
	 * Method isAtomic.
	 * <br><b>Summary:</b><br>
	 * returns true if action is atomic.
	 * Non atomic edit will be merged with previous in undo manager, and will be not single undoable. 
	 * @return  (<b>boolean</b>)   true if action is atomic.
	 */
	protected boolean isAtomic(){
		return true;
	}
}
