//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui.undo;

import java.awt.event.ActionEvent;

import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoManager;

import sears.gui.MainWindow;
import sears.tools.SearsAction;

/**
 * SearsUndoManager This manage the undo/redo for Sears.
 */
public class SearsUndoManager extends UndoManager implements
		UndoableEditListener {
	/** (<b>long</b>) serialVersionUID: The serialVersionUID */
	private static final long serialVersionUID = 5177764530749161060L;

	/** UNDO action key. */
	public static final String ACTION_KEY_UNDO = "Undo";
	/** REDO action key. */
	public static final String ACTION_KEY_REDO = "Redo";

	/** (<b>SearsAction</b>) undoAction: The undoAction */
	private SearsAction undoAction;

	/** (<b>SearsAction</b>) redoAction: The redoAction */
	private SearsAction redoAction;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.event.UndoableEditListener#undoableEditHappened(javax.swing.event.UndoableEditEvent)
	 */
	public void undoableEditHappened(UndoableEditEvent e) {
		addEdit(e.getEdit());
		// Update action status
		undoAction.setEnabled(canUndo());
		redoAction.setEnabled(canRedo());
	}

	/**
	 * Class UndoAction. <br>
	 * <b>Summary:</b><br>
	 * The undo Action
	 */
	class UndoAction extends SearsAction {

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = -513067279434959525L;

		public UndoAction() {
			super(ACTION_KEY_UNDO);
			setEnabled(false);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			undo();
			MainWindow.instance.updateTableAndActions();
			setEnabled(canUndo());
			redoAction.setEnabled(canRedo());
		}
	}

	/**
	 * Class RedoAction. <br>
	 * <b>Summary:</b><br>
	 * The undo Action
	 */
	class RedoAction extends SearsAction {

		/** (<b>long</b>) serialVersionUID: The serialVersionUID */
		private static final long serialVersionUID = -513067279434959525L;

		public RedoAction() {
			super(ACTION_KEY_REDO);
			setEnabled(false);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			redo();
			MainWindow.instance.updateTableAndActions();
			undoAction.setEnabled(canUndo());
			setEnabled(canRedo());
		}
	}

	/**
	 * Method getUndoAction. <br>
	 * <b>Summary:</b><br>
	 * Return the undoAction.
	 * 
	 * @return the undoAction
	 */
	public SearsAction getUndoAction() {
		if (undoAction == null) {
			undoAction = new UndoAction();

		}

		return undoAction;
	}

	/**
	 * Method getRedoAction. <br>
	 * <b>Summary:</b><br>
	 * Return the redoAction.
	 * 
	 * @return the redoAction
	 */
	public SearsAction getRedoAction() {
		if (redoAction == null) {
			redoAction = new RedoAction();

		}

		return redoAction;
	}

}
