/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.junit.file;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import sears.file.SrtFile;
import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.file.exception.io.FileConversionException;
import junit.framework.TestCase;

/**
 * Class SubtitleFileTest.
 * <br><b>Summary:</b><br>
 * Junit test of class SubtitleFile.
 * We will use SRTFile as implementation of subtitleFile to perform the tests.
 */
public class SubtitleFileTest extends TestCase {
    /*
     * Test method for 'sears.file.SubtitleFile.SubtitleFile()'
     */
    public void testSubtitleFile() {
        //just create a subtitle file.
        new SrtFile();
    }

    /*
     * Test method for 'sears.file.SubtitleFile.SubtitleFile(String, ArrayList)'
     */
    public void testSubtitleFileStringArrayList() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        try {
			new SrtFile(SubtitleFileTest.class.getResource("nominal.srt").getFile(), subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Check that file has been correctly parsed.
        //there must be 5 ST in the list.
        assertTrue(subtitleList.size() == 5);
        //check all subtitles.
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 1000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 2000, 3000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 4000, 5555, "hop hop !\n");
        assertSubtitleEqualsTo(subtitleList, 3, 4, 7000, 9000, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 10000, 11000, "Nous ne sommes pas seuls sur cette ile.\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.SubtitleFile(File, ArrayList)'
     */
    public void testSubtitleFileFileArrayList() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        try {
			new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()), subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Check that file has been correctly parsed.
        //there must be 5 ST in the list.
        assertTrue(subtitleList.size() == 5);
        //check all subtitles.
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 1000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 2000, 3000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 4000, 5555, "hop hop !\n");
        assertSubtitleEqualsTo(subtitleList, 3, 4, 7000, 9000, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 10000, 11000, "Nous ne sommes pas seuls sur cette ile.\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.getContentDirectory()'
     */
    public void testGetContentDirectory() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        File file = new File(SubtitleFileTest.class.getResource("nominal.srt").getFile());
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(file, subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        assertEquals(file.getParent(), subtitleFile.getContentDirectory().toString());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.getFile()'
     */
    public void testGetFile() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        File file = new File(SubtitleFileTest.class.getResource("nominal.srt").getFile());
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(file, subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        assertEquals(file, subtitleFile.getFile());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.getTemporaryFile()'
     */
    public void testGetTemporaryFile() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        File file = new File(SubtitleFileTest.class.getResource("nominal.srt").getFile());
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(file, subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Before calling write to temporary file, temp file must be null.
        assertNull(subtitleFile.getTemporaryFile());
        //then calling to temp file
        subtitleFile.writeToTemporaryFile();
        //must be non null now.
        assertNotNull(subtitleFile.getTemporaryFile());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.timeToString(int)'
     */
    public void testTimeToString() {
        int milliseconds = 0;
        String expected = "00:00:00,000";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 123;
        expected = "00:00:00,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 1123;
        expected = "00:00:01,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 60123;
        expected = "00:01:00,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 60123;
        expected = "00:01:00,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 61123;
        expected = "00:01:01,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 3661123;
        expected = "01:01:01,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        milliseconds = 36061123;
        expected = "10:01:01,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
        //test negative value
        milliseconds = -36061123;
        expected = "-10:01:01,123";
        assertEquals(expected, SubtitleFile.timeToString(milliseconds));
    }

    /*
     * Test method for 'sears.file.SubtitleFile.stringToTime(String)'
     */
    public void testStringToTime() {
        int expected = 0;
        String time = "00:00:00,000";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 123;
        time = "00:00:00,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 1123;
        time = "00:00:01,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 60123;
        time = "00:01:00,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 60123;
        time = "00:01:00,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 61123;
        time = "00:01:01,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 3661123;
        time = "01:01:01,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = 36061123;
        time = "10:01:01,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        //test negative values
        expected = -1;
        time = "-00:00:00,001";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        expected = -123;
        time = "-00:00:00,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
        
        expected = -36061123;
        time = "-10:01:01,123";
        assertEquals(expected, SubtitleFile.stringToTime(time));
    }

    /*
     * Test method for 'sears.file.SubtitleFile.addFakeSub()'
     */
    public void testAddFakeSub() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Check that file has been correctly parsed.
        //there must be 5 ST in the list.
        assertTrue(subtitleList.size() == 5);
        //Add a fake sub.
        subtitleFile.addFakeSub();
        //now there must be 6 ST in the list.
        assertTrue(subtitleList.size() == 6);
    }

    /*
     * Test method for 'sears.file.SubtitleFile.delay(int, int, int)'
     */
    public void testDelayIntIntInt() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //delay STs 1 to 3 of 1s
        subtitleFile.delay(1, 3, 1000);
        //1st must have not changed.
        assertSubtitleEqualsTo(subtitleList, 0, -1, 0, 1000, null);
        //2nd changed from 2,3 to 3,4
        assertSubtitleEqualsTo(subtitleList, 1, -1, 3000, 4000, null);
        //3rd changed
        assertSubtitleEqualsTo(subtitleList, 2, -1, 5000, 6555, null);
        //4th changed
        assertSubtitleEqualsTo(subtitleList, 3, -1, 8000, 10000, null);
        //5th did not changed.
        assertSubtitleEqualsTo(subtitleList, 4, -1, 10000, 11000, null);
        
        //delay STs 1 to 1 of 10s
        subtitleFile.delay(1, 1, 10000);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 13000, 14000, null);
    }

    /*
     * Test method for 'sears.file.SubtitleFile.delay(int[], int)'
     */
    public void testDelayIntArrayInt() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Array to be delayed.
        int[] indexToDelay = new int[]{0,2,4};
        //delay of 1s !
        subtitleFile.delay(indexToDelay, 1000);
        
        assertSubtitleEqualsTo(subtitleList, 0, -1, 1000, 2000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 2000, 3000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 5000, 6555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 7000, 9000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 11000, 12000, null);
    }

    /*
     * Test method for 'sears.file.SubtitleFile.delay(int)'
     */
    public void testDelayInt() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //delay all STs of 1s !
        subtitleFile.delay(1000);
        
        assertSubtitleEqualsTo(subtitleList, 0, -1, 1000, 2000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 3000, 4000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 5000, 6555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 8000, 10000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 11000, 12000, null);
        
    }

    /*
     * Test method for 'sears.file.SubtitleFile.resynchro(int[])'
     */
    public void testResynchro() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Resynchro !
        //Case 1: time shifting of 1s.
        int[] parameter = new int[]{0, 1000, 10000, 11000};
        subtitleFile.resynchro(parameter);
        assertSubtitleEqualsTo(subtitleList, 0, -1, 1000, 2000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 3000, 4000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 5000, 6555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 8000, 10000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 11000, 12000, null);
        //Case 2: time shifting of -1s.
        parameter = new int[]{1000, 0, 11000, 10000};
        subtitleFile.resynchro(parameter);
        assertSubtitleEqualsTo(subtitleList, 0, -1, 0, 1000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 2000, 3000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 4000, 5555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 7000, 9000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 10000, 11000, null);
        //Case 3: time expand *2
        parameter = new int[]{0, 0, 10000, 20000};
        subtitleFile.resynchro(parameter);
        assertSubtitleEqualsTo(subtitleList, 0, -1, 0, 1000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 4000, 5000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 8000, 9555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 14000, 16000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 20000, 21000, null);
        //Case 3: time constract /2
        parameter = new int[]{0, 0, 20000, 10000};
        subtitleFile.resynchro(parameter);
        assertSubtitleEqualsTo(subtitleList, 0, -1, 0, 1000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 2000, 3000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 4000, 5555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 7000, 9000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 10000, 11000, null);
        //Case 4: time expand *2 + shift 1s
        parameter = new int[]{0, 1000, 10000, 21000};
        subtitleFile.resynchro(parameter);
        assertSubtitleEqualsTo(subtitleList, 0, -1, 1000, 2000, null);
        assertSubtitleEqualsTo(subtitleList, 1, -1, 5000, 6000, null);
        assertSubtitleEqualsTo(subtitleList, 2, -1, 9000, 10555, null);
        assertSubtitleEqualsTo(subtitleList, 3, -1, 15000, 17000, null);
        assertSubtitleEqualsTo(subtitleList, 4, -1, 21000, 22000, null);
    }

    /*
     * Test method for 'sears.file.SubtitleFile.setFile(File)'
     */
    public void testSetFile() {
        SubtitleFile subtitleFile = new SrtFile();
        File file = new File(SubtitleFileTest.class.getResource("nominal.srt").getFile());
        subtitleFile.setFile(file);
        assertEquals(file, subtitleFile.getFile());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.addSubtitle(Subtitle)'
     */
    public void testAddSubtitleSubtitle() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Add a new subtitle.
        Subtitle newSub = new Subtitle(0, 12000, 13000, "new one");
        subtitleFile.addSubtitle(newSub);
        //check it is added correctly.
        assertSubtitleEqualsTo(subtitleList, 5, 6, 12000, 13000, "new one");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.addSubtitle(Subtitle, boolean)'
     */
    public void testAddSubtitleSubtitleBoolean() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //Add a new subtitle, updating its number! (with true)
        Subtitle newSub = new Subtitle(0, 12000, 13000, "new one");
        subtitleFile.addSubtitle(newSub, true);
        //check it is added correctly.
        assertSubtitleEqualsTo(subtitleList, 5, 6, 12000, 13000, "new one");
        //Add a new subtitle, without updating its number!
        newSub = new Subtitle(0, 12000, 13000, "new one");
        subtitleFile.addSubtitle(newSub, false);
        //check it is added correctly.
        assertSubtitleEqualsTo(subtitleList, 6, 0, 12000, 13000, "new one");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.split(File[], int, int)'
     */
    public void testSplit() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        //Split in 2 files.
        try {
            File file1 = File.createTempFile("split", "1");
            File file2 = File.createTempFile("split", "2");
            SubtitleFile[] results= subtitleFile.split(new File[]{file1, file2}, 2, 1000);
            //check file 1.
            results[0].writeToFile(file1);
            ArrayList<Subtitle> subtitleList1 = new ArrayList<Subtitle>();
            new SrtFile(file1,  subtitleList1);
            assertSubtitleEqualsTo(subtitleList1, 0, 1, 0, 1000, "Le premier sous titre.\n");
            assertSubtitleEqualsTo(subtitleList1, 1, 2, 2000, 3000, "Le deuxieme sous titre.\n");
            //Check file2
            results[1].writeToFile(file2);
            ArrayList<Subtitle> subtitleList2 = new ArrayList<Subtitle>();
            new SrtFile(file2,  subtitleList2);
            assertSubtitleEqualsTo(subtitleList2, 0, 1, 1000, 2555, "hop hop !\n");
            assertSubtitleEqualsTo(subtitleList2, 1, 2, 4000, 6000, "sabotes, kidnappes, assassines.\n");
            assertSubtitleEqualsTo(subtitleList2, 2, 3, 7000, 8000, "Nous ne sommes pas seuls sur cette ile.\n");
        } catch (IOException e) {
            fail("IOException in testSplit "+e.getMessage());
        }
    }

    /*
     * Test method for 'sears.file.SubtitleFile.append(SubtitleFile, int)'
     */
    public void testAppend() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        //construct another.
        ArrayList<Subtitle> subtitleList2 = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile2 = null;
		try {
			subtitleFile2 = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList2);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        //Append the subtitle file to itself.
        subtitleFile.append(subtitleFile2, 1000);
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 1000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 2000, 3000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 4000, 5555, "hop hop !\n");
        assertSubtitleEqualsTo(subtitleList, 3, 4, 7000, 9000, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 10000, 11000, "Nous ne sommes pas seuls sur cette ile.\n");
        assertSubtitleEqualsTo(subtitleList, 5, 6, 12000, 13000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 6, 7, 14000, 15000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 7, 8, 16000, 17555, "hop hop !\n");
        assertSubtitleEqualsTo(subtitleList, 8, 9, 19000, 21000, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 9, 10, 22000, 23000, "Nous ne sommes pas seuls sur cette ile.\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.isFileChanged()'
     */
    public void testIsFileChanged() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        assertFalse(subtitleFile.isFileChanged());
        subtitleFile.fileChanged();
        assertTrue(subtitleFile.isFileChanged());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.fileChanged()'
     */
    public void testFileChanged() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        assertFalse(subtitleFile.isFileChanged());
        subtitleFile.fileChanged();
        assertTrue(subtitleFile.isFileChanged());
    }

    /*
     * Test method for 'sears.file.SubtitleFile.accentRepair(int[])'
     */
    public void testAccentRepair() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("repair.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        subtitleFile.accentRepair(new int[]{1});
        assertSubtitleEqualsTo(subtitleList, 1, -1, -1, -1, "Le deuxieme sous titre.\n");
        //3 should not have been repaired
        assertSubtitleEqualsTo(subtitleList, 3, -1, -1, -1, "sabotes, kidnappés, assassines.\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.htmlRepair(int[])'
     */
    public void testHtmlRepair() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("repair.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        subtitleFile.htmlRepair(new int[]{4});
        assertSubtitleEqualsTo(subtitleList, 4, -1, -1, -1, "Nous ne sommes pas seuls sur cette ile.\n");
        //2 should not have been repaired
        assertSubtitleEqualsTo(subtitleList, 2, -1, -1, -1, "hop <b>hop</b> !\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.timeRepair()'
     */
    public void testTimeRepair() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("repair.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        subtitleFile.timeRepair();
        //2nd had its start date after its end date. It should have been fixed.
        assertSubtitleEqualsTo(subtitleList, 1, -1, 3000, 3001, null);
        //4 started before end of 3, should have been fixed.
        assertSubtitleEqualsTo(subtitleList, 3, -1, 8001, 9000, null);
    }

    /*
     * Test method for 'sears.file.SubtitleFile.orderRepair()'
     */
    public void testOrderRepair() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("repair.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        subtitleFile.orderRepair();
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 1000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 2000, 3000, "Nous ne <i>sommes</i> pas seuls sur cette ile.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 3000, 2000, "Le deuxième sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 3, 4, 4000, 8000, "hop <b>hop</b> !\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 7000, 9000, "sabotes, kidnappés, assassines.\n");
    }

    /*
     * Test method for 'sears.file.SubtitleFile.getSubtitleIndex(int)'
     */
    public void testGetSubtitleIndex() {
        //construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("nominal.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
       assertEquals(3, subtitleFile.getSubtitleAtDate(5000).getNumber());
       assertEquals(3, subtitleFile.getSubtitleAtDate(6000).getNumber());
       assertEquals(4, subtitleFile.getSubtitleAtDate(7000).getNumber());
       assertEquals(5, subtitleFile.getSubtitleAtDate(12000).getNumber());
    }
    
    
    public void testNormalizeDuration(){
    	//construct subtitle file.
        ArrayList<Subtitle> subtitleList = new ArrayList<Subtitle>();
        SubtitleFile subtitleFile = null;
		try {
			subtitleFile = new SrtFile(new File(SubtitleFileTest.class.getResource("normalize.srt").getFile()),
			        subtitleList);
		} catch (FileConversionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 1000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 5000, 6000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 10000, 15555, "hop hop !\n");
        assertSubtitleEqualsTo(subtitleList, 3, 4, 17000, 18000, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 19000, 26000, "Nous ne sommes pas seuls sur cette ile.\n");
        //test normalize action.
        //Normalize with min 2s and max 5s
        subtitleFile.normalizeDuration(2000, 5000);
        //First subtitle length should have moved.
        assertSubtitleEqualsTo(subtitleList, 0, 1, 0, 2000, "Le premier sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 1, 2, 5000, 7000, "Le deuxieme sous titre.\n");
        assertSubtitleEqualsTo(subtitleList, 2, 3, 10000, 15000, "hop hop !\n");
        //test that it does not overiden next subtitle start time.
        assertSubtitleEqualsTo(subtitleList, 3, 4, 17000, 18999, "sabotes, kidnappes, assassines.\n");
        assertSubtitleEqualsTo(subtitleList, 4, 5, 19000, 24000, "Nous ne sommes pas seuls sur cette ile.\n");
        
    }

    /**
     * Method subtitleEqualsTo.
     * <br><b>Summary:</b><br>
     * Get the subtitle at given index in the given subtitle list.<br/>
     * And check it properties.
     * If you don't want to filter on some fields, put -1 (for int) or null.
     * @param subtitleList          The list to find the subtitle.
     * @param index                 The index of the subtitle to get.
     * @param number                The number of the subtitle to check, or -1 not to check the number.
     * @param startDate             The startdate to check, or -1 not to check the start date.
     * @param endDate               The endDate to check, or -1 not to check the endDate.
     * @param subtitle              The subtitle to check, or null not to check the subtitle.
     */
    protected static void assertSubtitleEqualsTo(ArrayList<Subtitle> subtitleList, int index, int number, int startDate, int endDate,
            String subtitle) {
        //get the subtitle to check.
        Subtitle sub = (Subtitle) subtitleList.get(index);
        //check properties.
        if (number != -1 && number != sub.getNumber()) {
            //check number failed.
            fail("Number are different for subtitle. expected: " + number + " actual: " + sub.getNumber());
        }
        if (startDate != -1 && startDate != sub.getStartDate()) {
            //check startDate failed.
            fail("StartDate are different for subtitle. expected: " + startDate + " actual: " + sub.getStartDate());
        }
        if (endDate != -1 && endDate != sub.getEndDate()) {
            //check endDate failed.
            fail("EndDate are different for subtitle. expected: " + endDate + " actual: " + sub.getEndDate());
        }
        if (subtitle != null && !subtitle.equals(sub.getSubtitle())) {
            //check subtitle failed.
            fail("Subtitle are different for subtitle. expected: " + subtitle + " actual: " + sub.getSubtitle());
        }
    }
}
