/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.junit.file;

import sears.file.Subtitle;
import junit.framework.TestCase;

/**
 * Class .SubtitleTest
 * <br><b>Summary:</b><br>
 * The test of SubtitleTest class.
 */
public class SubtitleTest extends TestCase {
    
    /*
     * Test method for 'sears.file.Subtitle.Subtitle(int, int, int, String)'
     */
    public void testSubtitleIntIntIntString() {
        //just test constructor.
        new Subtitle(0, 0, 0, "subtitle");
    }

    /*
     * Test method for 'sears.file.Subtitle.Subtitle(Subtitle)'
     */
    public void testSubtitleSubtitle() {
        //This method permits to copy a subtitle.
        //construct source subtitle.
        Subtitle source = new Subtitle(0, 0, 1, "subtitle");
        Subtitle destination = new Subtitle(source);
        assertEquals(source, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.Subtitle()'
     */
    public void testSubtitle() {
        //test void constructor.
        new Subtitle();
    }

    /*
     * Test method for 'sears.file.Subtitle.delay(int)'
     */
    public void testDelay() {
        //Try to delay.
        Subtitle source = new Subtitle(0, 0, 1, "subtitle");
        //nominal cases.
        //delay of 0
        Subtitle expected = new Subtitle(0,0,1,"subtitle");
        Subtitle destination = new Subtitle(source);
        destination.delay(0);
        assertEquals(expected, destination);
        //delay of 1
        expected = new Subtitle(0,1,2,"subtitle");
        destination = new Subtitle(source);
        destination.delay(1);
        assertEquals(expected, destination);
        //delay of -1
        expected = new Subtitle(0,-1,0,"subtitle");
        destination = new Subtitle(source);
        destination.delay(-1);
        assertEquals(expected, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.setEndDate(int)'
     */
    public void testSetEndDate() {
        Subtitle source = new Subtitle(0, 0, 1, "subtitle");
        Subtitle destination = new Subtitle(source);
        //set at 0
        destination.setEndDate(0);
        Subtitle expected = new Subtitle(0,0,0,"subtitle");
        assertEquals(expected, destination);
        //Set at something positive
        destination.setEndDate(100);
        expected = new Subtitle(0,0,100,"subtitle");
        assertEquals(expected, destination);
        //Set at something negative
        destination.setEndDate(-100);
        expected = new Subtitle(0,0,-100,"subtitle");
        assertEquals(expected, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.setStartDate(int)'
     */
    public void testSetStartDate() {
        Subtitle source = new Subtitle(0, 1, 2, "subtitle");
        Subtitle destination = new Subtitle(source);
        //set at 0
        destination.setStartDate(0);
        Subtitle expected = new Subtitle(0,0,2,"subtitle");
        assertEquals(expected, destination);
        //Set at something positive
        destination.setStartDate(100);
        expected = new Subtitle(0,100,2,"subtitle");
        assertEquals(expected, destination);
        //Set at something negative
        destination.setStartDate(-100);
        expected = new Subtitle(0,-100,2,"subtitle");
        assertEquals(expected, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.setSubtitle(String)'
     */
    public void testSetSubtitle() {
        Subtitle source = new Subtitle(0, 1, 2, "subtitle");
        Subtitle destination = new Subtitle(source);
        //set at null
        destination.setSubtitle(null);
        Subtitle expected = new Subtitle(0,1,2,null);
        assertEquals(expected, destination);
        //Set at ""
        destination.setSubtitle("");
        expected = new Subtitle(0,1,2,"");
        assertEquals(expected, destination);
        //Set at something
        destination.setSubtitle("toto");
        expected = new Subtitle(0,1,2,"toto");
        assertEquals(expected, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.setNumber(int)'
     */
    public void testSetNumber() {
        Subtitle source = new Subtitle(1, 1, 2, "subtitle");
        Subtitle destination = new Subtitle(source);
        //set at 0
        destination.setNumber(0);
        Subtitle expected =new Subtitle(0, 1, 2, "subtitle");
        assertEquals(expected, destination);
        //Set at something positive
        destination.setNumber(100);
        expected = new Subtitle(100, 1, 2, "subtitle");
        assertEquals(expected, destination);
        //Set at something negative
        destination.setNumber(-100);
        expected = new Subtitle(-100, 1, 2, "subtitle");
        assertEquals(expected, destination);
    }

    /*
     * Test method for 'sears.file.Subtitle.getEndDate()'
     */
    public void testGetEndDate() {
        //with 0
        int endDate = 0;
        Subtitle source = new Subtitle(1, 1, endDate, "subtitle");
        assertEquals(endDate, source.getEndDate());
        //with 100
        endDate = 100;
        source = new Subtitle(1, 1, endDate, "subtitle");
        assertEquals(endDate, source.getEndDate());
        //with -100
        endDate = -100;
        source = new Subtitle(1, 1, endDate, "subtitle");
        assertEquals(endDate, source.getEndDate());
    }

    /*
     * Test method for 'sears.file.Subtitle.getNumber()'
     */
    public void testGetNumber() {
        //with 0
        int number = 0;
        Subtitle source = new Subtitle(number, 1, 2, "subtitle");
        assertEquals(number, source.getNumber());
        //with 100
        number = 100;
        source = new Subtitle(number, 1, 2, "subtitle");
        assertEquals(number, source.getNumber());
        //with -100
        number = -100;
        source = new Subtitle(number, 1, 2, "subtitle");
        assertEquals(number, source.getNumber());
    }

    /*
     * Test method for 'sears.file.Subtitle.getStartDate()'
     */
    public void testGetStartDate() {
        //with 0
        int startDate = 0;
        Subtitle source = new Subtitle(0, startDate, 2, "subtitle");
        assertEquals(startDate, source.getStartDate());
        //with 100
        startDate = 100;
        source = new Subtitle(0, startDate, 2, "subtitle");
        assertEquals(startDate, source.getStartDate());
        //with -100
        startDate = -100;
        source = new Subtitle(0, startDate, 2, "subtitle");
        assertEquals(startDate, source.getStartDate());
    }

    /*
     * Test method for 'sears.file.Subtitle.getSubtitle()'
     */
    public void testGetSubtitle() {
        //with null
        String subtitle = null;
        Subtitle source = new Subtitle(0, 0, 2, subtitle);
        assertEquals(subtitle, source.getSubtitle());
        //with ""
        subtitle = "";
        source = new Subtitle(0, 0, 2, subtitle);
        assertEquals(subtitle, source.getSubtitle());
        //with "toto"
        subtitle = "toto";
        source = new Subtitle(0, 0, 2, subtitle);
        assertEquals(subtitle, source.getSubtitle());
    }

    /*
     * Test method for 'sears.file.Subtitle.equals(Object)'
     */
    public void testEqualsObject() {
        Subtitle source = new Subtitle(0, 0, 1, "subtitle");
        Subtitle destination = new Subtitle(0, 0, 1, "subtitle");
        //equality.
        assertTrue(source.equals(destination));
        //different number
        destination = new Subtitle(1, 0, 1, "subtitle");
        assertFalse(source.equals(destination));
        //different start date.
        destination = new Subtitle(0, 1, 1, "subtitle");
        assertFalse(source.equals(destination));
        //different end date.
        destination = new Subtitle(0, 0, 2, "subtitle");
        assertFalse(source.equals(destination));
        //different subtitle.
        destination = new Subtitle(0, 0, 1, "subtitle_different");
        assertFalse(source.equals(destination));
    }

    /*
     * Test method for 'sears.file.Subtitle.accentRemove()'
     */
    public void testAccentRemove() {
        Subtitle source = new Subtitle(0, 0, 1, "test:èéêëûùüïîàâäöôçÈÉÊËÛÙÏÎÀÂÔÇ");
        String expected = "test:eeeeuuuiiaaaoocEEEEUUIIAAOC";
        source.accentRemove();
        assertEquals(expected, source.getSubtitle());
    }

    /*
     * Test method for 'sears.file.Subtitle.htmlRemove()'
     */
    public void testHtmlRemove() {
        Subtitle source = new Subtitle(0, 0, 1, "<b>test:</b><br/>Some <b>html</b> <i>stuff</i> has to be removed.");
        String expected = "test:Some html stuff has to be removed.";
        source.htmlRemove();
        assertEquals(expected, source.getSubtitle());
    }

    /*
     * Test method for 'sears.file.Subtitle.compareTo(Object)'
     */
    public void testCompareTo() {
        //compare to for subtitle is comparing their start dates.
        Subtitle s1 = new Subtitle(0, 0, 1, "subtitle_different");
        Subtitle s2 = new Subtitle(0, 1, 1, "subtitle_different");
        Subtitle s3 = new Subtitle(0, 0, 1, "subtitle_different");
        assertTrue(s1.compareTo(s2) < 0 );
        assertTrue(s2.compareTo(s1) > 0 );
        assertTrue(s1.compareTo(s3) == 0 );
        assertTrue(s1.compareTo(s1) == 0 );
    }
}
