//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.junit.tools;

import sears.tools.LinearInterpolation;
import junit.framework.TestCase;

public class LinearInterpolationTest extends TestCase {
	public void testLinearInterpolation(){
		new LinearInterpolation(new double[2], new double[2]);
		//try minimum parameter error.
		try{
			new LinearInterpolation(new double[1], new double[1]);
			fail("Linear interpolation must have thrownan exception with 1 data");
		}catch(IllegalArgumentException e){
			assertTrue(true);
		}
	}
	
	public void testInterpolate(){
		//test with y=x
		double[] x = new double[]{0, 1};
		double[] y = new double[]{0, 1};
		LinearInterpolation li = new LinearInterpolation(x, y);
		assertEquals((double) -1, li.interpolate(-1));
		assertEquals((double)0, li.interpolate(0));
		assertEquals((double)1, li.interpolate(1));
		assertEquals((double)0.5, li.interpolate(0.5));
		assertEquals((double)2, li.interpolate(2));
		//change datas: y = 4x
		double[] x1 = new double[]{0, 1};
		double[] y1 = new double[]{0, 2};
		li.resetDatas(x1, y1);
		assertEquals((double)0, li.interpolate(0));
		assertEquals((double)2, li.interpolate(1));
		assertEquals((double)1, li.interpolate(0.5));
		assertEquals((double)4, li.interpolate(2));
		
		//change datas y = x + 2
		double[] x2 = new double[]{0, 1};
		double[] y2 = new double[]{2, 3};
		li.resetDatas(x2, y2);
		assertEquals((double)2, li.interpolate(0));
		assertEquals((double)4, li.interpolate(2));
		assertEquals((double)3, li.interpolate(1));
		assertEquals((double)6, li.interpolate(4));
		
		//change datas [0-1] y = x
		//[1-2] y = 3*x -2
		double[] x3 = new double[]{0, 1, 2};
		double[] y3 = new double[]{0, 1, 4};
		li.resetDatas(x3, y3);
		assertEquals((double)-1.0, li.interpolate(-1.0));
		assertEquals((double)0, li.interpolate(0));
		assertEquals((double)0.5, li.interpolate(0.5));
		assertEquals((double)1, li.interpolate(1));
		assertEquals((double)2.5, li.interpolate(1.5));
		assertEquals((double)4, li.interpolate(2));
		assertEquals((double)10, li.interpolate(4));
		
		//change datas [0-1] y = x
		//[1-2] y = 3*x -2
		//[2-4] y = 4
		double[] x4 = new double[]{0, 1, 2, 4};
		double[] y4 = new double[]{0, 1, 4, 4};
		li.resetDatas(x4, y4);
		assertEquals((double)-1.0, li.interpolate(-1.0));
		assertEquals((double)0, li.interpolate(0));
		assertEquals((double)0.5, li.interpolate(0.5));
		assertEquals((double)1, li.interpolate(1));
		assertEquals((double)2.5, li.interpolate(1.5));
		assertEquals((double)4, li.interpolate(2));
		assertEquals((double)4, li.interpolate(3));
		assertEquals((double)4, li.interpolate(4));
		assertEquals((double)4, li.interpolate(5));
		
		//change datas [0-1] y = x
		//[1-2] y = 3*x -2
		//[2-4] y = 4
		//[4-5] y = x
		double[] x5 = new double[]{0, 1, 2, 4, 5};
		double[] y5 = new double[]{0, 1, 4, 4, 5};
		li.resetDatas(x5, y5);
		assertEquals((double)-1.0, li.interpolate(-1.0));
		assertEquals((double)0, li.interpolate(0));
		assertEquals((double)0.5, li.interpolate(0.5));
		assertEquals((double)1, li.interpolate(1));
		assertEquals((double)2.5, li.interpolate(1.5));
		assertEquals((double)4, li.interpolate(2));
		assertEquals((double)4, li.interpolate(3));
		assertEquals((double)4, li.interpolate(4));
		assertEquals((double)4.5, li.interpolate(4.5));
		assertEquals((double)5, li.interpolate(5));
		assertEquals((double)5.5, li.interpolate(5.5));
	}
}
