/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.junit.tools;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import junit.framework.TestCase;
import sears.tools.Utils;

/**
 * Class UtilsTest.
 * <br><b>Summary:</b><br>
 * JUnit test for UtilsClass.
 * 
 */
public class UtilsTest extends TestCase {

    /*
     * Test method for 'sears.tools.Utils.getExtension(File)'
     */
    public void testGetExtension() {
        //retrieve extension of files.
        String extension = "jpg";
        File file = new File("/tmp/toto."+extension);
        assertEquals(extension, Utils.getExtension(file));
        
        extension = "jpg";
        file = new File("/tmp/toto."+extension.toUpperCase());
        assertEquals(extension, Utils.getExtension(file));
        
        extension = "";
        file = new File("/tmp/toto");
        assertEquals(extension, Utils.getExtension(file));
        
    }

    /*
     * Test method for 'sears.tools.Utils.isWindowsPlatform()'
     */
    public void testIsWindowsPlatform() {
        System.setProperty("os.name", "windowsNT");
        assertTrue(Utils.isWindowsPlatform());
        System.setProperty("os.name", "linux");
        assertFalse(Utils.isWindowsPlatform());
        System.setProperty("os.name", "Mac OS X");
        assertFalse(Utils.isWindowsPlatform());
    }

    /*
     * Test method for 'sears.tools.Utils.copyFile(File, File)'
     */
    public void testCopyFile() {
        //Create a file and write something in it.
        String message = "write something.";
        try{
            File sourceFile = File.createTempFile("junit", "source");
            File destinationFile = File.createTempFile("junit", "destination");
            //create the source file.
            FileWriter fw = new FileWriter(sourceFile);
            fw.write(message, 0, message.length());
            fw.flush();
            fw.close();
            //Test the method copyFile.
            Utils.copyFile(sourceFile, destinationFile);
            //check that destination file is the same as source file.
            FileReader fr = new FileReader(destinationFile);
            String result = "";
            int c = fr.read();
            while (c != -1) {
                result += (char) c;
                c = fr.read();
            }
            assertEquals(message, result);
            
        } catch (FileNotFoundException e) {
            fail("FileNotFoundException in testCopyFile "+e.getMessage());
        } catch (IOException e) {
            fail("IOException in testCopyFile "+e.getMessage());
        }
        
    }

    /*
     * Test method for 'sears.tools.Utils.formatTime(int)'
     */
    public void testFormatTime() {
        int seconds = 0;
        String expected = "00:00:00";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 60;
        expected = "00:01:00";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 61;
        expected = "00:01:01";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 120;
        expected = "00:02:00";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 3601;
        expected = "01:00:01";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 36001;
        expected = "10:00:01";
        assertEquals(expected, Utils.formatTime(seconds));
        seconds = 36071;
        expected = "10:01:11";
        assertEquals(expected, Utils.formatTime(seconds));
    }
}
