/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com

//$Id: Main.java,v 1.14 2007/09/06 12:10:50 floriaen Exp $


package sears.main;
import java.awt.EventQueue;
import java.io.File;
import java.util.ArrayList;

import sears.file.SrtFile;
import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.file.exception.io.FileConversionException;
import sears.gui.MainWindow;
import sears.tools.Trace;
import sears.tools.Utils;
import sears.tools.eawt.AppException;
import sears.tools.eawt.SearsApp;

/**
 * Class Main.
 * <br><b>Summary:</b><br>
 * This is the main class.
 * It contains the main method that launches the Sears GUI or the inline commands.
 */
public class Main {
	/**The delay action*/
	private static String KEY_DELAY = "delay";
	/**The resynch action*/
	private static String KEY_RESYNCHRO = "resyn";
	/**The split action*/
	private static String KEY_SPLIT = "split";
	/**The append action*/
	private static String KEY_APPEND = "appen";

	/** The NEWLINE String, for this OS. */
	private static final String NEWL = System.getProperty("line.separator");

	/** The usage, that will displayed if user fails to enter correct parameters.*/
	private static String USAGE = "Sears"+NEWL
	+"DESCRIPTION:"+NEWL
	+"SEARS is a Subtitle Editor And Re-Synchroniser."+NEWL
	+"USAGE:"+NEWL
	+"java -jar sears.jar [action  ActionParameters]"+NEWL
	+"ACTIONS:"+NEWL
	+"-delay:"+NEWL
	+"  java -jar sears.jar "+KEY_DELAY+" d input_file [output_file]"+NEWL
	+"  will delay the input's file subtitle using the given integer delay d."+NEWL
	+"  result will be write in the input_file, or in output_file, if output_file is given."+NEWL
	+"-resyn:"+NEWL
	+"  java -jar sears.jar "+KEY_RESYNCHRO+" src1 dest1 src2 dest2 input_file [output_file]"+NEWL
	+"  will apply a resynchro to given input file."+NEWL
	+"  src and dest must be precised under format HH:MM:SS.MMM"+NEWL
	+"  result will be write in the input_file, or in output_file, if output_file is given."+NEWL
	+"-split:"+NEWL
	+"  java -jar sears.jar "+KEY_SPLIT+" index input_file output_file1 output_file2"+NEWL
	+"  will apply a split to given input file."+NEWL
	+"  It will cut the input file at subtitle which index is given."+NEWL
	+"  result will be write in the 2 output files output_file1 and output_file2."+NEWL
	+"-append:"+NEWL
	+"  java -jar sears.jar "+KEY_APPEND+" d input_file1 input_file2 [output_file]"+NEWL
	+"  will apply a append to given input file."+NEWL
	+"  It will append the subtitle of input_file2 with a delay of d."+NEWL
	+"  result will be write in the input_file, or in output_file, if output_file is given."+NEWL
	+""+NEWL
	+""+NEWL;

	/**The inputSubtitleFile.*/
	private static SubtitleFile inputSubtitleFile;
	/**The outputFile.*/
	private static File outputFile;

	private static MainWindow mainWindow = null;


	/**
	 * Method main.
	 * <br><b>Summary:</b><br>
	 * This is the method that launches the Sears main GUI.
	 * @param args  Arguments. Sears can take arguments to make inline operations.
	 *        arguments syntax is:
	 *        java.Main.main [actionKey] [action's parameters]
	 *         
	 */
	public static void main(String[] args) {
		if(args.length == 0){
			launchMainWindow(null);
		}else{
			String actionKey = args[0];
			//Check for an action key word.
			if(actionKey.equals(KEY_DELAY)){
				delayAction(args);
				System.out.println("Delay performed.");
			}else if(actionKey.equals(KEY_RESYNCHRO)){
				resynchroAction(args);
				System.out.println("Resynchro performed.");
			}else if(actionKey.equals(KEY_SPLIT)){
				splitAction(args);
				System.out.println("Split performed.");
			}else if(actionKey.equals(KEY_APPEND)){
				appendAction(args);
				System.out.println("Append performed.");
			}else if(args[0].endsWith(".srt")){ 
				launchMainWindow(args[0]);
			}else{
				//If not action key word has benn recognized, displayUsage.
				displayUsage();
			}
		}
	}
	
	/**
	 * Quick reboot, closed the main window, free the resource and reopened it
	 * <br>Useful when a gui changing is called
	 * @param file 	must be the opened file (could be null)
	 * @param reset if a properties reset is called
	 */
	public static void quickReboot(File file, boolean reset) {
		// TODO 0.9 version
	}

	/**
	 * Method launchMainWindow.
	 * <br><b>Summary:</b><br>
	 * This method permits to launch a Sears mainWindows, with an opened file.
	 * @param file		The file to open, or null if no file is to be opened.
	 */
	private static void launchMainWindow(String file) {
		Trace.trace("Constructing main window", Trace.ALGO_PRIORITY);
		if( !launchMainWindowOnMac() ) {
			mainWindow = new MainWindow();
			Runnable mainWindowVisible = new Runnable() {
				public void run() {	
					mainWindow.setVisible(true);					
				}				
			};
			// THREAD-SAFE, put the setVisible method in the SWING "event dispatching thread"
			EventQueue.invokeLater(mainWindowVisible);
			//If a file has been given open it.
			if(file != null){
				mainWindow.openFile(new File(file));
			}
		}
	}

	/**
	 * If Sears is launched on a Mac Machine, this method try to run 
	 * the implementation of Sears for Mac OS X
	 * @return true if operation is succeeded, false if not
	 */
	private static boolean launchMainWindowOnMac() {
		boolean state = false;
		// if Sears are launched on Mac OS:
		if( Utils.isMacPlatform ){
			// we create an instance of SearsApp class
			// which will make possible to have a better
			// integration of Sears on Mac OS X
			try {
				new SearsApp();
				state = true;
			} catch (AppException e) {
				state = false;
			}
		}
		return state;
	}

	/**
	 * Method <b>displayUsage</b>
	 * <br><b>Summary:</b><br>
	 * This method display the Searc command line usage.
	 */
	private static void displayUsage() {
		System.out.println(USAGE);
	}

	/**
	 * Method <b>appendAction</b>
	 * <br><b>Summary:</b><br>
	 * This method is called to perform an append action.
	 *  Must have at least 4 parameters.
	 *  appen d input_file1 input_file2 [output_file]
	 *  will apply a append to given input file.
	 *  It will append the subtitle of input_file2 with a delay of d.
	 *  result will be write in the input_file, or in output_file, if output_file is given.
	 * Parameters:
	 * @param args  The command line arguments.
	 * 
	 */
	private static void appendAction(String[] args) {
		Trace.trace("Append action.", Trace.MESSAGE_PRIORITY);
		//Split must have at least 4 parameters
		String errorMessage = "";
		if(args.length >= 4){
			double delay =0;
			SubtitleFile fileToAppend = null;
			try{
				//get the delay.
				delay = Double.parseDouble(args[1]);
				//open the file to append, it will be save temporally in inputSubtitleFile.
				errorMessage += openInputFile(args[3]);
				//save it.
				fileToAppend = inputSubtitleFile;
				//open the src file.
				errorMessage += openInputFile(args[2]);
				//If outputfile is precised, read it.
				if(args.length >= 5){
					errorMessage += openOutputFile(args[4]);
				}
			}catch (NumberFormatException e){
				errorMessage += "Delay must be a number."+NEWL;
			}
			//If there is no error, proceed to resynchro.
			if(errorMessage != null && errorMessage.equals("")){
				Trace.trace("Proceed to Append: with delay "+ delay +" on file : "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
				//delay * 1000 because delay is in seconds, with milliseconds.
				inputSubtitleFile.append(fileToAppend, (int) (delay*1000));
				if(outputFile != null){
					Trace.trace("Write result to: "+outputFile, Trace.MESSAGE_PRIORITY);
					try {
						inputSubtitleFile.writeToFile(outputFile);
					} catch (FileConversionException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}else{
					Trace.trace("Write result to: "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
					try {
						inputSubtitleFile.writeToFile(inputSubtitleFile.getFile());
					} catch (FileConversionException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}else{
			errorMessage += "Append action command must have at least 4 arguments."+NEWL+"appen d input_file1 input_file2 [output_file]";
		}
		if(errorMessage != null && !errorMessage.equals("")){
			displayErrorMessage(errorMessage);
		}
	}

	/**
	 * Method <b>splitAction</b>
	 * <br><b>Summary:</b><br>
	 *  java -jar sears.jar split index input_file output_file1 output_file2
	 *   will apply a split to given input file.
	 *   It must have at least 5 parameters.
	 * Parameters:
	 * @param args  The command line arguments.
	 * 
	 */
	private static void splitAction(String[] args) {
		Trace.trace("Split action.", Trace.MESSAGE_PRIORITY);
		//Split must have at least 5 parameters
		String errorMessage = "";
		if(args.length >= 5){
			int index =0;
			File[] destinationFiles = new File[2];
			try{
				//get the index.
				index = Integer.parseInt(args[1]);
				//open the input file.
				errorMessage += openInputFile(args[2]);
				//open first output file
				errorMessage += openOutputFile(args[3]);
				//save it.
				destinationFiles[0] = outputFile;
				//open second outputFile
				errorMessage += openOutputFile(args[4]);
				destinationFiles[1] = outputFile;
			}catch (NumberFormatException e){
				errorMessage += "Index must be a number."+NEWL;
			}
			//If there is no error, proceed to resynchro.
			if(errorMessage != null && errorMessage.equals("")){
				Trace.trace("Proceed to Split: At index "+ index +" on file : "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
				SubtitleFile[] resultFiles = inputSubtitleFile.split(destinationFiles, (index-1), 1);
				//save result files.
				try {
					resultFiles[0].writeToFile(resultFiles[0].getFile());
				} catch (FileConversionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				try {
					resultFiles[1].writeToFile(resultFiles[1].getFile());
				} catch (FileConversionException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}else{
			errorMessage += "Split action command must have at least 5 arguments."+NEWL+"split index input_file output_file1 output_file2";
		}
		if(errorMessage != null && !errorMessage.equals("")){
			displayErrorMessage(errorMessage);
		}
	}

	/**
	 * Method <b>resynchroAction</b>
	 * <br><b>Summary:</b><br>
	 * This method is called to perform a resynchro action.
	 * Must have at least 6 parameters:
	 *  resynch src1 dest1 src2 dest2 input_file [output_file]
	 *  src and dest must be precised under format HH:MM:SS.
	 *  result will be write in the input_file, or in output_file, if output_file is given.
	 * Parameters:
	 * @param args  The command line arguments.
	 * 
	 */
	private static void resynchroAction(String[] args) {
		Trace.trace("Resynchro action.", Trace.MESSAGE_PRIORITY);
		//Resynch must have at least 6 parameters.
		String errorMessage = "";
		if(args.length >= 6){
			int[] resynchParameters = new int[4];
			try{
				//get source and destinations.
				resynchParameters[0]  = SubtitleFile.stringToTime(args[1]);
				resynchParameters[1]  = SubtitleFile.stringToTime(args[2]);
				resynchParameters[2]  = SubtitleFile.stringToTime(args[3]);
				resynchParameters[3]  = SubtitleFile.stringToTime(args[4]);
				//open the input file.
				errorMessage += openInputFile(args[5]);
				//If outputfile is precised, read it.
				if(args.length >= 7){
					errorMessage += openOutputFile(args[6]);
				}
			}catch (NumberFormatException e){
				errorMessage += "Source and destination times must be under HH:MM:SS.MMM format."+NEWL;
			}
			//If there is no error, proceed to resynchro.
			if(errorMessage != null && errorMessage.equals("")){
				Trace.trace("Proceed to Resynch: on file : "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
				inputSubtitleFile.resynchro(resynchParameters);
				if(outputFile != null){
					Trace.trace("Write result to: "+outputFile, Trace.MESSAGE_PRIORITY);
					try {
						inputSubtitleFile.writeToFile(outputFile);
					} catch (FileConversionException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}else{
					Trace.trace("Write result to: "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
					try {
						inputSubtitleFile.writeToFile(inputSubtitleFile.getFile());
					} catch (FileConversionException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}


		}else{
			errorMessage += "Resynchro action command must have at least 6 arguments."+NEWL+"resyn src1 dest1 src2 dest2 input_file [output_file]";
		}
		if(errorMessage != null && !errorMessage.equals("")){
			displayErrorMessage(errorMessage);
		}
	}

	/**
	 * Method <b>delayAction</b>
	 * <br><b>Summary:</b><br>
	 * This method is called to perform a delay action.
	 * Delay action command must have at least 3 arguments.
	 * delay d input_file [output_file]
	 * d is the delay to apply, an integer.
	 * input_file is the file to perform the delay.
	 * [output_file] optionnal, if precised, action will be saved here.
	 * Parameters:
	 * @param args  The command line arguments.
	 * 
	 */
	private static void delayAction(String[] args) {
		Trace.trace("Delay action.", Trace.MESSAGE_PRIORITY);
		//Delay action command must have at least 3 arguments.
		//if not display message error.
		String errorMessage = "";
		//must have at least 3 parameter for a delay.
		if(args.length >= 3){
			try{
				//Get the delay to apply.
				double delay = Double.parseDouble(args[1]);
				//open the input file.
				errorMessage += openInputFile(args[2]);
				//If outputfile is precised, read it.
				if(args.length >= 4){
					errorMessage += openOutputFile(args[3]);
				}
				if(errorMessage != null && errorMessage.equals("")){
					Trace.trace("Proceed to Delay: delay "+delay+ " on file : "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
					//delay * 1000 because delay is in seconds, with milliseconds.
					inputSubtitleFile.delay((int)(delay*1000));
					if(outputFile != null){
						Trace.trace("Write result to: "+outputFile, Trace.MESSAGE_PRIORITY);
						try {
							inputSubtitleFile.writeToFile(outputFile);
						} catch (FileConversionException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}else{
						Trace.trace("Write result to: "+inputSubtitleFile.getFile(), Trace.MESSAGE_PRIORITY);
						try {
							inputSubtitleFile.writeToFile(inputSubtitleFile.getFile());
						} catch (FileConversionException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			}catch(NumberFormatException e){
				errorMessage += "Must enter an integer delay";
			}

		}else{
			errorMessage += "Delay action command must have at least 3 arguments."+NEWL+"delay d input_file [output_file]";
		}
		if(errorMessage != null && !errorMessage.equals("")){
			displayErrorMessage(errorMessage);
		}
	}


	/**
	 * Method <b>openOutputFile</b>
	 * <br><b>Summary:</b><br>
	 * This method open the given file.
	 * Store it in output file.
	 * Parameters:
	 * @param   file                The file to open.    
	 * @return  <b>String</b>       The error message, or "" if no error has been found.
	 */
	private static String openOutputFile(String file) {
		//The result of the Method
		String message ="";
		//Construct file
		File output = new File(file);
		//check that file could be opened.
		if(output.exists() && !output.canWrite()){
			message += "File "+file+" can not be written."+NEWL;
		}else{
			outputFile = output;
		}
		//return the error message.
		return message;
	}


	/**
	 * Method <b>openInputFile</b>
	 * <br><b>Summary:</b><br>
	 * This method open the given file.
	 * Store it in input file.
	 * Parameters:
	 * @param   file                The file to open.    
	 * @return  <b>String</b>       The error message, or "" if no error has been found.
	 * 
	 */
	private static String openInputFile(String file) {
		//The result of the Method
		String message ="";
		//Construct file
		File inputFile = new File(file);
		//check that file could be opened.
		if(!inputFile.canRead()){
			message += "File "+file+" can not be read."+NEWL;
		}
		//Construct subtitleFile
		try {
			inputSubtitleFile = new SrtFile(inputFile, new ArrayList<Subtitle>());
		} catch (FileConversionException e) {
			e.printStackTrace();
		}
		//return the error message.
		return message;
	}


	/**
	 * Method <b>displayErrorMessage</b>
	 * <br><b>Summary:</b><br>
	 * This method display an error.
	 * Parameters:
	 * @param error    The error to display.
	 * 
	 */
	private static void displayErrorMessage(String error) {
		System.out.println("Error : "+NEWL+error+NEWL);
	}
}

