/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

package sears.search.core;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;

import sears.gui.SubtitleCellComponent;
import sears.gui.glassPane.element.GPShapeManager;
import sears.search.gui.PillShape;

/**
 * the <code>PillManager</code> class provides a way to restrict the access to the <code>Pill</code> class.
 * <br>The associated word <i>"Manager"</i> in the class name is not a reference to the <i>"Manager"</i> design pattern.
 * <br>This class is more like a <i>"Factory"</i> ...
 */
public class PillManager implements GPShapeManager {
	// the pill to manage
	private PillShape pill;

	/**
	 * Constructs a new </code>PillManager</code> object
	 */
	public PillManager() {

	}

	/**
	 * Update pill
	 * @param text		a non null and non empty string
	 * @param cell		a cell that represents the row where the occurrence of <tt>text</tt>is
	 * @param index		the index of the first char of the <tt>text</tt> occurrence 
	 * @param location	the pill location
	 */
	public void firePillDataMustBeUpdated(String text, SubtitleCellComponent cell, 
			int index, Point location) {

		if( text != null && text.trim().length() > 0 ) {			
			String realSubString = cell.getSubStringBeginAtIndex(index, text);
			Rectangle bounds = cell.getBoundsBeginAtIndex(index, text);
			// realSubstring != null
			// blackBoxBounds != null blackBoxBounds.trim().length() > 0
			//pill = null;
			if( pill != null ) {
				pill = pill.getPillInstance(cell.getFont(), bounds, realSubString, location);
			} else {	
				pill = new PillShape(cell.getFont(), bounds, realSubString);
				pill.translate(location);
			}			
		}
	}

	/**
	 * Paints a pill in the graphics context given in parameters
	 * @param gr	the graphics context
	 */
	public void paintPill(Graphics2D gr) {
		if( gr != null ) {
			if( pill != null ) {
				pill.paint(gr);
			}
		} // DEVELOPPERS: else...
	}
}