/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.search.data;

import java.util.ArrayList;
import java.util.Iterator;

import sears.file.Subtitle;

/**
 * Search operations 
 *
 */
public class SubtitleFile {
	
	private ArrayList<Subtitle> listOfSubtitles;
	
	/**
	 * 
	 * @param aListOfSubtitles
	 */
	public SubtitleFile(ArrayList<Subtitle> aListOfSubtitles) {
		setListOfSubtitles(aListOfSubtitles);
	}
	
	/**
	 * 
	 * @param aListOfSubtitles
	 */
	public void setListOfSubtitles(ArrayList<Subtitle> aListOfSubtitles) {
		if( aListOfSubtitles == null ) {
			// ensures that the list is not null
			// seek methods could not performs on a null array
			throw new NullPointerException("the list of subtitle cannot be null here");
		}
		listOfSubtitles = aListOfSubtitles;
	}
	
	/**
	 * Gets the count of all the text occurrences given in parameters
	 * @param text	a non null and empty string 
	 * @return		the occurrences count
	 * @throws 		NullPointerException if <tt>text</tt> is null
	 * @throws 		IllegalArgumentException if <tt>text</tt> is an empty string
	 */
	public int getCountOfOccurrencesOfText(String text) {
		if( text == null ) {
			throw new NullPointerException("cannot search a null object");
		}
		if( text.trim().length() == 0 ) {
			throw new IllegalArgumentException("cannot search an empty string");
		}
		int count = 0;
		text = text.toLowerCase();		
		Subtitle subtitle = null;
		String str = "";
		Iterator<Subtitle> it = listOfSubtitles.iterator();
		while( it.hasNext() ) {
			subtitle = (Subtitle) it.next();			
			str = subtitle.getSubtitle().toLowerCase();
			int index = str.indexOf(text);
			while( index != -1 ) {
				count++;
				index = str.indexOf(text, index + 1);
			}
		}
		return count;
	}
	
	/**
	 * Constructs and return a list of all row that contains at least one occurrence of <tt>text</tt>
	 * @param text	a non null and a non empty string string
	 * @return		the list of all the row that contains at least one occurrence of <tt>text</tt>
	 */
	public ListOfRow getAllRowWhichContainsAtLeastOneOccurrenceOfText(String text) {
		if( text == null ) {
			throw new NullPointerException("cannot search a null object");
		}
		if( text.trim().length() == 0 ) {
			throw new IllegalArgumentException("cannot search an empty string");
		}
		// !! SIZE STATIC METHOD -->
		int size = 100;
		// <--
		// index of row in the list of subtitles:
		int index = -1;
		ListOfRow listOfRow = new ListOfRow(size);
		text = text.toLowerCase();		
		Subtitle subtitle = null;
		String str = "";
		Iterator<Subtitle> it = listOfSubtitles.iterator();
		while( it.hasNext() ) {
			index++;
			subtitle = (Subtitle) it.next();			
			str = subtitle.getSubtitle().toLowerCase();
			int result = str.indexOf(text);
			if( result != -1 ) {
				listOfRow.add(index);
			}
		}
		return listOfRow;
	}
}