/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.search.data;

/**
 * Iterator class for manage int array of char index
 */
public class Window implements Cloneable {
	
	private static final int[] NON_NULL_WINDOW = {-1};
	
	private int[] window;
	private int index;
	private static Window emptyWindow;
	
	/**
	 * Constructs an empty window
	 */
	public Window() {
		index = -1;
		window = NON_NULL_WINDOW;
		emptyWindow = this;
	}
	
	/**
	 * Constructs a window with parameters given
	 * @param text	the text
	 * @param str	the str to search in <tt>text</tt>
	 */
	public Window(String text, String str) {
		setWindowForGettingNextElement(text, str);
	}
	
	/**
	 * Sets a new window
	 * @param text	the text
	 * @param str	the string to search in <tt>text</tt>
	 * @param next	the direction of search
	 */
	private void setWindow(String text, String str, boolean next) {
		if( text == null || str == null ) {
			throw new NullPointerException("one or the two string given on parameters is null");
		}	
		if( text.trim().length() == 0 || str.trim().length() == 0 ) {
			// DEVELOPER:
			// TODO
		}
		// format all string
		text = formatString(text);
		str = formatString(str);
		
		window = getBeginCharIndexOfAllOccurencesOfStringInText(text,str);
		if( next ) {
			index = -1;
		} else {
			index = window.length;
		}
	}
	
	/**
	 * Sets a new window
	 * @param text 	the text
	 * @param str	the string to search in <tt>text</tt>
	 */
	public void setWindowForGettingNextElement(String text, String str) {
		setWindow(text,str,true);
	}	

	/**
	 * Sets a new window
	 * @param text	the text
	 * @param str	the string to search in <tt>text</tt>
	 */
	public void setWindowForGettingPreviousElement(String text, String str) {
		setWindow(text,str,false);
	}	
	
	/**
	 * A char index is always positive, this method returns the next char index
	 * @return the next char index or -1 if there's no next char index
	 */
	public int getNextCharIndex() {
		int charIndex = -1;
		if( index + 1 < window.length ) {
			charIndex = window[++index];
		}
		return charIndex;
	}
	
	/**
	 * A char index is always positive, this method returns the previous char index
	 * @return the previous char index or -1 if there's no previous char index
	 */
	public int getPreviousCharIndex() {
		int charIndex = -1;
		if( index - 1 >= 0 ) {
			charIndex = window[--index];
		}
		return charIndex;
	}
	
	// CAREFULL: method not synchronized !
	// text and str could be valid !!!
	/**
	 * Returns a new window.
	 * @param	text	the text
	 * @param	str		the string to search in <tt>text</tt>
	 * @return			an array of char index 
	 */
	private int[] getBeginCharIndexOfAllOccurencesOfStringInText(String text, String str) {
		// text is valid 
		// DEVELOPPER: text and str could be valid !!!
		int[] charIndex;
		int indexOf = text.indexOf(str);
		// FIRST LOOP, compute array's length,
		// take care about the first cell: 'row':
		int count = 0;
		while( indexOf != -1 ) {
			count++;
			indexOf = text.indexOf(str, indexOf + 1);
		}
		charIndex = new int[count];
		int lastIndexOf = 0;
		// SECOND LOOP, fill the int array:
		for( int i=0;i<count;i++) {
			charIndex[i] = text.indexOf(str, lastIndexOf);
			lastIndexOf = charIndex[i] + 1;
		}
		return charIndex;
	}
	
	/**
	 * Formats the string given in parameters and returns it:
	 * <ul>
	 * <li> string is convert to lower case </li>
	 * <li> removes all the line separator </li>
	 * </ul>
	 * @param str 	the string to format
	 * @return		the formatted string
	 */
	private String formatString(String str) {
		str = str.toLowerCase();
		str = str.replaceAll(System.getProperty("line.separator") , " ");
		return str;
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		Object clone = null;
		try {
			clone = super.clone();
		} catch (CloneNotSupportedException e) {
		}
		return clone;
	}
	
	/**
	 * Returns an empty cloned <tt>Window</tt> object
	 * or a new instance if clone method failed
	 * @return an empty cloned <tt>Window</tt> object
	 */
	public Window getEmptyClonedWindow() {
		Window clone = null;
		if( emptyWindow == null ) {
			emptyWindow = new Window();
		} else {
			clone = (Window) emptyWindow.clone();
			if( clone == null ) {
				clone = new Window();
			}
		}
		return clone;
	}
}