/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.search.gui;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import sears.gui.glassPane.element.GPShape;

/**
 * <br>A <tt>PillShape</tt> is a rounded bordered rectangle with text inside.
 */
public class PillShape implements GPShape, Cloneable {
	
	// DEVELOPPER: use for AffineTransform
	//protected static final int SCALE = 2;	
	
	// CONSTANTS:
	// ( EXPANDED % 2 ) must be equals to 0
	protected static final int EXPANDED = 6;
	/** expanded height factor */
	protected static final int EXPANDED_HEIGHT = EXPANDED + 4;
	/** expanded width factor */
	protected static final int EXPANDED_WIDTH = EXPANDED;
	
	protected static final float DECAY = 3.5f;
	/** arc factor, use or the rounded rectangle*/
	protected static final int ROUNDED_ARC = 12;
	/** stroke use for line limit */
	protected static final float BORDER_STROKE = 2.2f;
	/** pill foreground color */
	protected static final Color PILL_FOREGROUND_COLOR = Color.WHITE;
	/** pill background color */
	protected static final Color PILL_BACKGROUND_COLOR = new Color(255, 165, 0);
	
	// use to draw ...
	protected static final Stroke STROKE = new BasicStroke(BORDER_STROKE);
	protected static final Composite ALPHA_COMPOSITE = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.15f );
	
	// see TextLayout#getBlackBoxBounds() method:
	private Rectangle bounds;
	private String text;
	
	private Shape pill;
	private Shape shadow;
	
	// graphics context coordinates:
	private int x;
	private int y;
	
	private Font font;
	
	// DEVELOPPER: Carefull this variable could be null, 
	// you must used their accessor instead of its directly
	private Color backgroundColor = null;
	private Color foregroundColor = null;

	/**
	 * Creates a new instance of <code>PillShape</code> by passing ...
	 * 
	 * <p><b>WARNING:</b>
	 * <br><i>This class is a beta class, so there's some limitation:</i>
	 * <br>The parameter <code>bounds</code> must be the bounds of the <code>text</code> string
	 * <br>There's no control of any sort about this coherence, so use at your own risk.
	 * 
	 * @param font		the font
	 * @param bounds	the text bounds
	 * @param text		the text
	 * @throws NullPointerException if at least one of the given parameters is <tt>null</tt>
	 */
	public PillShape(Font font, Rectangle bounds, String text) {
		setData(font, bounds, text);
		translate(0,0);
		initPillElement();
	}
	
	/**
	 * Sets the background color for the pill
	 * @param background the background color, if <tt>null</tt> the default background color will be used
	 */
	public void setBackground(Color background) {
		backgroundColor = background;
	}
	
	/**
	 * Sets the foreground color for the pill
	 * <br>Foreground color is used for the text inside and pill's border
	 * @param foreground the foreground color, if <tt>null</tt> the default foreground color will be used
	 */
	public void setForeground(Color foreground) {
		foregroundColor = foreground;
	}
	
	/**
	 * Gets the foreground color
	 * @return the foreground color
	 */
	public Color getForeground() {
		if( foregroundColor == null ) {
			foregroundColor = PILL_FOREGROUND_COLOR;
		}
		return foregroundColor;
	}
	
	/**
	 * Gets the background color
	 * @return the foreground color
	 */
	public Color getBackground() {
		if( backgroundColor == null ) {
			backgroundColor = PILL_BACKGROUND_COLOR;
		}
		return backgroundColor;
	}
	
	/**
	 * Sets the data used by this object
	 * @param font		the font to rendering the text inside the pill
	 * @param bounds	the text's bounds
	 * @param text		the text to display in the pill
	 * @throws NullPointerException if at least one of the parameters is <tt>null</tt>
	 */
	private void setData(Font font, Rectangle bounds, String text) {
		if( font == null || bounds == null || text == null ) {
			throw new NullPointerException("An argument given in parameters is null");
		}
		this.font = font;
		this.bounds = bounds;
		this.text = text;
	}
	
	/**
	 * Constructs and initializes all the elements that is needed to draw the pill
	 */
	private void initPillElement() {
		// blacBoxBounds isn't null:
		Rectangle2D baseBounds = bounds;
		// PILL: rounded bounds:
		RoundRectangle2D roundedScaledBaseBounds = new RoundRectangle2D.Float();
		roundedScaledBaseBounds.setRoundRect(
				baseBounds.getX() - EXPANDED_WIDTH/2,
				baseBounds.getY() - baseBounds.getHeight() - EXPANDED_HEIGHT/2,
				baseBounds.getWidth() + EXPANDED_WIDTH,
				baseBounds.getHeight() + EXPANDED_HEIGHT,
				// arc
				ROUNDED_ARC, ROUNDED_ARC);
	
		// shadow
		RoundRectangle2D roundedShadow = (RoundRectangle2D) roundedScaledBaseBounds.clone();
		roundedShadow.setRoundRect(
				roundedShadow.getX() - DECAY/2, 
				roundedShadow.getY() - DECAY/2, 
				roundedShadow.getWidth() + DECAY, 
				roundedShadow.getHeight() + DECAY, 
				roundedShadow.getArcWidth(), 
				roundedShadow.getArcHeight());
		
		// INIT VARIABLES:
		pill = roundedScaledBaseBounds;
		shadow = roundedShadow;
	}
	
	/**
	 * Returns a <code>RoundRectangle2D</code> object, represents the real bounds of this shape
	 * @see sears.gui.glassPane.element.GPShape#getBounds()
	 */
	public Shape getBounds() {
		return pill;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.glassPane.element.GPShape#paint(java.awt.Graphics2D)
	 */
	public void paint(Graphics2D gr) {
		if( gr != null ) {
			Color grColor = gr.getColor();
			Composite defaultComposite = gr.getComposite();
			RenderingHints defaultRendering = gr.getRenderingHints();
			Stroke defaultStroke = gr.getStroke();
			// move graphics to location:
			gr.translate(x, y);			
			// PAINT:
			gr.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
			// paint shadow
			gr.setComposite(ALPHA_COMPOSITE);		
			gr.setPaint(Color.BLACK);
			gr.fill(shadow);
			gr.setComposite(defaultComposite);
			// paint pill's shape base
			gr.setColor(getBackground());
			gr.fill(pill);
			// paint inside text:
			gr.setColor(getForeground());
			//gr.draw(blacBoxBounds);
			gr.setFont(this.font);
			gr.drawString(text, (int) bounds.x, 
			 					(int) bounds.y);
			// paint borders:
			gr.setStroke(new BasicStroke(BORDER_STROKE));
			gr.draw(pill);
			// return the graphics state:
			gr.setColor(grColor);
			gr.setRenderingHints(defaultRendering);
			gr.setStroke(defaultStroke);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.glassPane.element.GPShape#translate(int, int)
	 */
	public void translate(int x, int y) {
		this.x = x;
		this.y = y;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.glassPane.element.GPShape#translate(java.awt.Point)
	 */
	// UPDATE GPSHAPE Javadoc
	public void translate(Point point) {
		if( point != null ) {
			translate(point.x, point.y);
		} else {
			translate(0, 0);
		}
	}	
	
	// -------------
	// CLONE METHODS
	// -------------

	/**
	 * Calls the super method and catch the clone exception if occurs.
	 * <br>The result of catching exception is that a null object could be returned
	 * 
	 * @return a clone <code>Object</code> or null if the super method failed to clone the object
	 * @see java.lang.Object#clone()
	 */
	protected Object clone() {
		Object clone = null;
		try{ 
			clone = super.clone(); 
		} catch(Exception e) {
			// nothing is done
			// clone method return null
		} 
		return clone;
	}

	/**
	 * Gets a clone of the <code>PillShape</code> object
	 * @return	a <code>PillShape</code> object or null if clone() method failed
	 */
	protected PillShape getClone() {
		return (PillShape) clone();
	}
	
	/**
	 * Gets an instance of the object with a new data set
	 * <br>Optimization...
	 * @param newFont		the font to associate to the pill 
	 * @param newBounds		the text bounds
	 * @param newText		the text
	 * @param newLocation		the pill location
	 * @return 				a <code>Pill</code> object
	 */
	public PillShape getPillInstance(Font newFont, Rectangle newBounds, String newText, Point newLocation) {
		PillShape clone = null;
		
		if( newFont.equals(font) && newText.contentEquals(text) && bounds != null ) {
			clone = this;			
			translate(newLocation.x + (newBounds.x - bounds.x), newLocation.y);
		} else {
			// TRY TO CLONE THIS:
			clone = getClone();
			if( clone != null ) {
				clone.setData(newFont, newBounds, newText);
				clone.initPillElement();
			} else {
				clone = new PillShape(newFont, newBounds, newText);
			}
			clone.translate(newLocation);
		}
		return clone;
	}
}