/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

package sears.search.gui;

import java.awt.Graphics2D;

import javax.swing.JViewport;

import sears.gui.glassPane.DefaultViewportGlassPane;
import sears.gui.glassPane.ViewportGlassPane;
import sears.search.core.PillManager;

/**
 * An object of this class represents a virtual glass pane over a viewport
 * <br>It allows to display <code>Pill</code> objects over it graphics context.
 * @see DefaultViewportGlassPane
 */
public class SearchViewportGlassPane extends DefaultViewportGlassPane {

	private static final long serialVersionUID = 2163386443816906875L;

	// to manage Pill object
	private PillManager pm;

	/**
	 * Creates a new virtual glass pane
	 * 
	 * @param viewport		the view port, needed for display the virtual glass pane
	 * @param pillManager	a pill manager, to manage pill position over the glass pane
	 * 
	 * @throws 				NullPointerException if view port is null
	 * @throws 				NullPointerException if pillMananger is null
	 * 
	 * @see ViewportGlassPane#ViewportGlassPane(JViewport)
	 * @see DefaultViewportGlassPane
	 */
	public SearchViewportGlassPane(JViewport viewport, PillManager pillManager) {
		super(viewport);
		if( pillManager == null ) {
			throw new NullPointerException("PillManager object cannot be null");
		}
		pm = pillManager;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.glassPane.DefaultViewportGlassPane#paintChildrenWithGraphics(java.awt.Graphics2D)
	 */
	protected void paintChildrenWithGraphics(Graphics2D gr) {
			pm.paintPill(gr);
	}

	/**
	 * Wrap method, calls the <code>repaint()</code> method
	 * <br>Avoid confusion ( and makes sure a repaint is needed )
	 */
	public void updatePill() {
		repaint();
	}
}