/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is availbale under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////


package sears.tools;

import java.io.*;


/**
 * This class gives methods for read information into an Info.plist XML file (without a parser)
 * <br> on a Mac OS X system.
 */
public class InfoPlist{

	private static final String infoPlist = "/Contents/Info";
	private static final String executableParentSubPath = "/Contents/MacOS/";
	private static final String separator = System.getProperty("file.separator");

	public InfoPlist(){
		//empty...
	}

	/**
	 * this method returns the path of the executable file contains in a .app bundle
	 * <br> with the path of .app bundle given in parameter.
	 *
	 * @param pathOfApp the path of the .app bundle.
	 * @return 			a String, path of the executable contains in .app bundle,
	 * <br><b>null</b> if it isn't found or if the method was called out of Mac OS X system.
	 */
	public static String getExecutablePath(String pathOfApp) {
		String result = getBundleExecutableName(pathOfApp);
		if(result != null) {
			result = pathOfApp + separator + executableParentSubPath + result;
		}
		return result;
	}

	/**
	 * this method returns the name of the executable file contains in a .app bundle
	 * <br> with the path of .app bundle given in parameter.
	 *
	 * @param pathOfApp the path of the .app bundle.
	 * @return 			the name of the executable file,
	 * <br><b>null</b> if it isn't found or if the method was called out of Mac OS X system.
	 */
	public static String getBundleExecutableName(String pathOfApp) {
		String result = null;
		// if method was called on a Mac OS X platform:
		if(Utils.isMacPlatform) {
			// unix command to read the key -CFBundleExecutable- into an Info.plist file
			String[] command = {"defaults", "read", pathOfApp + infoPlist, "CFBundleExecutable"};

			try {
				// we try to execute the unix command line:
				Process process = Runtime.getRuntime().exec(command);			

				BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
				// we read the result:
				result = bufferedReader.readLine();           
				bufferedReader.close();

			} catch (IOException e) {
				e.printStackTrace();
			}		
		}
		return result;
	}
}	