//////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.tools;

/**
 * LinearInterpolation This class permits to perform linear interpolation by
 * segments.
 */
public class LinearInterpolation {
	private double[] x;

	private double[] y;

	private double[] a;

	private double[] b;

	public LinearInterpolation(double[] x, double[] y) {
		this.x = x;
		this.y = y;
		computeInterpolation();
	}

	private void computeInterpolation() {
		if (x.length == y.length && x.length > 1) {
			// We have to compute the a and b values for each intervals.
			a = new double[x.length - 1];
			b = new double[y.length - 1];
			for (int i = 0; i < a.length; i++) {
				a[i] = (y[i + 1] - y[i]) / (x[i + 1] - x[i]);
				b[i] = y[i] - x[i] * a[i];
			}
		}else{
			throw new IllegalArgumentException("Linear interpolation must have at least 2 parameters");
		}
	}

	public void resetDatas(double[] x, double[] y) {
		this.x = x;
		this.y = y;
		computeInterpolation();
	}

	public double interpolate(double xx) {
		// the result of the method.
		double result = 0;
		// first is to know the interval of xx.
		int index = 0;
		if (xx <= x[0]) {
			// If x is before first point, take the first interpolation.
			index = 0;
		} else if (xx >= x[x.length-1]) {
			// If x is after last point, take the last point.
			index = x.length - 2;
		} else {
			while (xx >= x[index + 1]) {
				index++;
			}
		}
		// We have the index.
		result = a[index] * xx + b[index];
		// return the result
		return result;
	}
}
