/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////


package sears.tools;

/**
 * Class Trace.
 * <br><b>Summary:</b><br>
 * A class to show soft traces.
 */
public class Trace {
    public static final int ERROR_PRIORITY = 0;

    public static final int WARNING_PRIORITY = 1;

    public static final int MESSAGE_PRIORITY = 2;

    public static final int METHOD_PRIORITY = 3;

    public static final int ALGO_PRIORITY = 4;

    public static int DISPLAY_TRACE_LEVEL = 1;

    /**
     * Constructor Trace.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     */
    public Trace() {
    }

    /**
     * Method trace.
     * <br><b>Summary:</b><br>
     * Display a message using a priority level.
     * @param message       The message to display.
     * @param level         The message level.
     */
    public static void trace(String message, int level) {
        if (level <= DISPLAY_TRACE_LEVEL) {
            String messageToDisplay = "";
            switch (level) {
                case ERROR_PRIORITY:
                    messageToDisplay += "[ERR] ";
                    break;
                case WARNING_PRIORITY:
                    messageToDisplay += "[WAR] ";
                    break;
                case MESSAGE_PRIORITY:
                    messageToDisplay += "[MES] ";
                    break;
                case METHOD_PRIORITY:
                    messageToDisplay += "[MET] ";
                    break;
                case ALGO_PRIORITY:
                    messageToDisplay += "[ALG] ";
                    break;
                default:
                    messageToDisplay += "[MES] ";
            }
            messageToDisplay += (String) message;
            System.out.println(messageToDisplay);
        }
    }

    /**
     * Method trace.
     * <br><b>Summary:</b><br>
     * Trace a message with default priority level: MESSAGE_PRIORITY
     * @param message   The message to trace.
     */
    public static void trace(String message) {
        trace(message, MESSAGE_PRIORITY);
    }
    
    /**
     * Method trace.
     * <br><b>Summary:</b><br>
     * Trace a message with Warning priority level.
     * @param message   The message to trace.
     */
    public static void warning(String message){
    	trace(message, WARNING_PRIORITY);
    }
    
    /**
     * Method trace.
     * <br><b>Summary:</b><br>
     * Trace a message with error priority level.
     * @param message   The message to trace.
     */
    public static void error(String message){
    	trace(message, ERROR_PRIORITY);
    }
}
