/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is availbale under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Class Utils.
 * <br><b>Summary:</b><br>
 * This class provides some useful subtitles and general tools.
 */
public class Utils {
	
	// EXTENSION CONSTANTS
	/** Extension of jpeg file*/
    public static final String jpeg = "jpeg";
    /** Extension of jpg file*/
    public static final String jpg = "jpg";
    /** Extension of gif file*/
    public static final String gif = "gif";
    /** Extension of tiff file*/
    public static final String tiff = "tiff";
    /** Extension of tif file*/
    public static final String tif = "tif";
    /** Extension of png file*/
    public static final String png = "png";
    
    // LINE SEPARATOR
    /** Line separator, system dependent */
    public static final String LINE_SEPARATOR = System.getProperty("line.separator");    
    /** DOS line separator, useful for some DVD/DVIX player */
    public static final String DOS_LINE_SEPARATOR = "\r\n";
    
    // PLATFORM RECOVERY
    // to determine on which platform Sears is running
    /** OS name */
	public final static String OSname= System.getProperty("os.name");    
	/** if is Linux OS */
	public final static boolean isLinuxPlatform = OSname.toLowerCase().startsWith("linux");
	/** if is Mac OS */
	public final static boolean isMacPlatform = OSname.toLowerCase().startsWith("mac");
	/** if is Windows OS */
	public final static boolean isWindowsPlatform = OSname.toLowerCase().startsWith("windows");    

	/**
	 * Default constructor
	 */
    public Utils() {
    }

    /**
     * Method getExtension.
     * <br><b>Summary:</b><br>
     * return the file extension of a given file.
     * ex: from test.jpg, return <b>String</b> jpg
     * @param file              The <b>File</b> file to get the extension
     * @return <b>String</b>    The file extension.
     */
    public static String getExtension(File file) {
        String ext = "";
        String s = file.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 && i < s.length() - 1)
            ext = s.substring(i + 1).toLowerCase();
        return ext;
    }
    
    // Note: could be deleted ...
    /**
     * Method isWindowsPlatform.
     * <br><b>Summary:</b><br>
     * Try to determine whether this application is running under Windows
     * or some other platform by examing the "os.name" property.
     *
     * @return true if this application is running under a Windows OS
     */
    public static boolean isWindowsPlatform()
    {
    	boolean isWindows = false; 
        String os = System.getProperty("os.name");
        if ((os != null) && (os.toLowerCase().startsWith("windows")))
        	isWindows = true;

        return isWindows;
    }
    
    /**
     * Method copyStream.
     * <br><b>Summary:</b><br>
     * Copies src stream to dst stream.
     * If the dst stream does not exist, it is created.
     * @param src              The <b>File</b> source stream
     * @param dst              The <b>File</b> destination stream
     */
   public static void copyStream(InputStream src, OutputStream dst) throws IOException {
        // Transfer bytes from in to out
        byte[] buf = new byte[1024];
        int len;
        while ((len = src.read(buf)) > 0) {
        	dst.write(buf, 0, len);
        }
        src.close();
        dst.close();
    }
    
    /**
     * Method copyFile.
     * <br><b>Summary:</b><br>
     * Copies src file to dst file.
     * If the dst file does not exist, it is created.
     * @param src              The <b>File</b> source file
     * @param dst              The <b>File</b> destination file
     */
   public static void copyFile(File src, File dst) throws IOException {
        copyStream(new FileInputStream(src),new FileOutputStream(dst));
    }
   
   /**
    * Method formatTime.
    * <br><b>Summary:</b><br>
    * Format time in second to "HH:MM:SS".
    * @param timeInSecond The <b>int</b> time in second
    * @return the formated time
    */
   public static String formatTime(int timeInSecond) {
	   String time = "";
	   // Calculate time
	   int hours = timeInSecond / 3600;
	   int minutes = ((timeInSecond - (hours * 3600)) / 60);
	   int seconds = (timeInSecond - (hours * 3600) - (minutes * 60));
	   time = Integer.toString(seconds);
	   if(seconds < 10)
		   time = "0" + time;
	   time = ":" + time;
	   time = Integer.toString(minutes) + time;
	   if(minutes < 10)
		   time = "0" + time;
	   time = ":" + time;
	   time = Integer.toString(hours) + time;
	   if(hours < 10)
		   time = "0" + time;
	   
	   return time;
   }

 }
