/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//////////////////////////////////////////////////////
// this file comes from the EawtWrapper librairie 	//
// information: www.daem0n.fr/eawt 					//
//////////////////////////////////////////////////////

package sears.tools.eawt;

import java.lang.reflect.Method;

/**
 * this class is a wrap class of class <strong>com.apple.eawt.ApplicationEvent</strong> 
 * <br>With this class we can access to the method of the Apple class.
 */
public class ApplicationEventWrapper {
	/** com.apple.eawt.ApplicationEvent */
	@SuppressWarnings("unchecked")
	private Class classApplicationEvent = null;

	// all the methods of com.apple.eawt.ApplicationEvent:
	private Method applicationEventSetHandled = null;
	private Method applicationEventIsHandled = null;
	private Method applicationGetFilename = null;

	// use to call the methods, applicationEventObject instanceof ApplicationEvent
	private Object applicationEventObject = null;

	/**
	 * Construct a new ApplicationEventWrapper
	 */
	@SuppressWarnings("unchecked")
	public ApplicationEventWrapper() {		
		try {
			
			// we get back com.apple.eawt.ApplicationListener class:
			classApplicationEvent = Class.forName("com.apple.eawt.ApplicationEvent");
			// and its methods:
			applicationEventSetHandled = classApplicationEvent.getMethod("setHandled", boolean.class);
			applicationEventIsHandled = classApplicationEvent.getMethod("isHandled");
			applicationGetFilename = classApplicationEvent.getMethod("getFilename");
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * set the ApplicationEvent object
	 * @param anApplicationEventObject must be instance of ApplicationEvent
	 */
	public void applicationEventObject(Object anApplicationEventObject) {
		applicationEventObject = anApplicationEventObject;
	}

	/**
	 * <u>com.apple.eawt.ApplicationEvent#getFilename()</u>
	 * <br>Provides the filename associated with a particular AppleEvent.
	 * <br><i>If applicationEventObject isn't set or is not instance of ApplicationEvent or is null,
	 * <br>null is return.</i>
	 * @return the full path to the file associated with the event, if applicable, otherwise null
	 */
	public String getFilename() {
		String filename = null;
		// check conditions:
		if(applicationEventObject != null && 
				classApplicationEvent.isInstance(applicationEventObject)) {
			try {
				filename = (String)applicationGetFilename.invoke(applicationEventObject);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return filename;
	}
	
	/**
	 * <u>com.apple.eawt.ApplicationEvent#isHandled()</u>
	 * <br>Determines whether an ApplicationListener has acted on a particular event.
	 * <br><i>If applicationEventObject isn't set or is not instance of ApplicationEvent or is null,
	 * <br>false is return.</i>
	 * @return true if the event has been handled, otherwise false
	 */
	public boolean isHandled() {
		boolean result = false;
		// check conditions:
		if(applicationEventObject != null && 
				classApplicationEvent.isInstance(applicationEventObject)) {
			try {
				result = (Boolean)applicationEventIsHandled.invoke(applicationEventObject);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return result;
	}
	
	/**
	 * <u>com.apple.eawt.ApplicationEvent#setHandled(boolean)</u>
	 * <br>Sets the state of the event.
	 * <br><i>If applicationEventObject isn't set or is not instance of ApplicationEvent or is null,
	 * <br>this method is a no op</i>
	 * @param state true if the event has been handled, otherwise false.
	 */
	public void setHandled(boolean state) {
		if(applicationEventObject != null && 
				classApplicationEvent.isInstance(applicationEventObject)) {
			try {
				applicationEventSetHandled.invoke(applicationEventObject, state);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}