/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//////////////////////////////////////////////////////
// this file comes from the EawtWrapper librairie 	//
// information: www.daem0n.fr/eawt 					//
//////////////////////////////////////////////////////

package sears.tools.eawt;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.EventListener;

/**
 * Handler for communicate with com.apple.eawt.ApplicationListener class
 */
public class ApplicationListenerHandler implements InvocationHandler {
	
	/** com.apple.eawt.ApplicationListener */
	@SuppressWarnings("unchecked")
	private Class classApplicationListener = null;
	// instance of application listener, use for call its methods
	private ApplicationListener applicationListener = null;
	
	/**
	 * Construct a new handler
	 * @param listener an instance of ApplicationListener subclass
	 */
	public ApplicationListenerHandler(ApplicationListener listener) {
		applicationListener = listener;
		try {
			// we get back com.apple.eawt.ApplicationListener class:
			classApplicationListener = Class.forName("com.apple.eawt.ApplicationListener");			
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * return the class object which represents com.apple.eawt.ApplicationListener class
	 * @return com.apple.eawt.ApplicationListener 'Class object'
	 */
	@SuppressWarnings("unchecked")
	public Class getApplicationListenerInterface() {
		return classApplicationListener;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method, java.lang.Object[])
	 */
	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
		// we get back the method name:
		String methodName = method.getName();
		// and compare with the ApplicationListener methods:
		if(methodName.equals("handleAbout")) {
			applicationListener.handleAbout(args[0]);
		} else if(methodName.equals("handleOpenApplication")) {
			applicationListener.handleOpenApplication(args[0]);
		} else if(methodName.equals("handleOpenFile")) {
			applicationListener.handleOpenFile(args[0]);
		} else if(methodName.equals("handlePreferences")) {
			applicationListener.handlePreferences(args[0]);
		} else if(methodName.equals("handlePrintFile")) {
			applicationListener.handlePrintFile(args[0]);
		} else if(methodName.equals("handleQuit")) {
			applicationListener.handleQuit(args[0]);
		} else if(methodName.equals("handleReOpenApplication")) {
			applicationListener.handleReOpenApplication(args[0]);
		}		
		// always null:
		return null;
	}
	
	/**
	 * this method returns an instance of com.apple.eawt.ApplicationListener interface,
	 * <br>wrapped by EventListener interface...
	 * @param listener will be an instance of the Application listener implementation class
	 * @return an instance of com.apple.eawt.ApplicationListener, null if...
	 */
	@SuppressWarnings("unchecked")
	public static EventListener getWrappedApplicationListenerInstance(ApplicationListener listener) {
		EventListener returnedListener = null;
		
		ApplicationListenerHandler handler = 
			new ApplicationListenerHandler(listener);
		// we get back the com.java.eawt.ApplicationListener:
		Class applicationListener = handler.getApplicationListenerInterface();
		// and create an Object instance of EventListener,
		// an instance of the com.java.eawt.ApplicationListener interface:
		returnedListener = (EventListener) Proxy.newProxyInstance(
				ClassLoader.getSystemClassLoader(),
				new Class[] { applicationListener },
				handler );
		return returnedListener;
	}
}