/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////


//this file comes from the EawtWrapper librairie 	//
//information: www.daem0n.fr/eawt 					//


package sears.tools.eawt;

import java.lang.reflect.Method;
import java.util.EventListener;

/**
 * this class is a wrap class of class <strong>com.apple.eawt.Application</strong>
 * <br>With this class we can access to the method of the Apple class.
 */
public class ApplicationWrapper {

	/** com.apple.eawt.Application */
	@SuppressWarnings("unchecked")
	private Class classApplication = null;
	/** com.apple.eawt.ApplicationListener */
	@SuppressWarnings("unchecked")
	private Class classApplicationListener = null;

	// all the methods of class Application:
	private static Method applicationGetApplication = null;
	private Method applicationAddPreferencesMenuItem = null;
	private Method applicationSetEnabledPreferencesMenu = null;
	private Method applicationAddApplicationListener = null;

	/** null array of objects, use to past null array argument */
	private static final Object[] nullObject = null;

	/** instance of Application class */
	private Object app = null;

	/**
	 * Construct a new ApplicationWrapper object,
	 * get back Classes and methods
	 * @throws AppException if an error occurs during initialization
	 */
	@SuppressWarnings("unchecked")
	public ApplicationWrapper() throws AppException {		
		// we get back classes
		try {
			classApplication = Class.forName("com.apple.eawt.Application");
			classApplicationListener = Class.forName("com.apple.eawt.ApplicationListener");
			// and the interrested methods
			applicationGetApplication = classApplication.getMethod("getApplication");
			applicationAddPreferencesMenuItem = classApplication.getMethod("addPreferencesMenuItem");
			applicationSetEnabledPreferencesMenu = classApplication.getMethod("setEnabledPreferencesMenu", boolean.class);
			applicationAddApplicationListener = classApplication.getMethod("addApplicationListener", classApplicationListener);
			// we create an instance of Application class with a call to getApplication() static method:
			app = applicationGetApplication.invoke(nullObject, nullObject);
		} catch (Exception e) {
			throw new AppException();
		} 	
	}

	/**
	 * this method return an instance object of the class
	 * @return an instance of the class
	 * @throws AppException if instantiation failed
	 */
	public static ApplicationWrapper getApplication() throws AppException {		
		return new ApplicationWrapper();
	}

	/**
	 * <u>com.apple.eawt.Application#addPreferencesMenuItem()</u>
	 * <br>Adds the Preferences item to the application menu if the item is not already present.
	 */
	public void addPreferencesMenuItem() {
		try {
			if(app != null) {
				applicationAddPreferencesMenuItem.invoke(app, nullObject);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}	
	}

	/**
	 * <u>com.apple.eawt.ApplicationListener#setEnabledPreferencesMenu(boolean)</u>
	 * <br>Enables the About item in the application menu. 
	 * @param enable specifies whether the About item in the application menu should be enabled (true) or not (false)
	 */
	public void setEnabledPreferencesMenu(boolean enable) {
		try {
			if(app != null) {
				applicationSetEnabledPreferencesMenu.invoke(app, enable);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * <u>com.apple.eawt.ApplicationListener#addApplicationListener(com.apple.eawt.ApplicationEvent)</u>
	 * <br>Adds the specified ApplicationListener as a receiver of callbacks from this class.
	 * <i>if listener is not an implementation of ApplicationListener this method is no op</i>
	 * @param listener	an implementation of ApplicationListener that handles 
	 * 					ApplicationEvents generated by this class
	 */
	public void addApplicationListener(EventListener listener) {
		try {
			if(app != null) {
				// if object listener is instanceof ApplicationListener
				if(classApplicationListener.isInstance(listener)) {					
					applicationAddApplicationListener.invoke(app, 
							classApplicationListener.cast(listener));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}