/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

package sears.tools.eawt;

import java.awt.EventQueue;
import java.io.File;

import sears.gui.MainWindow;

/**
 * This class will implement com.apple.eawt.ApplicationListener interface
 */
public class SearsApplicationListener extends ApplicationAdapter {
	// the ApplicationEvent handle
	private ApplicationEventWrapper ew = null;
	private MainWindow mainWindow;

	/**
	 * Construct a new instance
	 * @throws 	NullPointerException if frame is null
	 */
	public SearsApplicationListener() {
		ew = new ApplicationEventWrapper();
		// constructs the main window:
		mainWindow = new MainWindow();
	}

	/*
	 * (non-Javadoc)
	 * @see sears.tools.eawt.ApplicationAdapter#handleAbout(java.lang.Object)
	 */
	public void handleAbout(Object event) {
		// we disable the defaut about dialog:
		ew.applicationEventObject(event);
		ew.setHandled(true);
		// and we call the sears aboout dialog:
		mainWindow.aboutAction();
	}

	/*
	 * (non-Javadoc)
	 * @see sears.tools.eawt.ApplicationAdapter#handleOpenApplication(java.lang.Object)
	 */
	public void handleOpenApplication(Object event) {
		handleOpenFile(event);
		Runnable mainWindowVisible = new Runnable() {
			public void run() {	
				mainWindow.setVisible(true);					
			}		
		};
		// THREAD-SAFE, put the setVisible method in the SWING "event dispatching thread"
		EventQueue.invokeLater(mainWindowVisible);
		ew.setHandled(true);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.tools.eawt.ApplicationAdapter#handleOpenFile(java.lang.Object)
	 */
	public void handleOpenFile(Object event) {
		// we get back the full path of the file:
		ew.applicationEventObject(event);
		String subtitleFilePath = ew.getFilename();
		// just in case of:
		if(subtitleFilePath != null){
			mainWindow.openFile(new File(subtitleFilePath));
		}
	}

	/*
	 * (non-Javadoc)
	 * @see sears.tools.eawt.ApplicationAdapter#handlePreferences(java.lang.Object)
	 */
	public void handlePreferences(Object event) {
		// we call the sears preference dialog: options
		mainWindow.optionsAction();

	}

	/*
	 * (non-Javadoc)
	 * @see sears.tools.eawt.ApplicationAdapter#handleQuit(java.lang.Object)
	 */
	public void handleQuit(Object arg0) {
		mainWindow.quitAction();

	}
}