/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.tools.player;

import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.StringTokenizer;
import sears.tools.SearsProperties;
import sears.tools.Trace;

/**
 * Class DefaultPlayer.
 * <br><b>Summary:</b><br>
 * That is the default player, which permit to launch a player on a video file and a subtitle file.
 */
public class DefaultPlayer implements PlayerInterface {
    /**The pattern to found in the defines command line, and to be replaced by the file path.*/
    private static final String FILE_PATTERN = "%f%";
    /**The pattern to found in the defines command line, and to be replaced by the subtitle path.*/
    private static final String SUB_PATTERN = "%s%";
    /**The current process that is playing the video.*/
    private Process currentProcess;
    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#play(java.lang.String, java.lang.String)
     */
    public void play(String videoFile, String subtitleFile) throws PlayerException {
        if(currentProcess != null){
            currentProcess.destroy();
        }
        //beware of null subtitle file.
        if(subtitleFile == null){
            subtitleFile = "";
        }
        //First is to construct the command that will be executed.
        String playerCommand = SearsProperties.getProperty(SearsProperties.PLAYER_FULL_PATH);
        //To do that, we will parse the command line given, spliting in tokens.
        StringTokenizer stk = new StringTokenizer(playerCommand);
        //The array of commands.
        ArrayList<String> commands = new ArrayList<String>();
        while(stk.hasMoreTokens()){
            //get the next token in command line.
            String token = stk.nextToken();
            //If found the file pattern, replace with videoFile.
            if(token.equals(FILE_PATTERN)){
                token = videoFile;
            }else if (token.equals(SUB_PATTERN)){
                //else if found the subtitle patter, replace with the subtitle file.
                token = subtitleFile;
            }
            //Add the commands token to array.
            commands.add(token);
        }
        try {
            //Then just have to launch the process with commands array.
            currentProcess = Runtime.getRuntime().exec((String[]) commands.toArray(new String[commands.size()]));
            
            //Flust the outputs not block the process.
            final InputStreamReader input = new InputStreamReader(currentProcess.getInputStream());
            Thread inputFlusher = new Thread(){
              public void run(){
                  char c;
                  try{
                  while((c = (char) input.read()) != -1){
                      System.out.print(c);
                  }
                  }catch(IOException e){
                      Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
                  }
              }
            };
            inputFlusher.start();
            final InputStreamReader err = new InputStreamReader(currentProcess.getErrorStream());
            Thread errFlusher = new Thread(){
                public void run(){
                    char c;
                    try{
                    while((c = (char) err.read()) != -1){
                        System.err.print(c);
                    }
                    }catch(IOException e){
                        Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
                    }
                }
              };
              errFlusher.start();
        } catch (IOException e) {
            throw new PlayerException("IOException ("+e.getMessage()+") occurs when launching :"+playerCommand);
        }
    }

    /**
     * Empty method, not implemented for default player
     * @param offset			the offset
     * @throws PlayerException	if method failed
     */
    public void goToOffset(int offset) throws PlayerException {
        //Not implemented for default player
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#quit()
     */
    public void quit() {
        //simply kill the process.
        if(currentProcess != null){
            currentProcess.destroy();
        }
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#getPosition()
     */
    public int getPosition() throws PlayerException {
        // TODO Auto-generated method stub
        return 0;
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#getPosition()
     */
    public int getLength() throws PlayerException {
        // TODO Auto-generated method stub
        return 0;
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#pause()
     */
    public void pause() throws PlayerException {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#pause()
     */
    public void stop() throws PlayerException {
        // TODO Auto-generated method stub
        
    }

    /* (non-Javadoc)
     * @see sears.tools.player.PlayerInterface#setPosition(int)
     */
    public void setPosition(int offset) throws PlayerException {
        // TODO Auto-generated method stub
        
    }
}
