/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.tools.player;

/**
 * Class PlayerInterface.
 * Summary:
 * This class defines the interface to play subtitled video files.
 */
public interface PlayerInterface {
    /**
     * Method play
     * Summary:
     * Use it to launch a video, with its subtitles.
     * Or to resume from pause.
     * @param videoFile         The path to the videoFile.
     * @param subtitleFile      The path to the subtitleFile.
     * @throws PlayerException 	if an error occurs
     */
    public void play(String videoFile, String subtitleFile) throws PlayerException;
    
    /**
     * Method pause
     * Summary:
     * Pause the video.
     * Or to resume from previous pause.
     * @throws PlayerException 	if an error occurs
     */
    public void pause() throws PlayerException;
    
    /**
     * Method stop
     * Summary:
     * stop the video.
     * @throws PlayerException 	if an error occurs
     */
    public void stop() throws PlayerException;
  
    /**
     * Method setPosition
     * Summary:
     * This method permit to shift to the correct offset (in s) in the video.
     * @param offset    		The offset in the video to go.
     * @throws PlayerException 	if an error occurs
     */
    public void setPosition(int offset) throws PlayerException;
    
    /**
     * Method getPosition
     * Summary:
     * This method permit to get the current position (in seconds) in the video.
     * @return 					the current position in the video or -1 if video is currently stopped;
     * @throws PlayerException 	if an error occurs
     */
    public int getPosition() throws PlayerException;
    
    /**
     * Method getLength
     * Summary:
     * This method permit to get the video time length(in seconds).
     * @return 					the video time length or -1 if video is currently stopped;
     * @throws PlayerException 	if an error occurs
     */
    public int getLength() throws PlayerException;
    
    /**
     * Method quit
     * Summary:
     * This method allows to kill the player.
      */
    public void quit();
}
